@@ -24,7 +24,8 @@ for x in $L; do
  if diff $x tmp.txt >/dev/null; then
   true
  else
-  echo "$x: non US-ASCII"
+  echo "$x: non US-ASCII near line `cat tmp.txt | wc -l`:"
+  tail -1 tmp.txt; echo
   status=1
  fi
 done 
