@@ -30,20 +30,24 @@
 # author: Pierre.Saramito@imag.fr
 #
 arg=$1
-srcdir=$2
-string=`expr ${arg} : '\(.*\).fig' \| ${arg}`
-input=${string}.fig
-here=`pwd`
+#srcdir=$2
+srcdir=`expr ${arg} : '\(.*\)/.*' \| "."`
+filename=`expr ${arg} : '.*/\(.*\)' \| ${arg}`
+basename=`expr ${filename} : '\(.*\).fig' \| ${filename}`
+input=${srcdir}/${basename}.fig
+builddir=`pwd`
 
-name=`expr ${string} : '.*/\(.*\)' \| ${string}`
-echo name $name
+#echo "SRCDIR=$srcdir" >&2
+#echo "BUILDDIR=$builddir" >&2
+#echo "FILENAME=$filename" >&2
+#echo "BASENAME $basename"
 #exit 0
 tmp="tmp"
 
-fig2dev -L pstex                          ${input} > ${tmp}-1-${name}.ps
-fig2dev -L pstex_t -p ${tmp}-1-${name}.ps ${input} > ${tmp}-1-${name}.tex
+fig2dev -L pstex                          ${input} > ${tmp}-1-${basename}.ps
+fig2dev -L pstex_t -p ${tmp}-1-${basename}.ps ${input} > ${tmp}-1-${basename}.tex
 
-cat > ${tmp}-2-${name}.tex << EOF2
+cat > ${tmp}-2-${basename}.tex << EOF2
 \documentclass{article}
 \usepackage{epsfig}
 \usepackage{color}
@@ -52,34 +56,34 @@ cat > ${tmp}-2-${name}.tex << EOF2
 \usepackage{amsfonts}
 \usepackage{amssymb}
 \begin{document}
-\input{${tmp}-1-${name}.tex}
+\input{${tmp}-1-${basename}.tex}
 \thispagestyle{empty}
 \end{document}
 EOF2
 
-latex ${tmp}-2-${name}.tex
-dvips -E -f ${tmp}-2-${name}.dvi -o ${tmp}-2-${name}.ps
-ps2epsi ${tmp}-2-${name}.ps ${tmp}-2-${name}.eps
-epstopdf ${tmp}-2-${name}.eps
-mv ${tmp}-2-${name}.pdf ${name}.pdf
+latex ${tmp}-2-${basename}.tex
+dvips -E -f ${tmp}-2-${basename}.dvi -o ${tmp}-2-${basename}.ps
+ps2epsi ${tmp}-2-${basename}.ps ${tmp}-2-${basename}.eps
+epstopdf ${tmp}-2-${basename}.eps
+mv ${tmp}-2-${basename}.pdf ${basename}.pdf
 
-/bin/rm -f tmp-1-${name}.tex
-/bin/rm -f tmp-1-${name}.ps
-/bin/rm -f tmp-2-${name}.tex 
-/bin/rm -f tmp-2-${name}.aux
-/bin/rm -f tmp-2-${name}.log
-/bin/rm -f tmp-2-${name}.dvi
-/bin/rm -f tmp-2-${name}.ps
-/bin/rm -f tmp-2-${name}.eps
+/bin/rm -f tmp-1-${basename}.tex
+/bin/rm -f tmp-1-${basename}.ps
+/bin/rm -f tmp-2-${basename}.tex 
+/bin/rm -f tmp-2-${basename}.aux
+/bin/rm -f tmp-2-${basename}.log
+/bin/rm -f tmp-2-${basename}.dvi
+/bin/rm -f tmp-2-${basename}.ps
+/bin/rm -f tmp-2-${basename}.eps
 
-if test x"$srcdir" != x""; then
-    here=`pwd`
-    cd $srcdir
-    dest=`pwd`
-    cd $here
-    if test $here != $dest; then
-        command="mv ${name}.pdf $srcdir"
-        echo $command
-        eval $command
-    fi
-fi
+#if test x"$srcdir" != x""; then
+#    builddir=`pwd`
+#    cd $srcdir
+#    dest=`pwd`
+#    cd $builddir
+#    if test $builddir != $dest; then
+#        command="mv ${basename}.pdf $srcdir"
+#        echo $command
+#        eval $command
+#    fi
+#fi
