@@ -20,7 +20,7 @@
 #
 # -------------------------------------------------------------------------
 SRCDIR=${SRCDIR-"."}
-TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+TOP_SRCDIR=${TOP_SRCDIR-"../../../rheolef"}
 GEODIR="${TOP_SRCDIR}/nfem/ptst"
 SBINDIR="../../nfem/sbin"
 BINDIR="../../nfem/pbin"
@@ -29,9 +29,6 @@ NPROC_MAX=${NPROC_MAX-"8"}
 
 status=0
 
-#echo "      not yet (skipped)"
-#exit 0
-
 run "${SBINDIR}/mkgeo_grid_1d 500 -a -4 -b 4 -v4 2>/dev/null | ${BINDIR}/geo -upgrade - > mesh-1d.geo 2>/dev/null"
 run "${SBINDIR}/mkgeo_grid_2d  20 -a -2 -b 2 -c -2 -d 2 -v4 2>/dev/null | ${BINDIR}/geo -upgrade - > mesh-2d.geo 2>/dev/null"
 run "${SBINDIR}/mkgeo_grid_3d  10 -a -2 -b 2 -c -2 -d 2 -f -2 -g 2 -v4 2>/dev/null | ${BINDIR}/geo -upgrade - > mesh-3d.geo 2>/dev/null"
