@@ -6,8 +6,9 @@ int main(int argc, char**argv) {
   geo omega (argv[1]);
   space Xh (omega, argv[2]);
   Xh.block ("boundary");
-  form a (Xh, Xh, "grad_grad");
-  field lh = riesz (Xh, 1);
+  trial u (Xh); test v (Xh);
+  form  a  = integrate (dot(grad(u),grad(v)));
+  field lh = integrate (v);
   field uh (Xh);
   uh ["boundary"] = 0;
   solver sa (a.uu());
