@@ -11,14 +11,12 @@ int main(int argc, char**argv) {
   Float beta = (k+1)*(k+d)/d;
   trial u (Xh); test v (Xh);
   form  a = integrate (dot(grad_h(u),grad_h(v)))
-          + integrate (omega.sides(), 
-                  beta*penalty()*jump(u)*jump(v)
+          + integrate ("sides", beta*penalty()*jump(u)*jump(v)
 	 	- jump(u)*average(dot(grad_h(v),normal()))
 	 	- jump(v)*average(dot(grad_h(u),normal())));
   field lh = integrate (f(d)*v)
-           + integrate (omega.boundary(),
-                  beta*penalty()*g(d)*jump(v)
-	 	- g(d)*average(dot(grad_h(v),normal())));
+           + integrate ("boundary", beta*penalty()*g(d)*v
+	 	- g(d)*dot(grad_h(v),normal()));
   solver sa (a.uu());
   field uh(Xh);
   uh.set_u() = sa.solve(lh.u());
