@@ -10,12 +10,11 @@ int main(int argc, char**argv) {
   size_t d = omega.dimension();
   point f (0,0,0);
   f[d-1] = -1;
-  field lh = riesz (Xh, f);
-  form a1 (Xh, Xh, "div_div");
-  form a2 (Xh, Xh, "2D_D");
-  form a = lambda*a1 + a2;
-  field uh (Xh, 0);
+  trial u (Xh); test v (Xh);
+  form  a  = integrate (lambda*div(u)*div(v) + 2*ddot(D(u),D(v)));
+  field lh = integrate (dot(f,v));
   solver sa (a.uu());
+  field uh (Xh, 0);
   uh.set_u() = sa.solve (lh.u() - a.ub()*uh.b());
   dout << catchmark("inv_lambda") << 1/lambda << endl
        << catchmark("u")          << uh;
