@@ -26,6 +26,11 @@ ROUNDER="../../nfem/pbin/field -field -round"
 NPROC_MAX=${NPROC_MAX-"8"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
+if test x"${QD_EXT}" != x""; then
+  echo "      not yet (skiped when QD lib is active)"
+  exit 0
+fi
+
 status=0
 
 L="
