@@ -5,12 +5,11 @@ using namespace std;
 int main(int argc, char**argv) {
   environment rheolef(argc, argv);
   geo gamma (argv[1]);
-  space Wh (gamma, argv[2]);
   size_t d = gamma.dimension();
-  form m (Wh, Wh, "mass");
-  form a (Wh, Wh, "grad_grad");
-  a = m + a;
-  field lh = riesz(Wh, f(d));
+  space Wh (gamma, argv[2]);
+  trial u (Wh); test v (Wh);
+  form  a  = integrate (u*v + dot(grad_s(u),grad_s(v)));
+  field lh = integrate (f(d)*v);
   field uh (Wh);
   solver sa (a.uu());
   uh.set_u() = sa.solve (lh.u() - a.ub()*uh.b());
