@@ -13,11 +13,12 @@ int main(int argc, char** argv) {
   size_t d = Xh.get_geo().dimension();
   point f (0,0,0);
   f [d-1] = -1;
-  field lh = riesz (Xh, f);
-  form mp (Qh, Qh, "mass");
-  form a (Xh, Xh, "2D_D");
-  form b (Xh, Qh, "div"); b = -b;
-  form c = inv_lambda*mp;
+  trial u (Xh), p (Qh); test v (Xh), q (Qh);
+  field lh = integrate (dot(f,v));
+  form  a  = integrate (2*ddot(D(u),D(v)));
+  form  b  = integrate (-div(u)*q);
+  form  mp = integrate (p*q);
+  form  c  = inv_lambda*mp;
   field m_ru = a*uh + b.trans_mult(ph) - lh;
   field m_rp = b*uh - c*ph;
   for (size_t i_comp = 0; i_comp < d; i_comp++) {
