@@ -1,14 +1,14 @@
-struct u : std::unary_function<point,point> {
-  point operator() (const point & x) {
+struct u : field_functor<u,point> {
+  point operator() (const point & x) const {
     return (d == 1) ? point(u0) : point(x[1], -x[0]); }
   u (size_t d1) : d(d1), u0 (0.5/acos(Float(-1))) {}
   protected: size_t d; Float u0;
 };
-struct phi : std::unary_function<point,Float> {
+struct phi : field_functor<phi,Float> {
   static Float sigma(size_t d, Float nu1, Float t) {
     const Float t0 = 0.2;
     return 4*nu1/t0 - 2*d*nu1/(t0 + 4*nu1*t); }
-  Float operator() (const point& x) {
+  Float operator() (const point& x) const {
     point x0t;
     if (d == 1) { x0t = point(x0[0] + u0*t); }
     else {        x0t = point( x0[0]*cos(t) + x0[1]*sin(t),
