@@ -6,12 +6,13 @@ int main(int argc, char**argv) {
   environment rheolef (argc, argv);
   geo  omega (argv[1]);
   space Xh = cavity_space (omega, "P2");
-  field uh = cavity_field (Xh, 1);
   space Qh (omega, "P1");
+  trial u (Xh), p (Qh); test v (Xh), q (Qh);
+  form a  = integrate (2*ddot(D(u),D(v)));
+  form b  = integrate (-div(u)*q);
+  form mp = integrate (p*q);
+  field uh = cavity_field (Xh, 1);
   field ph (Qh, 0.);
-  form a  (Xh, Xh, "2D_D");
-  form b = - form(Xh, Qh, "div");
-  form mp (Qh, Qh, "mass");
   solver_abtb stokes (a.uu(), b.uu(), mp.uu());
   stokes.solve (-(a.ub()*uh.b()), -(b.ub()*uh.b()),
                 uh.set_u(),       ph.set_u());
