@@ -22,8 +22,8 @@
 SRCDIR=${SRCDIR-"."}
 TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 GEODIR="${TOP_SRCDIR}/nfem/ptst"
-BINDIR="${TOP_SRCDIR}/nfem/pbin"
-SBINDIR="${TOP_SRCDIR}/nfem/sbin"
+SBINDIR="../../nfem/sbin"
+BINDIR="../../nfem/pbin"
 ROUNDER="../../nfem/pbin/field - -field -round"
 NPROC_MAX=${NPROC_MAX-"6"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
