@@ -1,5 +1,4 @@
 #include "rheolef.h"
-#include "rheolef/catchmark.h"
 using namespace rheolef;
 using namespace std;
 size_t d;
@@ -21,7 +20,8 @@ int main(int argc, char**argv) {
   space Xh = uh.get_space();
   field pi_h_u = interpolate(Xh, u);
   field eh = pi_h_u - uh;
-  form m (Xh, Xh, "mass");
+  trial u (Xh); test v (Xh);
+  form m = integrate (u*v);
   derr << "error_linf "  << eh.max_abs() << endl
        << "error_l2   "  << sqrt(m(eh,eh)) << endl;
   return (eh.max_abs() < error_linf_expected) ? 0 : 1;
