@@ -25,7 +25,8 @@ int main(int argc, char**argv) {
   derr << "err_l2   = " << err_l2   << endl
        << "err_linf = " << err_linf << endl;
   if (dump) {
-    dout << catchmark("phi")      << phi_h
+    dout << catchmark("phi_h")    << phi_h
+         << catchmark("phi")      << interpolate(phi_h.get_space(), phi(sigma))
          << catchmark("e")        << eh;
   }
   return (err_l2 < tol) ? 0 : 1;
