@@ -4,7 +4,8 @@
 #
 # authors: Ibrahim Cheddadi and Pierre Saramito
 #
-# last update: 25 fev 2013
+# changelogs:
+# * 25 sept 2014, Pierre Saramito : handle file path
 #
 if test $# -eq 0; then
   echo "usage: $0 [-eps] file[.plot]" 1>&2
@@ -15,13 +16,17 @@ if test x"$1" = x"-eps"; then
   shift
   output_format_eps=true
 fi
-name=`expr $1 : '\(.*\).plot' \| $1`
-if test ! -f $name.plot; then
-  echo "$0: file \"$name.plot\" not found" 1>&2
+input=$1
+pathname=`expr $input : '\(.*\).plot' \| $input`
+input=$pathname.plot
+toclean=""
+if test ! -f $input; then
+  echo "$0: file \"$input\" not found" 1>&2
   exit 1
 fi
+name=`expr $pathname : '.*/\(.*\)' \| $pathname`
 liste=""
-for mot in $(cat $name.plot); do
+for mot in $(cat $input); do
   if [ "${mot:(-5)}" = ".tex\"" ]; then
 	mot=${mot#\"}
 	mot=${mot%\"}
@@ -40,16 +45,17 @@ done
 # -----------
 # run gnuplot
 # -----------
-command="gnuplot $name.plot"
+command="gnuplot $input"
 echo $command
 eval $command 2> $name.glog
+toclean="$toclean $name.glog"
 status=$?
 cat $name.glog
 if test $status -ne 0; then
-  echo "$0: command failed: gnuplot $name.plot" 1>&2
+  echo "$0: command failed: gnuplot $input" 1>&2
   for i in $liste; do
     n=${i%.tex}
-    rm -f $n.dvi $n.aux $n.log $n.tex $n-inc.eps $n.glog
+    rm -f $n.dvi $n.aux $n.log $n.tex $n-inc.eps $n.glog $toclean
   done
   exit 1
 fi
@@ -61,7 +67,7 @@ if test `grep -i warning $name.glog | \
   echo "$0: gnuplot warning changed to error"
   for i in $liste; do
     n=${i%.tex}
-    rm -f $n.dvi $n.aux $n.log $n.tex $n-inc.eps $n.glog
+    rm -f $n.dvi $n.aux $n.log $n.tex $n-inc.eps $n.glog $toclean
   done
   exit 1
 fi
@@ -69,12 +75,11 @@ fi
 # -------------------------------------
 # is generated latex file standalone ?
 # -------------------------------------
-toclean=""
-if `grep '^set[ \t][ \t]*terminal[ \t].*pdf' $name.plot 2>/dev/null >/dev/null`; then
+if `grep '^set[ \t][ \t]*terminal[ \t].*pdf' $input 2>/dev/null >/dev/null`; then
  run_latex=false
 else
  run_latex=true
- if `grep '^set[ \t][ \t]*terminal[ \t].*standalone' $name.plot 2>/dev/null >/dev/null`; then
+ if `grep '^set[ \t][ \t]*terminal[ \t].*standalone' $input 2>/dev/null >/dev/null`; then
   standalone=true
  else
   standalone=false
@@ -169,7 +174,7 @@ if $run_latex; then
   nerr=`grep -i "Latex Error" $n.log | wc -l`
   if test $nerr -ne 0 || test ! -f $n.dvi; then
     cat $n.glog	
-    echo "! $0: latex ERROR with $name.plot"
+    echo "! $0: latex ERROR with $input"
     rm -f $n.dvi $n.aux $n.log $n.tex $n-inc.eps $n.glog
     exit 1
   fi
@@ -178,7 +183,7 @@ if $run_latex; then
   eval $command
   status=$?
   if test $status -ne 0; then
-    echo "! $0: dvips ERROR with $name.plot"
+    echo "! $0: dvips ERROR with $input"
     exit 1
   fi
   command="ps2epsi $n.ps $n.eps"
@@ -188,7 +193,7 @@ if $run_latex; then
   rm -f $n.dvi $n.aux $n.log $n.tex $n-inc.eps $n.glog $n.ps
   if test $status -ne 0; then
     rm -f $n.eps
-    echo "! $0: ps2epsi ERROR with $name.plot"
+    echo "! $0: ps2epsi ERROR with $input"
     exit 1
   fi
   if $output_format_eps; then
@@ -203,7 +208,7 @@ if $run_latex; then
   rm -f $n.eps
   if test $status -ne 0; then
     rm -f $n.pdf
-    echo "! $0: epstopdf ERROR with $name.plot"
+    echo "! $0: epstopdf ERROR with $input"
     exit 1
   fi
  done
