@@ -64,7 +64,7 @@ numbering_H3::idof (
         // and then all dof based on interior nodes (e.g. quadrangles)
 
 	// for mixed triangle-quadrangle mesh : assume triangle are numbered first
-        switch (K.type()) {
+        switch (K.variant()) {
 	  case reference_element::p: 
             return (1+i_dof_local)*K.index();
 	  case reference_element::e: 
@@ -100,8 +100,10 @@ numbering_H3::ndof (
 	return 2*mesh_n_geo[0];
       case 1:
 	return 2*mesh_n_geo[0];
+      case 2:
+	return 2*mesh_n_geo[0];
       default:
-	fatal_macro ("unsupported H3 approximation in `" << mesh_map_dimension << "'");
+	fatal_macro ("unsupported H3 approximation in `" << mesh_map_dimension << "D'");
 	return 0;
     }
 }
