@@ -27,7 +27,7 @@ echo "      $command"
 eval $command 2>/dev/null | diff $SRCDIR/P2.out.valid -
 if test $? -ne 0; then status=1; echo "        => *NO*"; fi
 
-command="./numbering_tst P2_numbering"
+command="./numbering_tst P2"
 echo "      $command"
 eval $command 2>/dev/null | diff $SRCDIR/P2_numbering.out.valid -
 if test $? -ne 0; then status=1; echo "        => *NO*"; fi
