@@ -51,10 +51,15 @@ put_gpl (ostream& out)
 	;
 }
 ex
-basis_symbolic_nodal_on_geo::indexed_symbol (ex expr0) const
+basis_symbolic_nodal_on_geo::indexed_symbol (const ex& expr0) const
 {
-    ex expr = expr0;
     size_type d = dimension();
+    ex expr = expr0;
+    // first optimize;
+    if (d > 0) expr = collect(expr,x);
+    if (d > 1) expr = collect(expr,y);
+    if (d > 2) expr = collect(expr,z);
+    // then subst symbols:
     if (d > 0) expr = expr.subs(x == symbol("hat_x[0]"));
     if (d > 1) expr = expr.subs(y == symbol("hat_x[1]"));
     if (d > 2) expr = expr.subs(z == symbol("hat_x[2]"));
@@ -77,7 +82,7 @@ basis_symbolic_nodal_on_geo::put_cxx_header(ostream& out) const
       << "  static point grad_eval(" << endl
       << "    size_type    i_dof_local," << endl
       << "    const point& hat_x);" << endl
-      << "  static basic_point<point> hessian_eval(" << endl
+      << "  static tensor hessian_eval(" << endl
       << "    size_type    i_dof_local," << endl
       << "    const point& hat_x);" << endl
       << "  static void eval(" << endl
@@ -88,7 +93,7 @@ basis_symbolic_nodal_on_geo::put_cxx_header(ostream& out) const
       << "    vector<point>& values);" << endl
       << "  static void hessian_eval(" << endl
       << "    const point&   hat_x," << endl
-      << "    vector<basic_point<point> >& values);" << endl
+      << "    vector<tensor >& values);" << endl
       << "  static void hat_node(" << endl
       << "    vector<point>& hat_node);" << endl
       << "};" << endl;
@@ -107,7 +112,7 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
       << class_name.str() << "::dof_family(" << endl
       << "  size_type    i_dof_local)" << endl
       << "{" << endl
-      << "  return reference_element::Lagrange;" << endl
+      << "  return element_constant::Lagrange;" << endl
       << "}" << endl;
   // --------------------------------------------------
   // eval
@@ -122,7 +127,7 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
       << "  switch (i_dof_local) {" << endl;
   for (size_type i = 0; i < _basis.size(); i++) {
     out << "    case " << i << ": {" << endl;
-    ex expr = indexed_symbol(_basis[i]);
+    ex expr = indexed_symbol (_basis[i]);
     out << "      val = ";
     expr.print(print_csrc_double(cout));
     out << ";" << endl;
@@ -149,7 +154,7 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
   for (size_type i = 0; i < _basis.size(); i++) {
     out << "    case " << i << ": {" << endl;
     for (size_type j = 0; j < d; j++) {
-	ex expr = indexed_symbol(_grad_basis[i][j]);
+	ex expr = indexed_symbol (_grad_basis[i][j]);
         out << "      val[" << j << "] = ";
         expr.print(print_csrc_double(cout));
         out << ";" << endl;
@@ -166,14 +171,13 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
   // --------------------------------------------------
   // hessian_eval
   // --------------------------------------------------
-  out << "basic_point<point>" << endl
+  out << "tensor" << endl
       << class_name.str() << "::hessian_eval(" << endl
       << "  size_type    i_dof_local," << endl
       << "  const point& hat_x)" << endl
       << "{" << endl
       << "  typedef Float T;" << endl
-      << "  point ini_p(0);" << endl
-      << "  basic_point<point> val(ini_p,ini_p,ini_p);" << endl
+      << "  tensor val(0.);" << endl
       << "  error_macro (\"hessian_eval: invalid basis " << name() << "\");" << endl
       << "  return val;" << endl
       << "}" << endl;
@@ -188,7 +192,7 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
       << "  values.resize(" << _basis.size() << ");" << endl
       << "  typedef Float T;" << endl;
   for (size_type i = 0; i < _basis.size(); i++) {
-    ex expr = indexed_symbol(_basis[i]);
+    ex expr = indexed_symbol (_basis[i]);
     stringstream idx_stream;
     out << "  values[" << i << "] = ";
     expr.print(print_csrc_double(cout));
@@ -207,7 +211,7 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
       << "  typedef Float T;" << endl;
   for (size_type i = 0; i < _basis.size(); i++) {
     for (size_type j = 0; j < d; j++) {
-      ex expr = indexed_symbol(_grad_basis[i][j]);
+      ex expr = indexed_symbol (_grad_basis[i][j]);
       out << "  values[" << i << "][" << j << "] = ";
       expr.print(print_csrc_double(cout));
       out << ";" << endl;
@@ -220,10 +224,9 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
   out << "void" << endl
       << class_name.str() << "::hessian_eval(" << endl
       << "  const point&   hat_x," << endl
-      << "  vector<basic_point<point> >& values)" << endl
+      << "  vector<tensor >& values)" << endl
       << "{" << endl
-      << "  point ini_p(0);" << endl
-      << "  basic_point<point> ini_val(ini_p,ini_p,ini_p);" << endl
+      << "  tensor ini_val(0.);" << endl
       << "  values.resize(" << _basis.size() << ",ini_val);" << endl
       << "  typedef Float T;" << endl
       << "  error_macro (\"hessian_eval: invalid basis " << name() << "\");" << endl
@@ -239,7 +242,7 @@ basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
   for (size_type i = 0; i < _basis.size(); i++) {
     out << "  x[" << i << "] = point(";
     for (size_type j = 0; j < d; j++) {
-      ex expr = indexed_symbol(_node[i][j]);
+      ex expr = indexed_symbol (_node[i][j]);
       expr.print(print_csrc_double(cout));
       if (j != d-1) out << ", ";
     }
@@ -258,12 +261,13 @@ basis_symbolic_nodal::put_cxx_header(ostream& out) const
     out << "#ifndef _RHEOLEF_" << name() << "_H" << endl
         << "#define _RHEOLEF_" << name() << "_H" << endl
         << "#include \"rheolef/basis.h\"" << endl
+        << "#include \"rheolef/tensor.h\"" << endl
         << "namespace rheolef {" << endl
         << endl
         << "class " << class_name.str() << ": public basis_rep {" << endl
         << "public:" << endl
         << "  typedef size_t size_type;" << endl
-        << "  " << class_name.str() << "() : basis_rep(reference_element::Lagrange) {}" << endl
+        << "  " << class_name.str() << "() : basis_rep(\"" << name() << "\", element_constant::Lagrange) {}" << endl
         << "  ~" << class_name.str() << "();" << endl
         << "  size_type degree () const;" << endl
         << "  size_type size (" << endl
@@ -279,7 +283,7 @@ basis_symbolic_nodal::put_cxx_header(ostream& out) const
         << "    reference_element hat_K," << endl
         << "    size_type         i_dof_local," << endl
         << "    const point&      hat_x) const;" << endl
-        << "  basic_point<point> hessian_eval(" << endl
+        << "  tensor hessian_eval(" << endl
         << "    reference_element hat_K," << endl
         << "    size_type         i_dof_local," << endl
         << "    const point&      hat_x) const;" << endl
@@ -294,7 +298,7 @@ basis_symbolic_nodal::put_cxx_header(ostream& out) const
         << "  void hessian_eval(" << endl
         << "    reference_element hat_K," << endl
         << "    const point&      hat_x," << endl
-        << "    std::vector<basic_point<point> >&    values) const;" << endl
+        << "    std::vector<tensor >&    values) const;" << endl
         << "  void hat_node(" << endl
         << "    reference_element hat_K," << endl
         << "    std::vector<point>&    hat_node) const;" << endl
@@ -311,10 +315,10 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
     out << "#include \"" << name() << ".h\"" << endl;
     out << "using namespace rheolef;" << endl;
     out << "using namespace std;" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       operator[](i).put_cxx_header(out);
     }
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       operator[](i).put_cxx_body(out);
     }
     stringstream class_name;
@@ -341,9 +345,9 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
 	<< class_name.str() << "::size (" << endl
 	<< "    reference_element hat_K, basis_rep::dof_family_type family) const" << endl
         << "{" << endl
-        << "    if (family!=reference_element::Lagrange && family!=reference_element::dof_family_max) return 0;" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+	<< "    if (family != element_constant::Lagrange && family != element_constant::dof_family_max) return 0;" << endl
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
@@ -364,8 +368,8 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << "    reference_element hat_K," << endl
         << "    size_type         i_dof_local) const" << endl
         << "{" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
@@ -374,7 +378,7 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
     }
     out << "      default : {" << endl
         << "        error_macro (\"dof_family: unsupported `\" << hat_K.name() << \"' element type\");" << endl
-        << "        return reference_element::dof_family_max;" << endl
+        << "        return element_constant::dof_family_max;" << endl
         << "      }" << endl
         << "    }" << endl
         << "}" << endl;
@@ -387,8 +391,8 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << "    size_type         i_dof_local," << endl
         << "    const point&      hat_x) const" << endl
         << "{" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
@@ -410,8 +414,8 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << "    size_type         i_dof_local," << endl
         << "    const point&      hat_x) const" << endl
         << "{" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
@@ -427,15 +431,14 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
      // --------------------------------------------------
     // hessian_eval
     // --------------------------------------------------
-    out << "basic_point<point>" << endl
+    out << "tensor" << endl
         << class_name.str() << "::hessian_eval(" << endl
         << "    reference_element hat_K," << endl
         << "    size_type         i_dof_local," << endl
         << "    const point&      hat_x) const" << endl
         << "{" << endl
         << "  error_macro (\"hessian_eval: invalid basis " << name() << "\");" << endl
-        << "  point ini_p(0);" << endl
-        << "  basic_point<point> ini_val(ini_p,ini_p,ini_p);" << endl
+        << "  tensor ini_val(0.);" << endl
         << "  return ini_val;" << endl
         << "}" << endl;
     // --------------------------------------------------
@@ -447,8 +450,8 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << "    const point&      hat_x," << endl
         << "    vector<Float>&    values) const" << endl
         << "{" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
@@ -469,8 +472,8 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << "    const point&      hat_x," << endl
         << "    vector<point>&    values) const" << endl
         << "{" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
@@ -489,7 +492,7 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << class_name.str() << "::hessian_eval(" << endl
         << "    reference_element hat_K," << endl
         << "    const point&      hat_x," << endl
-        << "    vector<basic_point<point> >&    values) const" << endl
+        << "    vector<tensor >&    values) const" << endl
         << "{" << endl
         << "  error_macro (\"hessian_eval: invalid basis " << name() << "\");" << endl
         << "}" << endl;
@@ -501,8 +504,8 @@ basis_symbolic_nodal::put_cxx_body(ostream& out) const
         << "    reference_element hat_K," << endl
         << "    vector<point>&    hat_node) const" << endl
         << "{" << endl
-        << "    switch (hat_K.type()) {" << endl;
-    for (size_type i = 0; i < reference_element::max_size; i++) {
+        << "    switch (hat_K.variant()) {" << endl;
+    for (size_type i = 0; i < reference_element::max_variant; i++) {
       const basis_symbolic_nodal_on_geo& b = operator[](i);
       if (b.size() == 0) continue;
       out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
