@@ -32,7 +32,7 @@ main (int argc, const char **argv)
   {
     basis b(argv[1]);
 
-    reference_element::enum_type typ = reference_element::p;
+    reference_element::variant_type typ = reference_element::p;
     reference_element hat_K (typ);
     size_t            i_dof_local = 0;
     point             hat_x (0);
