@@ -20,34 +20,13 @@
 /// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ///
 /// =========================================================================
-#include "rheolef/numbering.h"
 
-namespace rheolef { 
-class numbering_bubble : public numbering_rep {
-public:
+#include "rheolef/config.h"
 
-// typedefs:
+#ifdef _RHEOLEF_USE_NEW_CODE
+#include "bubble_numbering_v2.h"
+#else
+#include "bubble_numbering_v1.h"
+#endif
 
-    typedef numbering_rep::size_type size_type;
-
-// global method:
-
-    std::string name() const;
-    size_type ndof (
-              size_type  mesh_map_dimension,
-        const size_type* mesh_n_geo,
-        const size_type* mesh_n_element) const;
-    size_type idof (
-	const size_type*      mesh_n_geo,
-	const size_type*      mesh_n_element,
-	const geo_element&    K, 
-	size_type             i_dof_local) const;
-    void idof (
-	const size_type*      mesh_n_geo,
-	const size_type*      mesh_n_element,
-	const geo_element&    K, 
-	std::vector<size_type>&    i_dof) const;
-    bool is_continuous() const;
-};
-}// namespace rheolef
 #endif // _RHEOLEF_BUBBLE_NUMBERING_H
