@@ -391,6 +391,7 @@ int main(int argc, char**argv)
     bool have_input = false;
     bool def_cut_opt = false;
     bool def_fill_opt = false;
+    bool use_tmp = false;
     show_type show = show_none;
     render_type render = no_render;
     //
@@ -412,6 +413,8 @@ int main(int argc, char**argv)
 	else if (strcmp ("-mmg3d",   argv [i]) == 0) { cout << mmg3d; render = file_render; }
 	else if (strcmp ("-gmsh",    argv [i]) == 0) { cout << gmsh; render = file_render; }
 
+	else if (strcmp ("-tmp",     argv [i]) == 0) { use_tmp=true; }
+
 	else if (strcmp ("-input-field",   argv [i]) == 0) input_format = input_field;
 	else if (strcmp ("-input-bamg",    argv [i]) == 0) input_format = input_bamg;
 	else if (strcmp ("-input-mmg3d",   argv [i]) == 0) input_format = input_mmg3d;
@@ -544,6 +547,7 @@ int main(int argc, char**argv)
 	cerr << "field: no input specifield." << endl;
 	usage();
     }
+    if (use_tmp) { name="/tmp/" + name; cerr << "tmp "<< name; }
     if (render == no_render) {
         if (u.dimension() == 1) {
             cout << gnuplot;
