@@ -41,6 +41,8 @@
 #include "rheolef/geo_element_indirect.h"
 #include "rheolef/heap_allocator.h"
 #include "rheolef/array.h"
+#include "rheolef/reference_element_face_transformation.h"
+
 #include <boost/serialization/serialization.hpp>
 #include <boost/serialization/base_object.hpp>
 
@@ -104,13 +106,23 @@ public:
 // typedefs:
 
   enum {
-        _variant_offset    = 0,
-        _order_offset      = 1,
-	_dis_ie_offset     = 2,
-        _ios_dis_ie_offset = 3,
-        _last_offset       = 4
+        _variant_offset    = 0, // i.e. type, as triangle(t) or tetra(T), etc
+        _order_offset      = 1, // i.e. k, when Pk curved element
+	_dis_ie_offset     = 2, // internal numbering, depend upon partitionand nproc
+        _ios_dis_ie_offset = 3, // i/o numbering, independent of parition and nproc
+        _master_offset     = 4, // (d-1)-side has one or two master d-element that contains it
+        _last_offset       = 6  // here starts node indexes, face indexes, etc
   };
-
+  // Implementation note: _master_offset  reserve 2 size_type but is used only for sides,
+  // i.e. tri or quad in 3d mesh, edge in 2d mesh, or point in 1d
+  // => waste a lot of place
+  // it would be better with a polymorphic class
+  // and the geo class would define an array of smart_pointers on this class
+  // so, we could define edge with or without the 2 size_type for master elements
+  // then, hack_array will become obsolete (good thing)
+  // and reference_element could be also polymorphic, avoiding large swich (variant)
+  // in all internal loops. This change of implementation will be considered
+  // in the future.
   typedef reference_element::size_type    size_type;
   typedef reference_element::variant_type variant_type;
   typedef size_type*                         iterator;
@@ -147,18 +159,23 @@ public:
 
 // accessors & modifiers:
 
-  variant_type variant() const { return variant_type( *(_data_begin() +  _variant_offset)); }
-  size_type order()      const { return *(_data_begin() +  _order_offset); }
-  size_type dis_ie()     const { return *(_data_begin() +  _dis_ie_offset); }
-  size_type ios_dis_ie() const { return *(_data_begin() +  _ios_dis_ie_offset); }
+  variant_type variant()    const { return variant_type( *(_data_begin() +  _variant_offset)); }
+  size_type order()         const { return *(_data_begin() + _order_offset); }
+  size_type dis_ie()        const { return *(_data_begin() + _dis_ie_offset); }
+  size_type ios_dis_ie()    const { return *(_data_begin() + _ios_dis_ie_offset); }
+  size_type master (bool i) const { return *(_data_begin() + _master_offset + i); }
 
   size_type dimension()  const { return reference_element::dimension (variant()); }
   size_type size()       const { return reference_element::n_vertex (variant()); }
   char      name()       const { return reference_element::name     (variant()); }
   size_type n_node()     const { return reference_element::n_node (variant(), order()); }
 
-  void set_dis_ie    (size_type dis_ie)     { *(_data_begin() +  _dis_ie_offset)     = dis_ie; }
-  void set_ios_dis_ie(size_type ios_dis_ie) { *(_data_begin() +  _ios_dis_ie_offset) = ios_dis_ie; }
+  void set_dis_ie         (size_type dis_ie) { *(_data_begin() + _dis_ie_offset)     = dis_ie; }
+  void set_ios_dis_ie (size_type ios_dis_ie) { *(_data_begin() + _ios_dis_ie_offset) = ios_dis_ie; }
+  void set_master (bool i, size_type dis_ie) const {
+    const_iterator p = _data_begin() + _master_offset + i; // mutable member fct
+    *(const_cast<iterator>(p)) = dis_ie;
+  }
 
   iterator       begin()       { return _data_begin() +  _node_offset (variant(), order()); }
   const_iterator begin() const { return _data_begin() +  _node_offset (variant(), order()); }
@@ -213,6 +230,19 @@ public:
 
 // orientation accessors:
 
+    // seach S in all sides of K
+    orientation_type get_side_informations (
+  		const geo_element& S,
+  		size_type& loc_isid,
+  		size_type& shift) const;
+
+    void get_side_informations (
+  		const geo_element& S,
+		side_information_type& sid) const;
+
+    orientation_type get_side_orientation (const geo_element& S) const;
+
+    // compare two sides: S and *this
     bool get_orientation_and_shift (const geo_element& S, 
 	  orientation_type& orient, shift_type& shift) const;
     orientation_type get_edge_orientation (size_type dis_iv0, size_type dis_iv1) const;
@@ -314,7 +344,7 @@ public:
 
 // typedefs:
 
-  typedef A		                     allocator_type;
+  typedef A		                  allocator_type;
   typedef reference_element::size_type    size_type;
   typedef reference_element::variant_type variant_type;
   typedef geo_element::iterator           iterator;
