@@ -25,6 +25,11 @@ DATA_DIR="${TOP_SRCDIR}/nfem/data"
 NPROC_MAX=${NPROC_MAX-"7"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
+if test x"${QD_EXT}" != x""; then
+  echo "      not yet (skiped when QD lib)"
+  exit 0
+fi
+
 status=0
 
 run "./tensor_exp_tst >/dev/null 2>/dev/null"
