@@ -108,8 +108,11 @@ field::gnuplot2d_velocity (const string& basename, plot_options& opt) const
   if (opt.meshplot) plot << "load \"" << basename << ".mesh.plot\"" << endl << "replot "; 
   else 
    {
-     plot << "set xrange ["<<opt.xmin[0]<<":"<<opt.xmax[0]<<"]" << endl;
-     plot << "set yrange ["<<opt.xmin[1]<<":"<<opt.xmax[1]<<"]" << endl;
+     if (!opt.keep_user_range) 
+      {
+     	plot << "set xrange ["<<opt.xmin[0]<<":"<<opt.xmax[0]<<"]" << endl;
+     	plot << "set yrange ["<<opt.xmin[1]<<":"<<opt.xmax[1]<<"]" << endl;
+      }
      plot << "plot ";
    }
   plot << "\"" << basename 
@@ -153,7 +156,7 @@ field::gnuplot2d_velocity (const string& basename, plot_options& opt) const
   //
   plot << endl; // because of the trailing "\"
   plot << endl; 
-  plot << "pause -1 \"<return>\"\n";
+  if (opt.pause) plot << "pause -1 \"<return>\"\n";
   if (opt.postscript && (!opt.noxplot)) 
    {
 	plot << "set output '" << basename << ".eps'" << endl;
