@@ -35,7 +35,8 @@ form_element_rep::form_element_rep ()
    _is_weighted(false),
    _bw_table(),
    _use_coordinate_system_weight(true),
-   _use_coordinate_system_dual_weight(false)
+   _use_coordinate_system_dual_weight(false),
+   _skip_geo_compatibility_check(false)
 {
 }
 form_element_rep::~form_element_rep()
@@ -50,7 +51,7 @@ form_element_rep::initialize (
   _Y = Y;
   _X.freeze();
   _Y.freeze();
-  check_macro (_X.get_global_geo() == _Y.get_global_geo(),
+  check_macro (_skip_geo_compatibility_check || _X.get_global_geo() == _Y.get_global_geo(),
 	"form assembly: global spaces based on different meshes: unsupported");
 
   size_type quad_order = get_first_basis().degree() + get_second_basis().degree();
@@ -62,7 +63,7 @@ form_element_rep::initialize (
   _by_table.set (_quad, get_second_basis());
   _tr_p1_table.set (_quad, get_p1_transformation());
   if (_is_weighted) {
-    check_macro (get_weight().get_space().get_global_geo() == get_global_geo(),
+    check_macro (_skip_geo_compatibility_check || get_weight().get_space().get_global_geo() == get_global_geo(),
 	"weighted form assembly: weight space based on a different mesh: not supported");
     _bw_table.set (_quad, get_weight_basis());
   }
@@ -267,7 +268,49 @@ form_element_rep::cumul_otimes (
       basis_on_quadrature::const_iterator_hessian hessian_psi = first_hessian_psi;
       Float phi_w = (*phi)*w;
       for (size_type j = 0; hessian_psi != last_hessian_psi; hessian_psi++, j++)
-        m(i,j) += (*hessian_psi)[0][0]*phi_w;
+        m(i,j) += (*hessian_psi)(0,0)*phi_w;
+    }
+}
+// ----------------------------------------------------------
+// add to elementary matrix : (d phi/ds otimes d^2 psi/ds^2) * w
+// ----------------------------------------------------------
+void
+form_element_rep::cumul_otimes (
+    ublas::matrix<Float>& m, 
+    const Float& w,
+    basis_on_quadrature::const_iterator_grad grad_phi,
+    basis_on_quadrature::const_iterator_grad last_grad_phi,
+    basis_on_quadrature::const_iterator_hessian first_hessian_psi,
+    basis_on_quadrature::const_iterator_hessian last_hessian_psi)
+{
+    // Only the 0,0 component of hessian of psi is relevant (curvilinear derivative
+    // along an edge), and the 0 component of grad phi (d phi/ds)
+    for (size_type i = 0; grad_phi != last_grad_phi; grad_phi++, i++) {
+      basis_on_quadrature::const_iterator_hessian hessian_psi = first_hessian_psi;
+      Float phi_w = (*grad_phi)[0]*w;
+      for (size_type j = 0; hessian_psi != last_hessian_psi; hessian_psi++, j++)
+        m(i,j) += (*hessian_psi)(0,0)*phi_w;
+    }
+}
+// ----------------------------------------------------------
+// add to elementary matrix : (d^2 phi/ds^2 otimes d^2 psi/ds^2) * w
+// ----------------------------------------------------------
+void
+form_element_rep::cumul_otimes (
+    ublas::matrix<Float>& m, 
+    const Float& w,
+    basis_on_quadrature::const_iterator_hessian hessian_phi,
+    basis_on_quadrature::const_iterator_hessian last_hessian_phi,
+    basis_on_quadrature::const_iterator_hessian first_hessian_psi,
+    basis_on_quadrature::const_iterator_hessian last_hessian_psi)
+{
+    // Only the 0,0 component of hessian of psi is relevant (curvilinear derivative
+    // along an edge), and the 0 component of grad phi (d phi/ds)
+    for (size_type i = 0; hessian_phi != last_hessian_phi; hessian_phi++, i++) {
+      basis_on_quadrature::const_iterator_hessian hessian_psi = first_hessian_psi;
+      Float phi_w = (*hessian_phi)(0,0)*w;
+      for (size_type j = 0; hessian_psi != last_hessian_psi; hessian_psi++, j++)
+        m(i,j) += (*hessian_psi)(0,0)*phi_w;
     }
 }
 // ----------------------------------------------------------
@@ -385,12 +428,12 @@ InputIterator
 mypiola_basis (const reference_element& K, const basis& b, const tensor& DF, 
 	InputIterator first_in, InputIterator last_in, OutputIterator first_out)
 {
-  if (b.family()==reference_element::Lagrange) return first_in;
-  else if (b.family()==reference_element::Hermite) 
+  if      (b.family() == element_constant::Lagrange) return first_in;
+  else if (b.family() == element_constant::Hermite) 
    {
    	for (piola::size_type i=0; first_in!=last_in; first_in++, i++)
 	 {
-	 	if (b.dof_family(K, i)==reference_element::Lagrange)
+	 	if (b.dof_family(K, i) == element_constant::Lagrange)
 			first_out[i]=*first_in;
 		else // Hermite
 			{
@@ -409,8 +452,8 @@ form_element_rep::build_scalar_mass (const geo_element& K, ublas::matrix<Float>&
   reference_element hat_K = K;
   m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
   vector<Float> F_bx_table (get_first_basis().size(hat_K));
   vector<Float> F_by_table (get_second_basis().size(hat_K));
   tensor DF;
@@ -440,8 +483,8 @@ form_element_rep::build_general_mass (const geo_element& K, ublas::matrix<Float>
     build_scalar_mass (K, m);
     return;
   }
-  check_macro (get_first_basis().family()==reference_element::Lagrange &&
-  	get_second_basis().family()==reference_element::Lagrange, 
+  check_macro (get_first_basis().family() == element_constant::Lagrange &&
+  	      get_second_basis().family() == element_constant::Lagrange, 
 	"Only Lagrange elements supported");
   // space is vectorial or tensorial: weight could be scalar, tensor or tensor4
   reference_element hat_K = K;
@@ -543,8 +586,8 @@ form_element_rep::build_general_mass (const geo_element& K, ublas::matrix<Float>
 void
 form_element_rep::d_dx (const geo_element& K, ublas::matrix<Float>& m, size_type idx) const
 {
-  check_macro (get_first_basis().family()==reference_element::Lagrange &&
-  	get_second_basis().family()==reference_element::Lagrange, 
+  check_macro (get_first_basis().family() == element_constant::Lagrange &&
+  	      get_second_basis().family() == element_constant::Lagrange, 
 	"Only Lagrange elements supported");
   size_type map_d = K.dimension();
   check_macro (idx < map_d, "unexpected `d_dx" << idx << "' on element type "
@@ -555,8 +598,8 @@ form_element_rep::d_dx (const geo_element& K, ublas::matrix<Float>& m, size_type
   reference_element hat_K = K;
   m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
   tensor DF;
   tensor invDF;
   for (size_type q = 0; first != last; first++, q++) {
@@ -598,8 +641,8 @@ form_element_rep::build_d_ds (const geo_element& K, ublas::matrix<Float>& m) con
   reference_element hat_K = K;
   m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
   vector<point> F_bx_table_grad (get_first_basis().size(hat_K));
   vector<Float> F_by_table (get_second_basis().size(hat_K));
   tensor DF;
@@ -631,10 +674,9 @@ form_element_rep::build_d2_ds2 (const geo_element& K, ublas::matrix<Float>& m) c
   reference_element hat_K = K;
   m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
-  vector<basic_point<point> > F_bx_table (get_first_basis().size(hat_K), 
-  	basic_point<point>(point(0),point(0),point(0)));
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
+  vector<tensor> F_bx_table (get_first_basis().size(hat_K)); 
   vector<Float> F_by_table (get_second_basis().size(hat_K));
   tensor DF;
   tensor invDF;
@@ -647,7 +689,7 @@ form_element_rep::build_d2_ds2 (const geo_element& K, ublas::matrix<Float>& m) c
     // Jacobian of Piola transform appears inverted in
     //    int_e d^2 phi/ds^2 psi ds 
     wq /= det_jacobian_piola_transformation (DF, K.dimension());
-    vector<basic_point<point> >::const_iterator Fx_begin_hessian 
+    vector<tensor >::const_iterator Fx_begin_hessian 
     	= piola_basis (get_first_space(), K, get_first_basis(), DF,
     	_bx_table.begin_hessian (hat_K, q), _bx_table.end_hessian (hat_K, q), F_bx_table.begin());
     vector<Float>::const_iterator Fy_begin = piola_basis (get_second_space(), K, get_second_basis(), DF,
@@ -658,22 +700,94 @@ form_element_rep::build_d2_ds2 (const geo_element& K, ublas::matrix<Float>& m) c
   }
 }
 // --------------------------------------------------------------------
-//   (d_ds phi, d_ds psi) along an edge e
+//   (d^2_ds^2 phi, d_ds psi) along an edge e
 // --------------------------------------------------------------------
 void 
-form_element_rep::build_d_ds_d_ds (const geo_element& K, ublas::matrix<Float>& m) const
+form_element_rep::build_d2_ds2_d_ds (const geo_element& K, ublas::matrix<Float>& m) const
 {
-  bool scalar_diffusion = !is_weighted() || (is_weighted() && _wh.n_component() == 1);
-#ifdef TO_CLEAN
   check_macro(K.dimension()==1, 
-  	"unsupported `d_ds_d_ds' form on element type `" << K.name() << "'.");
-#endif // TO_CLEAN
+  	"unsupported `d2_ds2_d_ds' form on element type `" << K.name() << "'.");
   size_type map_d = K.dimension();
   reference_element hat_K = K;
   m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
+  vector<tensor > F_bx_table (get_first_basis().size(hat_K)); 
+  vector<point> F_by_table (get_second_basis().size(hat_K));
+  tensor DF;
+  tensor invDF;
+  for (size_type q = 0; first != last; first++, q++) {
+    jacobian_piola_transformation (K,q,DF);
+    invDF = pseudo_inverse_jacobian_piola_transformation (DF, map_d);
+    Float wq = weight_coordinate_system (K, q);
+    wq *= (*first).w;
+    wq *= (is_weighted() ? weight(K,q) : Float(1));
+    // Jacobian of Piola transform appears at power -2 in
+    //    int_e d^2 phi/ds^2 d psi/ds ds 
+    wq /= sqr(det_jacobian_piola_transformation (DF, K.dimension()));
+    vector<tensor >::const_iterator Fx_begin_hessian 
+    	= piola_basis (get_first_space(), K, get_first_basis(), DF,
+    	_bx_table.begin_hessian (hat_K, q), _bx_table.end_hessian (hat_K, q), F_bx_table.begin());
+    vector<point>::const_iterator Fy_begin_grad = piola_basis (get_second_space(), K, get_second_basis(), DF,
+    	_by_table.begin_grad (hat_K, q), _by_table.end_grad (hat_K, q), F_by_table.begin());
+    cumul_otimes (m, wq, 
+      Fy_begin_grad, Fy_begin_grad+get_second_basis().size(hat_K),
+      Fx_begin_hessian, Fx_begin_hessian+get_first_basis().size(hat_K));
+  }
+}
+// --------------------------------------------------------------------
+//   (d^2_ds^2 phi, d^2_ds^2 psi) along an edge e
+// --------------------------------------------------------------------
+void 
+form_element_rep::build_d2_ds2_d2_ds2 (const geo_element& K, ublas::matrix<Float>& m) const
+{
+  check_macro(K.dimension()==1, 
+  	"unsupported `d2_ds2_d2_ds2' form on element type `" << K.name() << "'.");
+  size_type map_d = K.dimension();
+  reference_element hat_K = K;
+  m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
+  m.clear();
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
+  vector<tensor > F_bx_table (get_first_basis().size(hat_K));
+  vector<tensor > F_by_table (get_second_basis().size(hat_K)); 
+  tensor DF;
+  tensor invDF;
+  for (size_type q = 0; first != last; first++, q++) {
+    jacobian_piola_transformation (K,q,DF);
+    invDF = pseudo_inverse_jacobian_piola_transformation (DF, map_d);
+    Float wq = weight_coordinate_system (K, q);
+    wq *= (*first).w;
+    wq *= (is_weighted() ? weight(K,q) : Float(1));
+    // Jacobian of Piola transform appears at power -3 in
+    //    int_e d^2 phi/ds^2 d^2 psi/ds^2 ds 
+    Float det=det_jacobian_piola_transformation (DF, K.dimension());
+    wq /= det*det*det;
+    vector<tensor >::const_iterator Fx_begin_hessian 
+    	= piola_basis (get_first_space(), K, get_first_basis(), DF,
+    	_bx_table.begin_hessian (hat_K, q), _bx_table.end_hessian (hat_K, q), F_bx_table.begin());
+    vector<tensor >::const_iterator Fy_begin_hessian 
+    	= piola_basis (get_second_space(), K, get_second_basis(), DF,
+    	_by_table.begin_hessian (hat_K, q), _by_table.end_hessian (hat_K, q), F_by_table.begin());
+    cumul_otimes (m, wq, 
+      Fy_begin_hessian, Fy_begin_hessian+get_second_basis().size(hat_K),
+      Fx_begin_hessian, Fx_begin_hessian+get_first_basis().size(hat_K));
+  }
+}
+// ----------------------------
+// scalar grad_grad (Laplacian)
+// ----------------------------
+void 
+form_element_rep::build_grad_grad (const geo_element& K, ublas::matrix<Float>& m) const
+{
+  bool scalar_diffusion = !is_weighted() || (is_weighted() && _wh.n_component() == 1);
+  size_type map_d = K.dimension();
+  reference_element hat_K = K;
+  m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
+  m.clear();
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
   vector<point> F_bx_table (get_first_basis().size(hat_K));
   vector<point> F_by_table (get_second_basis().size(hat_K));
   tensor DF;
@@ -703,61 +817,26 @@ form_element_rep::build_d_ds_d_ds (const geo_element& K, ublas::matrix<Float>& m
       map_d);
   }
 }
-// ----------------------------
-// scalar grad_grad (Laplacian)
-// ----------------------------
-void 
-form_element_rep::build_grad_grad (const geo_element& K, ublas::matrix<Float>& m) const
-{
-  check_macro (get_first_basis().family()==reference_element::Lagrange &&
-  	get_second_basis().family()==reference_element::Lagrange, 
-	"Only Lagrange elements supported");
-  bool scalar_diffusion = !is_weighted() || (is_weighted() && _wh.n_component() == 1);
-  size_type map_d = K.dimension();
-  check_macro (coordinate_dimension() == K.dimension(),
-    "unsupported `grad_grad' form on element type `" << K.name()
-    << " in " << coordinate_dimension() << "D geometry");
-  reference_element hat_K = K;
-  m.resize (get_second_basis().size(hat_K), get_first_basis().size(hat_K));
-  m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
-  tensor DF;
-  tensor invDF;
-  tensor Dw;
-  tensor W;
-  for (size_type q = 0; first != last; first++, q++) {
-    jacobian_piola_transformation (K,q,DF);
-    invDF = inv(DF,map_d);
-    Float wq = weight_coordinate_system (K, q);
-    wq *= det_jacobian_piola_transformation (DF, K.dimension());
-    wq *= (*first).w;
-    if (scalar_diffusion) {
-       wq *= (is_weighted() ? weight(K,q) : Float(1));
-       Dw = wq*invDF*trans(invDF,map_d);
-    } else { // tensorial diffusion
-       weight (K, q, W);
-       Dw = wq*invDF*W*trans(invDF,map_d);
-    }
-    cumul_otimes (m, Dw,
-      _by_table.begin_grad (hat_K, q), _by_table.end_grad (hat_K, q),
-      _bx_table.begin_grad (hat_K, q), _bx_table.end_grad (hat_K, q),
-      map_d);
-  }
-}
 // --------------------------------------------------------------------
-// vector-valued : (div_div phi, psi) = (div phi, div psi)
-// Mij += trans(grad_phi_i) * Dw * grad_psi_j
+//   (d_ds phi, d_ds psi) along an edge e
+// --------------------------------------------------------------------
+void
+form_element_rep::build_d_ds_d_ds (const geo_element& K, ublas::matrix<Float>& m) const
+{
+    warning_macro ("obsolete `d_ds_d_ds' form: ussing `grad_grad' instead...");
+    build_grad_grad (K, m);
+}
 // --------------------------------------------------------------------
 // int_K (gradt u : grad v) w dK = int_K [(M U).V] w dK,
 //		with u = Sum_j,l U_{nx*l+j} psi_j
 //			 v = Sum_i,k V_{ny*k+i} phi_i
 // so M_{ny*k+i,nx*l+j} = int_K d psi_j/dk * d phi_i/dl w dK 
+// --------------------------------------------------------------------
 void
 form_element_rep::build_gradt_grad (const geo_element& K, ublas::matrix<Float>& m) const
 {
-  check_macro (get_first_basis().family()==reference_element::Lagrange &&
-  	get_second_basis().family()==reference_element::Lagrange, 
+  check_macro (get_first_basis().family() == element_constant::Lagrange &&
+  	      get_second_basis().family() == element_constant::Lagrange, 
 	"Only Lagrange elements supported");
   size_type map_d = K.dimension();
   reference_element hat_K = K;
@@ -768,8 +847,8 @@ form_element_rep::build_gradt_grad (const geo_element& K, ublas::matrix<Float>&
   size_type ny = get_second_basis().size(hat_K);
   m.resize (map_d*ny, map_d*nx);
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
   tensor DF;
   tensor invDF;
   tensor invDFt;
@@ -810,8 +889,8 @@ form_element_rep::build_gradt_grad (const geo_element& K, ublas::matrix<Float>&
 void
 form_element_rep::build_div_div (const geo_element& K, ublas::matrix<Float>& m) const
 {
-  check_macro (get_first_basis().family()==reference_element::Lagrange &&
-  	get_second_basis().family()==reference_element::Lagrange, 
+  check_macro (get_first_basis().family() == element_constant::Lagrange &&
+  	      get_second_basis().family() == element_constant::Lagrange, 
 	"Only Lagrange elements supported");
   size_type map_d = K.dimension();
   reference_element hat_K = K;
@@ -822,8 +901,8 @@ form_element_rep::build_div_div (const geo_element& K, ublas::matrix<Float>& m)
   size_type ny = get_second_basis().size(hat_K);
   m.resize (map_d*ny, map_d*nx);
   m.clear();
-  quadrature::const_iterator first = _quad.begin(hat_K);
-  quadrature::const_iterator last  = _quad.end  (hat_K);
+  quadrature<Float>::const_iterator first = _quad.begin(hat_K);
+  quadrature<Float>::const_iterator last  = _quad.end  (hat_K);
   tensor DF;
   tensor invDF;
   tensor invDFt;
