@@ -429,7 +429,7 @@ georep_metric_adapt_bamg (const field& mh0, georep& adapt_g, const adapt_option_
     //
     // append domain names
     //
-    ofstream dmn (adapt_bamg_name.c_str(), ios::app);
+    ofstream dmn (adapt_bamg_name.c_str(), std::ios::app);
     // count the 1D domains:
     size_type ndom_edge = 0;
     for (size_type k = 0; k < g.n_domain(); k++) {
@@ -612,7 +612,7 @@ georep_adapt_bamg (const field& u0, georep& adapt_g, const adapt_option_type& op
     //
     // append domain names
     //
-    ofstream dmn (adapt_bamg_name.c_str(), ios::app);
+    ofstream dmn (adapt_bamg_name.c_str(), std::ios::app);
     // count the 1D domains:
     size_type ndom_edge = 0;
     for (size_type k = 0; k < g.n_domain(); k++) {
@@ -780,7 +780,7 @@ georep_adapt_bamg (const Vector<field>& criteria, georep& adapt_g, const adapt_o
     //
     // append domain names
     //
-    ofstream dmn (adapt_bamg_name.c_str(), ios::app);
+    ofstream dmn (adapt_bamg_name.c_str(), std::ios::app);
     size_type ndom=0;
     for (size_type k = 0; k < g.n_domain(); k++) 
         if (g.get_domain(k).dimension() == 1) ndom++;
