@@ -186,7 +186,7 @@ georep::get_bamg(istream& s, bool check_optional_domain_name)
 	 for (; s.good() && iter_elt != last_elt; iter_elt++, K_idx++) {
 
       	    geo_element& K = (*iter_elt);
-      	    K.set_type (n_vert_per_elt, _dim);
+      	    K.set_variant (n_vert_per_elt, _dim);
       	    K.set_index(K_idx) ;
       	    for (size_type i = 0 ; i < n_vert_per_elt; i++) {
 		s >> K[i];
@@ -199,7 +199,7 @@ georep::get_bamg(istream& s, bool check_optional_domain_name)
       	    }
             // TODO: get region number and build 2d-domains
             size_type dummy; s >> dummy;
-           _count_element [K.type()]++;
+           _count_element [K.variant()]++;
       	   _count_geo     [_dim]++;
 	 }
       }
@@ -319,18 +319,26 @@ static
 void
 insert_vertex_bdr (
     vertex_bdr_type& vertex_bdr, 
-    size_type i_vertex)
+    vertex_bdr_type& vertex_corner,
+    size_type i_vertex, size_type i_domain)
 {
     vertex_bdr_type::iterator p = vertex_bdr.find(i_vertex);
     if (p == vertex_bdr.end()) {
 	size_type i_bdr_vertex = vertex_bdr.size();
-	vertex_bdr.insert(make_pair(i_vertex,numeric_limits<size_type>::max()));
+	// the second size_t is now used to remember the domain to which i_vertex
+	// belongs, it will then be overwritten to number the boundary vertices.
+	vertex_bdr.insert(make_pair(i_vertex,i_domain));
     }
+    else
+    if ( (*p).second != i_domain )
+    // this vertex appears in several different domains: it is a corner point
+    vertex_corner.insert(make_pair(i_vertex,numeric_limits<size_type>::max()));
 }
 ostream& 
 georep::put_bamg (ostream& os) const
 {
   bool verbose = iorheo::getverbose(os);
+  size_t AngleCorner = iorheo::getanglecorner(os);
   if (dimension() != 2) {
       error_macro ("bamg output: " << dimension() << "D geometries not supported.");
   }
@@ -366,14 +374,15 @@ georep::put_bamg (ostream& os) const
      << n_bdr_edge << endl;
   size_type i_domain = 0;
   vertex_bdr_type vertex_bdr;
+  vertex_bdr_type vertex_corner;
   for (domlist_type::const_iterator iter = _domlist.begin(); iter != last; iter++, i_domain++) {
     const domain& d = *iter;
     if (d.dimension() == dimension() -1) {
       for (domain::const_iterator iter_s = d.begin(); iter_s != d.end(); iter_s++) {
           const geo_element& S = *iter_s;
           os << S[0]+1 << " " << S[1]+1 << " " << i_domain+1 << endl;
-          insert_vertex_bdr (vertex_bdr, S[0]);
-          insert_vertex_bdr (vertex_bdr, S[1]);
+          insert_vertex_bdr (vertex_bdr, vertex_corner, S[0], i_domain);
+          insert_vertex_bdr (vertex_bdr, vertex_corner, S[1], i_domain);
       }
     }
   }
@@ -450,7 +459,7 @@ georep::put_bamg (ostream& os) const
   //
   cad << setprecision(numeric_limits<Float>::digits10) 
       << "MeshVersionFormatted\n0\n\n"
-      << "AngleOfCornerBound\n0\n\n"
+      << "AngleOfCornerBound\n" << AngleCorner << "\n\n"
       << "Dimension\n2\n\n"
       << "Vertices\n"
       << vertex_bdr.size() << endl;
@@ -461,7 +470,7 @@ georep::put_bamg (ostream& os) const
   vector<size_t> vertice_domain_bamg_number(n_node(), 1);
   for (domlist_type::const_iterator iter = _domlist.begin(); iter != last; iter++, i_domain++) {
     const domain& d = *iter;
-    if (d.dimension() == 0) {
+    if (d.dimension() == 0) { 
       for (domain::const_iterator iter_s = d.begin(); iter_s != d.end(); iter_s++) {
         const geo_element& vertex = *iter_s;
         size_type i_vertex = vertex[0];
@@ -479,6 +488,22 @@ georep::put_bamg (ostream& os) const
     cad << x[i_vertex][0] << " " << x[i_vertex][1] << " "
         << vertice_domain_bamg_number[i_vertex] << endl;
   }
+#ifdef BUG_BAMG
+  //
+  //
+  // cad corners
+  //
+  if (vertex_corner.size()>0)
+   {
+	  cad << endl << "Corners " << vertex_corner.size() << endl;
+	  size_t i_corner=0;
+	  for (vertex_bdr_type::iterator p = vertex_corner.begin();
+		p != vertex_corner.end(); p++, i_corner++) {
+	    size_type i_vertex = (*p).first;
+	    cad << (*p).first+1 << endl;
+	  }
+   }
+#endif // BUG_BAMG
   //
   // cad edges
   //
