@@ -146,7 +146,7 @@ void geomap::init() {
 	 cerr << "Elem " << K_index_Th_1 << endl;
 	 const geo_element& K_Th_1 =*iK_Th_1;
 	 // TO BE MODIFIED: This may slow down mixed meshes (triangles+quadrangles,...)
-//	if (!((*K_Th_1).type() ==geo_element::t))
+//	if (!((*K_Th_1).variant() ==geo_element::t))
 //	 {
 //	     cerr << "Not yet implemented for non triangular meshes";
 //	 }
@@ -281,11 +281,14 @@ meshpoint geomap::robust_advect_N (const point& x0, const point& v0, const field
   } while (true);
 }
 static Float h_moy (geo::const_iterator_node p, const geo_element& K) {
-  if (K.type() == geo_element::t) {
+  if (K.variant() == geo_element::e) {
+    return norm(p[K[1]] - p[K[0]]);
+  }
+  if (K.variant() == geo_element::t) {
     Float area = fabs( vect2d( p[K[1]] -p[K[0]], p[K[2]] -p[K[0]] ))/2;
     return ::sqrt(area);
   }
-  if (K.type() == geo_element::q) {
+  if (K.variant() == geo_element::q) {
     Float area1 = fabs( vect2d( p[K[1]] -p[K[0]], p[K[2]] -p[K[0]] ))/2;
     Float area2 = fabs( vect2d( p[K[2]] -p[K[0]], p[K[3]] -p[K[0]] ))/2;
     return ::sqrt(area1 + area2);
@@ -305,7 +308,7 @@ static inline size_t adapt_n_track_step (Float h0, const point& v0, size_t n_tra
 void geomap::init (const field& advection) {
   const space& Va  = advection.get_space();
   const geo&   g_1 = _Vh_1.get_geo();
-  init_localizer (Va.get_geo());
+  init_localizer (_Th_2);
   if (Va.n_component() !=  Va.dimension())
 	error_macro("Advection field should be a " << Va.dimension() <<"D vectors field."); 
   if (_Vh_1.n_component() != 1) {
