@@ -49,7 +49,7 @@ namespace rheolef {
 // ============================================================================
 
 
-class georep : public occurence, public Vector<geo_element> {
+class georep : public Vector<geo_element> {
 public:
   
 // typedefs:
@@ -66,6 +66,8 @@ public:
         point xmin;             // If xmin==xmax then use geo's
         point xmax;
 	bool equal_scales;		
+	bool keep_user_range;   // For live plotting: don't rewrite the range from last iteration
+	bool replot;   		// For live plotting: don't erase previous plot (i.e. use replot)
         Float border;           // In units of diagonal length
         bool postscript;        // Generate postscript output (gnuplot)
         bool noxplot;           // Generate postscript output and do not open
@@ -73,19 +75,23 @@ public:
         bool colour_postscript;
         Float linewidth;
 	Float vscale;		// velocity scaling
+	bool map;		// no elevation 
 
 	bool double_precision;	// Should the current mesh be saved in double precision
 	
 	bool execute;
 	bool clean;
 	bool verbose;
+	bool pause;
+	bool persist;
 
         plot_options(): bare(false), empty(false), silhouette(false), meshonly(false), meshplot(false),
 		xmirror(false), ymirror(false),
-                xmin(0,0,0), xmax(0,0,0), equal_scales(true), border(0),
+                xmin(0,0,0), xmax(0,0,0), equal_scales(true), keep_user_range(false), replot(false),
+		border(0),
                 postscript(false), noxplot(false), colour_postscript(true),
-                linewidth(0.1), vscale(0.2),
-		execute(true), clean(true), verbose(false)
+                linewidth(0.1), vscale(0.2), map(false),
+		execute(true), clean(true), verbose(false), pause(true), persist(true)
          {}
    };
   void write_gnuplot_postscript_options (std::ostream& plot, const plot_options& opt) const;
@@ -165,6 +171,7 @@ public:
   fem_helper::coordinate_type coordinate_system_type() const { return _coord_sys; }
   size_type number() const;
   size_type serial_number() const;
+  size_type zero_level_set_number() const;
   bool localizer_initialized () const;
   const domain& boundary () const;
 
@@ -176,11 +183,11 @@ public:
   size_type n_triangle() const ;
   size_type n_quadrangle() const ;
   size_type n_volume() const ;
-  size_type n_tetraedra() const ;
+  size_type n_tetrahedra() const ;
   size_type n_prism() const ;
   size_type n_hexaedra() const ;
   size_type n_subgeo(size_type d) const;
-  size_type n_element(reference_element::enum_type t) const;
+  size_type n_element(reference_element::variant_type t) const;
 
   const geo_element& element (size_type K_idx) const;
   const point&       vertex  (size_type i) const;
@@ -282,7 +289,7 @@ protected:
   // n_point, n_edge, n_face, n_volume
   const size_type* get_geo_counter() const;
 
-  // 	.. n_triangle ... n_tetraedra ..
+  // 	.. n_triangle ... n_tetrahedra ..
   const size_type* get_element_counter() const;
 
   void reset_counters();
@@ -365,6 +372,7 @@ protected:
   std::string 	      _name;
   size_type	      _number;
   size_type	      _serial_number;
+  mutable size_type   _zero_level_set_number;
   size_type           _dim;        	// 0, 1, 2 or 3 for elements
   fem_helper::coordinate_type _coord_sys;  // cartesian, axisymmetric
   nodelist_type       _x;		// coordinates
@@ -374,7 +382,7 @@ protected:
   bool		      _save_double_precision;
  
   size_type           _count_geo [4];
-  size_type           _count_element [reference_element::max_size];
+  size_type           _count_element [reference_element::max_variant];
   size_type           _version;
   mutable geo_tree    _tree;
   mutable hazel	      _h;
@@ -394,11 +402,11 @@ void georep_calculate_metric_bamg (const class field& u0,
 inline
 georep::georep()
 
- : occurence(),
-   Vector<geo_element>(),
+ : Vector<geo_element>(),
    _name(),
    _number(std::numeric_limits<size_type>::max()),
    _serial_number(std::numeric_limits<size_type>::max()),
+   _zero_level_set_number(0),
    _dim(0), 
    _coord_sys(fem_helper::cartesian),
    _x(), 
@@ -503,6 +511,12 @@ georep::serial_number() const
 }
 inline
 georep::size_type
+georep::zero_level_set_number() const
+{
+  return _zero_level_set_number++;
+}
+inline
+georep::size_type
 georep::size() const
 {
   return Vector<geo_element>::size();
@@ -551,7 +565,7 @@ georep::n_volume() const
 }
 inline
 georep::size_type
-georep::n_tetraedra() const
+georep::n_tetrahedra() const
 {
   return _count_element[reference_element::T] ;
 }
@@ -575,7 +589,7 @@ georep::n_subgeo(georep::size_type d) const
 }
 inline
 georep::size_type
-georep::n_element(reference_element::enum_type t) const
+georep::n_element(reference_element::variant_type t) const
 {
   return _count_element[t];
 }
