@@ -36,6 +36,7 @@ using namespace std;
          IO_RHEO_SCALAR (point, normal)
 # undef  IO_RHEO_SCALAR
 
+namespace rheolef {
 // ---------------------------------------------------------------------
 // constructor and destructor, copy and assignement
 // ---------------------------------------------------------------------
@@ -60,9 +61,4 @@ template class iorheobase_memory_handler<iofem>;
 // static variable initialization in template class:
 template<> list<iofem*> *iorheobase_memory_handler<iofem>::pointer_list = 0;
 
-iofem* 
-iofem::get_pointer (ios& s)
-{
-    return iorheobase_memory_handler<iofem>::get_pointer(s);
-}
-
+} // namespace rheolef
