@@ -177,6 +177,8 @@ public:
 	const domain& interface, const domain& subgeo, const std::string& valued = "scalar");
 	//! For spaces with a discontinuity through domain `interface', but otherwise continuous
     space (const geo& g, const domain& d, const std::string& approx_name);
+    space (const field& phi, const std::string& tag, const std::string& approx_name = "P1");
+
     space(const const_space_component&);
 
     space operator = (const const_space_component&);
@@ -199,6 +201,9 @@ public:
     void unblock_dof (size_type dof_idx) const;
     void set_periodic (const domain& d1, const domain& d2) const;
     void set_periodic (const std::string& d1_name, const std::string& d2_name) const;
+    void set_uniform (const domain& d, size_type i_comp) const;
+    	//! On a uniform boundary, all dofs have the same value.
+	//! Useful e.g. for solid boundaries with translation only.
     void lock_components (const domain& d, point locked_dir) const;
     void lock_components (const std::string& d_name, point locked_dir) const;
     template <class Function>
@@ -282,7 +287,7 @@ public:
     void  set_global_dof (const geo_element& K, tiny_vector<size_type>& idx) const;
     void  set_dof        (const geo_element& K, tiny_vector<size_type>& idx, size_type i_comp) const;
     void  set_global_dof (const geo_element& K, tiny_vector<size_type>& idx, size_type i_comp) const;
-    void  set_dof        (const geo_element& K, tiny_vector<size_type>& idx, size_type i_comp, reference_element::dof_family_type family) const;
+    void  set_dof        (const geo_element& K, tiny_vector<size_type>& idx, size_type i_comp, element_constant::dof_family_type family) const;
     point x_dof      (const geo_element& K, size_type i_local) const;
      //! Hermite/Argyris: returns 1 if the global dof contains the derivative along outward normal of K, -1 else  
     void  dof_orientation(const geo_element& K, tiny_vector<int>& orientation) const;
@@ -403,6 +408,10 @@ inline
 space::space (const geo& g, const domain& d, const std::string& approx_name) 
   : smart_pointer<spacerep> (new_macro(spacerep(g, d, approx_name))) {}
 
+inline
+space::space (const field& phi, const std::string& tag, const std::string& approx_name)
+  : smart_pointer<spacerep> (new_macro(spacerep(phi, tag, approx_name))) {}
+
 inline
 space::space (const space& X, const space& Y) 
   : smart_pointer<spacerep> (new_macro(spacerep(X.data(),Y.data()))) {}
@@ -689,6 +698,12 @@ space::set_periodic (const std::string& d1_name, const std::string& d2_name) con
 }
 inline
 void 
+space::set_uniform (const domain& d, size_type i_comp) const
+{
+  data().set_uniform(d, i_comp);
+}
+inline
+void 
 space::lock_components (const domain& d, point locked_dir) const
 {
   data().lock_components(d, locked_dir);
@@ -776,7 +791,7 @@ space::set_dof( const geo_element& K, tiny_vector<size_type>& idx,
 inline
 void
 space::set_dof( const geo_element& K, tiny_vector<size_type>& idx,
-		size_type i_comp, reference_element::dof_family_type family) const
+		size_type i_comp, element_constant::dof_family_type family) const
 {
   data().set_dof(K, idx, i_comp, family);
 }
