@@ -145,6 +145,10 @@ OTHER OPTIONS:
         of the branch, and this could be problematic when this
         field is initialy zero.
 
+    @item -subdivide int
+        When using a high order geometry, the number of points per edge used to draw
+        a curved element. Default value is the mesh order.
+
 @cindex topography
     @item -topography @var{filename}[.field[.gz]]
         performs a tridimensionnal elevation view based
@@ -268,6 +272,7 @@ void usage()
            << " [-scale float]"
            << " [-umin float]"
            << " [-umax float]"
+           << " [-subdivide int]"
 	   << " [-topography filename]"
 	   << " [-proj]"
 	   << " [-paraview|-vtk|-gnuplot|-plotmtv]"
@@ -410,6 +415,10 @@ int main(int argc, char**argv)
         } else if (strcmp (argv[i], "-scale") == 0)   {
             if (i+1 == argc || !is_float(argv[i+1])) usage();
             scale_value = to_float (argv[++i]);
+        } else if (strcmp (argv[i], "-subdivide") == 0) {
+            if (i == argc-1) { cerr << "branch -subdivide: option argument missing" << endl; usage(); }
+            size_t nsub = atoi(argv[++i]);
+            dout.os() << setsubdivide (nsub);
         } else if (strcmp (argv[i], "-topography") == 0)   {
 
             if (i+1 == argc) usage();
