@@ -46,7 +46,12 @@ INPUT FILE SPECIFICATION:
 
     @item -
 	read field on standard input instead on a file.
-
+    @item -name
+        when the field comes from standard input, the file base name
+        is not known and is set to "output" by default.
+        This option allows to change this default.
+        Useful when dealing with output formats (graphic, format conversion)
+        that creates auxilliary files, based on this name.
 @cindex RHEOPATH environment variable
     @item -I@var{dir} 
     @itemx -I @var{dir} 
@@ -58,8 +63,8 @@ INPUT FILE SPECIFICATION:
 
 
 @clindex catchmark
-    @item -catchmark @var{label}
-	Jump accross the file to the specifield label.
+    @item -catchmark @var{string}
+	Jump accross the file to the specifield @var{string}.
         Label start at the begining of a line, preceded by a @file{#} mark (see @pxref{catchmark algorithm}).
     @end table
 
@@ -85,17 +90,28 @@ OUTPUT FILE FORMAT OPTIONS:
 @cindex image file format
 @cindex graphic render
 @fiindex @file{.png} image
-@fiindex @file{.gif} image
 @fiindex @file{.jpg} image
-@fiindex @file{.ps} image
+@fiindex @file{.gif} image
+@fiindex @file{.tif} image
+@fiindex @file{.ppm} image
+@fiindex @file{.bmp} image
 @fiindex @file{.pdf} image
+@fiindex @file{.eps} image
+@fiindex @file{.ps} image
+@fiindex @file{.svg} image
   @item -image-format @code{string}
         The argument is any valid image format, such as
-        bitmap @code{png}, @code{jpg}, @code{gif} or vectorial @code{ps} or @code{pdf}
-        image file formats, and that could be handled by the corresponding render.
-        The output file is e.g. @emph{basename}.@code{png} when @emph{basename}
-        is the name of the mesh, or can be set with the @code{-name} option.
-
+        bitmap @code{png}, @code{jpg}, @code{gif}, @code{tif}, @code{ppm}, @code{bmp}
+        or vectorial @code{pdf}, @code{eps}, @code{ps}, @code{svg}
+	image file formats.
+	this option can be used with the @code{paraview} and the @code{gnuplot} renders.
+	The output file is e.g. @emph{basename}.@code{png} when @emph{basename}
+	is the name of the mesh, or can be set with the @code{-name} option.
+  @item -resolution @code{int} @code{int}
+        The argument is a couple of sizes, for the image resolution.
+	This option can be used together with the @code{-image-format}
+	for any of the bitmap image formats.
+	This option requires the @code{paraview} render.
   @end table
 
 GETTING INFORMATION:
@@ -116,6 +132,10 @@ RENDER OPTIONS:
   @item -mayavi
         use @code{mayavi} tool.
         This is the default for tridimensional geometries.
+@toindex @code{paraview}
+  @item -paraview
+        use @code{paraview} tool.
+        This is feature is under development.
   @end table
 
 RENDERING OPTIONS:
@@ -126,6 +146,10 @@ RENDERING OPTIONS:
     @itemx -bw
 	Use (color/gray scale/black and white) rendering.
 	Color rendering is the default.
+    @item -label
+    @itemx -nolabel
+        Show or hide title, color bar and various anotations.
+        Default is to show labels.
     @item -elevation
     @itemx -noelevation
 	For two dimensional field, represent values as elevation
@@ -138,17 +162,21 @@ RENDERING OPTIONS:
     @item -stereo
     @itemx -nostereo
         Rendering mode suitable for red-blue anaglyph 3D stereoscopic glasses.
-        This option is only available with @code{mayavi}.
+        This option is only available with @code{mayavi} and @code{paraview}.
     @item -fill
 	isoline intervals are filled with color.
 	This is the default.
     @item -nofill
 	draw isolines by using lines.
+    @item -volume
+    @itemx -novolume
+	for 3D data, render values using a colored translucid volume.
+        This option requires the @code{paraview} code.
     @item -cut
     @itemx -nocut
         Cut by a specified plane.
         The cutting plane is specified by its origin point and normal vector.
-        This option requires the @code{mayavi} code.
+        This option requires the @code{mayavi} or @code{paraview} code.
     @item -origin @var{float} [@var{float} [@var{float}]]
         set the origin of the cutting plane.
         Default is (0.5, 0.5, 0.5).
@@ -191,11 +219,11 @@ RENDERING OPTIONS:
 
     @item -deformation
         Render vector-valued fields as deformed mesh
-        using @code{mayavi} or @code{gnuplot}.
+        using @code{mayavi}, @code{paraview} or @code{gnuplot}.
         This is the default vector field representation.
 
     @item -velocity
-        Render vector-valued fields as arrows using @code{mayavi}.
+        Render vector-valued fields as arrows using @code{mayavi} or  @code{paraview}.
 
     @end table
 COMPONENT EXTRACTION AND DOMAIN REDUCTION:
@@ -299,15 +327,19 @@ void usage() {
            << "[-catchmark string] "
            << "[-min|-max] "
            << "[-field|-gmsh|-gmsh-pos|-bamg-bb] "
-           << "[-gnuplot|-mayavi] "
-           << "[-[no]fill] [-[no]elevation] "
+           << "[-gnuplot|-mayavi|-paraview] "
+           << "[-image-format string] "
+           << "[-resolution int [int]] "
+           << "[-name string] "
            << "[-color|-gray|-black-and-white|-bw] "
+           << "[-[no]label] "
+           << "[-[no]fill] "
+           << "[-[no]elevation] "
            << "[-[no]stereo] "
-           << "[-[no]cut| "
+           << "[-[no]cut] "
 	   << "[-noiso|-iso float] "
            << "[-n-iso int] "
            << "[-n-iso-negative int] "
-           << "[-image-format string] "
            << "[-subdivide int] "
            << "[-comp string] "
            << "[-domain string] "
@@ -332,6 +364,7 @@ typedef enum {
         no_render,
         gnuplot_render,
         mayavi_render,
+        paraview_render,
         plotmtv_render,
         vtk_render,
         atom_render,
@@ -359,6 +392,7 @@ int main(int argc, char**argv) {
     // ----------------------------
     string filename = "";
     string name = "output";
+    dout.os() << setbasename(name);
     string format = "";
     dout.os() << verbose; bool bverbose = true;
     std::string mark = "";
@@ -398,10 +432,14 @@ int main(int argc, char**argv) {
         else if (strcmp (argv[i], "-gmsh") == 0)            { dout.os() << gmsh;     render = file_render; }
         else if (strcmp (argv[i], "-gmsh-pos") == 0)        { dout.os() << gmsh_pos; render = file_render; }
         else if (strcmp (argv[i], "-bamg-bb") == 0)         { dout.os() << bamg;     render = file_render; }
-
-	// render spec:
+        else if (strcmp (argv[i], "-name") == 0)            {
+            if (i+1 == argc) { std::cerr << "field -name: option argument missing" << std::endl; usage(); }
+            name = argv[++i];
+	}
+        // render spec:
         else if (strcmp (argv[i], "-gnuplot") == 0)         { dout.os() << gnuplot;  render = gnuplot_render; }
         else if (strcmp (argv[i], "-mayavi")  == 0)         { dout.os() << mayavi;   render = mayavi_render; }
+        else if (strcmp (argv[i], "-paraview")  == 0)       { dout.os() << paraview; render = paraview_render; }
 
 	// filter 
         else if (strcmp (argv[i], "-proj") == 0)            { do_proj = true; }
@@ -421,12 +459,23 @@ int main(int argc, char**argv) {
         else if (strcmp (argv[i], "-gray") == 0)            { dout.os() << gray; }
         else if (strcmp (argv[i], "-black-and-white") == 0) { dout.os() << black_and_white; }
         else if (strcmp (argv[i], "-bw") == 0)              { dout.os() << black_and_white; }
-        else if (strcmp (argv[i], "-stereo") == 0)          { dout.os() << stereo << mayavi;   render = mayavi_render; }
+        else if (strcmp (argv[i], "-label") == 0)           { dout.os() << label; }
+        else if (strcmp (argv[i], "-nolabel") == 0)         { dout.os() << nolabel; }
+        else if (strcmp (argv[i], "-stereo") == 0)          { dout.os() << stereo; 
+							      if (render != mayavi_render && 
+                                                                  render != paraview_render) {
+                                                                dout.os() << paraview; 
+							        render  = paraview_render;
+                                                              }
+                                                            }
         else if (strcmp (argv[i], "-nostereo") == 0)        { dout.os() << nostereo; }
-        else if (strcmp (argv[i], "-no-show-cut") == 0)     { dout.os() << nocut; def_plane_cut_opt = true; }
-        else if (strcmp (argv[i], "-show-cut") == 0)        { dout.os() << cut;   def_plane_cut_opt = true; }
+        else if (strcmp (argv[i], "-volume") == 0)          { dout.os() << paraview << volume;   
+							      render  = paraview_render; }
+        else if (strcmp (argv[i], "-novolume") == 0)        { dout.os() << novolume; }
         else if (strcmp (argv[i], "-cut") == 0)             { do_cut = true; }
         else if (strcmp (argv[i], "-nocut") == 0)           { do_cut = false; }
+        else if (strcmp (argv[i], "-show-cut") == 0)        { dout.os() << cut;   def_plane_cut_opt = true; }
+        else if (strcmp (argv[i], "-no-show-cut") == 0)     { dout.os() << nocut; def_plane_cut_opt = true; }
         else if (strcmp (argv[i], "-noiso") == 0)           { dout.os() << noiso; do_iso3d = false; }
         else if (strcmp (argv[i], "-iso") == 0)   {
 
@@ -464,6 +513,12 @@ int main(int argc, char**argv) {
             if (format == "postscript") format = "ps";
             dout.os() << setimage_format(format);
         }
+        else if (strcmp (argv[i], "-resolution") == 0) {
+            if (i == argc-1 || !isdigit(argv[i+1][0])) { std::cerr << "geo -resolution: option argument missing" << std::endl; usage(); }
+            size_t nx = atoi(argv[++i]);
+            size_t ny = (i < argc-1 && isdigit(argv[i+1][0])) ? atoi(argv[++i]) : nx;
+	    dout.os() << setresolution(point_basic<size_t>(nx,ny));
+        }
         else if (strcmp (argv[i], "-catchmark") == 0)       {
             if (i == argc-1) {
                 cerr << "field -catchmark: option argument missing" << endl;
@@ -536,8 +591,12 @@ int main(int argc, char**argv) {
         cerr << "field: no input file specified" << endl;
         usage();
     } else if (filename == "-") {
+        // field on stdin
+        std::string thename;
+        if (name != "output") thename = name;
+        std::cin >> setbasename(thename);
         if (mark != "") din >> catchmark(mark);
-        dout.os() << setbasename("output") << reader_on_stdin;
+        dout.os() << setbasename(name) << reader_on_stdin;
         din >> uh;
     } else {
         idiststream ids;
@@ -546,19 +605,19 @@ int main(int argc, char**argv) {
         if (mark != "") ids >> catchmark(mark);
         ids >> uh;
         std::string root_name = delete_suffix (delete_suffix(filename, "gz"), "field");
-        std::string name = get_basename (root_name);
+        name = get_basename (root_name);
         dout.os() << setbasename(name);
     }
     if (uh.valued_tag() == space_constant::vector) { 
-      if (vector_style == velocity_style) {
-        // gnuplot does not support yet velocity style
-        dout.os() << mayavi; render = mayavi_render;
+      if (vector_style == velocity_style && (render != mayavi_render && render != paraview_render)) {
+        // gnuplot does not support yet velocity style: requieres mayavi or paraview
+        dout.os() << paraview; render = paraview_render;
       }
     }
     if (render == no_render) {
         // try to choose the best render from dimension
         if (uh.get_geo().map_dimension() == 3) {
-          dout.os() << mayavi;
+          dout.os() << paraview;
         } else {
           dout.os() << gnuplot;
         }
@@ -591,8 +650,22 @@ int main(int argc, char**argv) {
       uh = vh;
     }
     if (reduce_to_domain != "") {
-      field_basic<Float,sequential> vh = uh[reduce_to_domain];
+      geo_basic<Float,sequential> dom = uh.get_geo()[reduce_to_domain];
+      field_basic<Float,sequential> vh;
+      if (uh.get_space().get_numbering().is_discontinuous() && dom.map_dimension()+1 == uh.get_geo().map_dimension()) {
+        // brdy trace of a discontinuous field
+        space_basic<Float,sequential> Wh (dom, uh.get_approx(), uh.valued());
+	uh.get_geo().neighbour_guard();
+        vh = interpolate (Wh, uh);
+      } else {
+        vh = uh[reduce_to_domain];
+      }
       uh = vh;
+      if (render == file_render) {
+	// put field on dout: save also the cutted geo:
+	odiststream geo_out (uh.get_geo().name(), "geo");
+        geo_out << uh.get_geo();
+      }
     }
     if (! do_proj && render == mayavi_render && uh.valued() != "scalar"
         && uh.get_space().get_numbering().has_compact_support_inside_element()) {
@@ -636,23 +709,23 @@ int main(int argc, char**argv) {
                << uh.max() << endl;
           return 0;
     }
-    // generates label (mayavi support it now):
+    // generates marks (mayavi support it now):
     {
       string root_filename = iorheo::getbasename(dout.os());
-      string label = (root_filename == "output") ? "" : root_filename;
+      string full_mark = (root_filename == "output") ? "" : root_filename;
       if (mark != "") {
-        // label = (label == "") ? mark : label + "." + mark;
-        label = mark;
+        // full_mark = (full_mark == "") ? mark : full_mark + "." + mark;
+        full_mark = mark;
       }
       if (i_comp_name != "") { 
-         label = (label == "") ? "value" : label;
-         label = label + "[" + i_comp_name + "]";
+         full_mark = (full_mark == "") ? "value" : full_mark;
+         full_mark = full_mark + "[" + i_comp_name + "]";
       }
       if (reduce_to_domain != "") {
-         label = (label == "") ? "value" : label;
-         label = label + "[" + reduce_to_domain + "]";
+         full_mark = (full_mark == "") ? "value" : full_mark;
+         full_mark = full_mark + "[" + reduce_to_domain + "]";
       }
-      dout.os() << setmark (label);
+      dout.os() << setmark (full_mark);
     }
     // final output:
     dout << uh;
