@@ -100,8 +100,11 @@ RENDER SPECIFICATION:
         use gnuplot tool.
         This is the default.
 @toindex @code{mayavi}
+@toindex @code{paraview}
   @item -mayavi
         use @code{mayavi} tool.
+  @item -paraview
+        use @code{paraview} tool.
   @end table
 
 RENDER OPTIONS:
@@ -120,9 +123,14 @@ RENDER OPTIONS:
 @cindex @code{stereo 3D rendering}
   @item -[no]stereo
         Rendering mode suitable for red-blue anaglyph 3D stereoscopic glasses.
-        Option only available with @code{mayavi}.
+        Option only available with @code{mayavi} or @code{paraview}.
+  @item -[no]shrink
+        shrink elements (with @code{paraview} only).
   @item -[no]cut
-        cut by plane and clip (with @code{mayavi} only).
+        cut by plane and clip (with @code{mayavi} or @code{paraview} only).
+  @item -[no]label
+        Show or hide labels, boundary domains and various annotations.
+        By default, domains are showed with a specific color.
   @end table
 
 OUTPUT FILE FORMAT OPTIONS:
@@ -140,17 +148,29 @@ OUTPUT FILE FORMAT OPTIONS:
 @cindex graphic render
 @cindex image file format
 @cindex graphic render
-@fiindex @file{.png} image 
-@fiindex @file{.gif} image
+@fiindex @file{.png} image
 @fiindex @file{.jpg} image
-@fiindex @file{.ps} image
+@fiindex @file{.gif} image
+@fiindex @file{.tif} image
+@fiindex @file{.ppm} image
+@fiindex @file{.bmp} image
 @fiindex @file{.pdf} image
+@fiindex @file{.eps} image
+@fiindex @file{.ps} image
+@fiindex @file{.svg} image
   @item -image-format @code{string}
         The argument is any valid image format, such as
-        bitmap @code{png}, @code{jpg}, @code{gif} or vectorial @code{ps} or @code{pdf}
-	image file formats, and that could be handled by the corresponding render.
+        bitmap @code{png}, @code{jpg}, @code{gif}, @code{tif}, @code{ppm}, @code{bmp}
+        or vectorial @code{pdf}, @code{eps}, @code{ps}, @code{svg}
+	image file formats.
+	this option can be used with the @code{paraview} and the @code{gnuplot} renders.
 	The output file is e.g. @emph{basename}.@code{png} when @emph{basename}
 	is the name of the mesh, or can be set with the @code{-name} option.
+  @item -resolution @code{int} @code{int}
+        The argument is a couple of sizes, for the image resolution.
+	This option can be used together with the @code{-image-format}
+	for any of the bitmap image formats.
+	This option requires the @code{paraview} render.
   @end table
 
 OTHERS OPTIONS:
@@ -197,6 +217,12 @@ OTHERS OPTIONS:
 
 INQUIRE OPTIONS:
   @table @code
+  @item -size
+  @itemx -n-vertex
+        print the mesh size, i.e. the number of elements (resp. of vertices) and then exit.
+  @item  -hmin
+  @itemx -hmax
+        print the smallest (resp. largest) edge length and then exit.
   @item -min-element-measure
   @itemx -max-element-measure
         print the smallest (resp. largest) element measure and then exit.
@@ -211,9 +237,7 @@ End:
 */
 
 # include "rheolef.h"
-//# include "rheolef/iorheo.h"
-//# include "rheolef/iofem.h"
-//# include "rheolef/rheostream.h"
+# include "rheolef/iofem.h"
 using namespace rheolef;
 void usage() {
       std::cerr
@@ -224,15 +248,18 @@ void usage() {
            << "[-check] "
            << "[-upgrade] "
            << "[-if {geo,bamg,vtk}] "
-           << "[-geo|-gmsh|-gnuplot|-mayavi] "
+           << "[-geo|-gmsh|-gnuplot|-mayavi|-paraview] "
            << "[-name string] "
            << "[-[no]full|-[no]fill|-[no]lattice] "
            << "[-subdivide int] "
-           << "[-[no]stereo|-[no]full|-[no]cut] "
+           << "[-[no]stereo|-[no]full|-[no]cut|-[no]shrink|-[no]label] "
            << "[-image-format string] "
+           << "[-resolution int [int]] "
            << "[-add-boundary] "
            << "[-zr|-rz] "
-           << "[-[min|max]-elment-measure] "
+           << "[-size] [-n-vertex] "
+           << "[-hmin] [-hmax] "
+           << "[-[min|max]-element-measure] "
            << "[-[no]clean] [-[no]execute] [-[no]verbose] "
            << std::endl;
       exit (1);
@@ -251,6 +278,7 @@ typedef enum {
         no_render,
         gnuplot_render,
         mayavi_render,
+        paraview_render,
         plotmtv_render,
         vtk_render,
         gmsh_render,
@@ -259,6 +287,29 @@ typedef enum {
         file_render
 } render_type;
 
+#ifdef TO_CLEAN
+void h_extrema (const geo_basic<Float,sequential>& omega, Float& hmin, Float& hmax)
+{
+  hmin = std::numeric_limits<Float>::max();
+  hmax = 0;
+  for (size_t iedg = 0, nedg = omega.geo_element_ownership(1).size(); iedg < nedg; ++iedg) {
+    const geo_element& E = omega.get_geo_element (1, iedg);
+    const point& x0 = omega.dis_node(E[0]);
+    const point& x1 = omega.dis_node(E[1]);
+    Float hloc = dist(x0,x1);
+    hmin = min(hmin, hloc);
+    hmax = max(hmax, hloc);
+  }
+#ifdef TODO
+ // M=sequential here (just if this fct goes into the geo<M,T> class later)
+#ifdef _RHEOLEF_HAVE_MPI
+  hmin = mpi::all_reduce (comm(), hmin, mpi::minimum<Float>());
+  hmax = mpi::all_reduce (comm(), hmax, mpi::maximum<Float>());
+#endif // _RHEOLEF_HAVE_MPI
+#endif // TODO
+}
+#endif // TO_CLEAN
+
 Float extrema_element_measure (const geo_basic<Float,sequential>& omega, bool is_min)
 {
   space_basic<Float,sequential> Xh (omega, "P0");
@@ -275,6 +326,11 @@ int main(int argc, char**argv) {
     bool do_upgrade  = false ;
     bool do_check    = false ;
     bool do_add_bdry = false ;
+    bool do_stereo   = false ;
+    bool show_n_element = false ;
+    bool show_n_vertex = false ;
+    bool show_hmin = false ;
+    bool show_hmax = false ;
     bool show_min_element_measure = false ;
     bool show_max_element_measure = false ;
     std::string name = "output";
@@ -284,6 +340,7 @@ int main(int argc, char**argv) {
     dout.os() << lattice;
     dout.os() << nofill;
     dout.os() << grid;
+    dout.os() << label;
 #ifdef TO_CLEAN
     dout.os() << setorigin (point(0,0,0));
     dout.os() << setnormal (point(1,0,0));
@@ -306,6 +363,10 @@ int main(int argc, char**argv) {
         else if (strcmp (argv[i], "-add-boundary") == 0) { do_add_bdry = true; }
         else if (strcmp (argv[i], "-rz") == 0)        { sys_coord = "rz"; }
         else if (strcmp (argv[i], "-zr") == 0)        { sys_coord = "zr"; }
+        else if (strcmp (argv[i], "-size") == 0)      { show_n_element = true; }
+        else if (strcmp (argv[i], "-n-vertex") == 0)  { show_n_vertex = true; }
+        else if (strcmp (argv[i], "-hmin") == 0)      { show_hmin = true; }
+        else if (strcmp (argv[i], "-hmax") == 0)      { show_hmax = true; }
         else if (strcmp (argv[i], "-min-element-measure") == 0)  { show_min_element_measure = true; }
         else if (strcmp (argv[i], "-max-element-measure") == 0)  { show_max_element_measure = true; }
         else if (strcmp (argv[i], "-I") == 0)         {
@@ -319,19 +380,24 @@ int main(int argc, char**argv) {
 
 	// render spec:
         else if (strcmp (argv[i], "-gnuplot") == 0)   { dout.os() << gnuplot;  render = gnuplot_render; }
-        else if (strcmp (argv[i], "-mayavi") == 0)   { dout.os() << mayavi;  render = mayavi_render; }
+        else if (strcmp (argv[i], "-mayavi") == 0)    { dout.os() << mayavi;   render = mayavi_render; }
+        else if (strcmp (argv[i], "-paraview") == 0)  { dout.os() << paraview; render = paraview_render; }
 
 	// render option:
         else if (strcmp (argv[i], "-full") == 0)      { dout.os() << full; }
         else if (strcmp (argv[i], "-nofull") == 0)    { dout.os() << nofull; }
         else if (strcmp (argv[i], "-fill") == 0)      { dout.os() << fill; }
         else if (strcmp (argv[i], "-nofill") == 0)    { dout.os() << nofill; }
-        else if (strcmp (argv[i], "-stereo") == 0)    { dout.os() << stereo << mayavi; render = mayavi_render; }
+        else if (strcmp (argv[i], "-stereo") == 0)    { dout.os() << stereo; do_stereo = true; }
         else if (strcmp (argv[i], "-nostereo") == 0)  { dout.os() << nostereo; }
         else if (strcmp (argv[i], "-cut") == 0)       { dout.os() << cut; }
         else if (strcmp (argv[i], "-nocut") == 0)     { dout.os() << nocut; }
+        else if (strcmp (argv[i], "-shrink") == 0)    { dout.os() << shrink; }
+        else if (strcmp (argv[i], "-noshrink") == 0)  { dout.os() << noshrink; }
         else if (strcmp (argv[i], "-lattice") == 0)   { dout.os() << lattice; }
         else if (strcmp (argv[i], "-nolattice") == 0) { dout.os() << nolattice; }
+        else if (strcmp (argv[i], "-label") == 0)     { dout.os() << label; }
+        else if (strcmp (argv[i], "-nolabel") == 0)   { dout.os() << nolabel; }
         else if (strcmp (argv[i], "-subdivide") == 0) {
             if (i == argc-1) { std::cerr << "geo -subdivide: option argument missing" << std::endl; usage(); }
             size_t nsub = atoi(argv[++i]);
@@ -344,7 +410,12 @@ int main(int argc, char**argv) {
 	    if (format == "postscript") format = "ps";
 	    dout.os() << setimage_format(format);
         }
-
+        else if (strcmp (argv[i], "-resolution") == 0) {
+            if (i == argc-1 || !isdigit(argv[i+1][0])) { std::cerr << "geo -resolution: option argument missing" << std::endl; usage(); }
+            size_t nx = atoi(argv[++i]);
+            size_t ny = (i < argc-1 && isdigit(argv[i+1][0])) ? atoi(argv[++i]) : nx;
+	    dout.os() << setresolution(point_basic<size_t>(nx,ny));
+        }
 	// input options:
         else if (strcmp (argv[i], "-upgrade") == 0)   { do_upgrade = true; dout.os() << rheo; render = file_render; }
         else if (strcmp (argv[i], "-check") == 0)   { do_check = true; }
@@ -391,7 +462,7 @@ int main(int argc, char**argv) {
       check_macro(ips.good(), "\"" << filename << "[."<<suffix<<"[.gz]]\" not found.");
       if (do_upgrade) ips.is() >> upgrade;
       std::string root_name = delete_suffix (delete_suffix(filename, "gz"), suffix);
-      std::string name = get_basename (root_name);
+      name = get_basename (root_name);
       ips.is() >> setbasename(name);
       set_input_format (ips, input_format);
       ips >> omega;
@@ -403,8 +474,12 @@ int main(int argc, char**argv) {
     if (do_check) {
         check_macro (omega.check(bverbose), "geo check failed");
     }
-    bool show = show_min_element_measure || show_max_element_measure;
+    bool show = show_min_element_measure || show_max_element_measure || show_hmin || show_hmax || show_n_element || show_n_vertex;
     if (show) {
+      if (show_hmin)      { dout << "hmin = " << omega.hmin() << std::endl; }
+      if (show_hmax)      { dout << "hmax = " << omega.hmax() << std::endl; }
+      if (show_n_element) { dout << "size = " << omega.dis_size()  << std::endl; }
+      if (show_n_vertex)  { dout << "n_vertex  = " << omega.dis_size(0) << std::endl; }
       if (show_min_element_measure) {
         dout << "min_element_measure = " << extrema_element_measure (omega, true) << std::endl;
       }
@@ -413,6 +488,12 @@ int main(int argc, char**argv) {
       }
       return 0;
     }
-    if (render == no_render) { dout.os() << gnuplot; }
+    if (render == no_render) { 
+      if (!do_stereo) {
+         dout.os() << gnuplot;
+      } else {
+         dout.os() << paraview;
+      }
+    }
     dout << omega;
 }
