@@ -83,7 +83,7 @@ adapt_bamg (
   // --------------------------------------------
   string bb_name = i_name + "-crit.bb";
   command = "field -noverbose -bamg-bb " + crit_name + " > " + bb_name;
-  if (do_verbose) clog << "! " << command << endl;
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: unix command failed");
   // ----------------------------------------
   // 3) run bamg in adapt mode:
@@ -121,8 +121,8 @@ adapt_bamg (
     + options
     + " -b " + bamg_name
     + " -Mbb " + bb_name
-    + " -o " + i1_name + ".bamg";
-  if (do_verbose) clog << "! " << command << endl;
+    + " -o " + i1_name + ".bamg 1>&2";
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: command failed");
   // ----------------------------------------
   // 4) conversion : nom-crit-<i>.bamg
@@ -131,7 +131,7 @@ adapt_bamg (
   command = "bamg2geo -" + omega.coordinate_system_name()
             + " " + dmn_name + " " + i1_name + ".bamg | gzip -9 > "
             + i1_name + ".geo.gz";
-  if (do_verbose) clog << "! " << command << endl;
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: command failed");
   // ----------------------------------------
   // 5) chargement  nom-<i+1>.geo
