@@ -67,7 +67,7 @@ adapt_gmsh (
   // 1b) conversion geo : nom-<i>.msh
   // ----------------------------------------
   command = "geo -noverbose -gmsh " + geo_name + " > " + i_name + ".msh";
-  if (do_verbose) clog << "! " << command << endl;
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: unix command failed");
 #endif // TO_CLEAN
   // -----------------------------------------
@@ -83,7 +83,7 @@ adapt_gmsh (
   // --------------------------------------------
   string pos_name = i_name + "-crit.pos";
   command = "field -noverbose -gmsh-pos " + crit_name + " > " + pos_name;
-  if (do_verbose) clog << "! " << command << endl;
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: unix command failed");
   // ----------------------------------------
   // 3) run gmsh in adapt mode with bamg:
@@ -102,13 +102,13 @@ adapt_gmsh (
           + " -" + itos(d) + " -algo " + algo
           + " -bgm " + pos_name
           + " -format msh -o " + i1_name + ".msh";
-  if (do_verbose) clog << "! " << command << endl;
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: command failed");
   // ----------------------------------------
   // 4) conversion : nom-crit-<i>.msh
   // ----------------------------------------
   command = "msh2geo " + i1_name + ".msh | geo -upgrade - -geo | gzip -9 > " + i1_name + ".geo.gz";
-  if (do_verbose) clog << "! " << command << endl;
+  if (do_verbose) derr << "! " << command << endl;
   check_macro (dis_system (command) == 0, "adapt: command failed");
   // ----------------------------------------
   // 5) chargement  nom-<i+1>.geo
