@@ -93,30 +93,36 @@ branch_basic<T,M>::get_header (idiststream& in)
 }
 template <class T, class M>
 idiststream& 
-operator>> (idiststream& in, branch_basic<T,M>& b)
+operator>> (idiststream& in, branch_basic<T,M>& b) 
 {
-    b.get_header (in);
-    if (!in || !dis_scatch(in, b._parameter_name)) {
+  b.get (in);
+  return in;
+}
+template <class T, class M>
+void
+branch_basic<T,M>::get (idiststream& in)
+{
+    get_header (in);
+    if (!in || !dis_scatch(in, "#"+parameter_name())) {
 	// no more events
-	return in;
+	return;
     }
     T value;
     in >> value;
     if (!in) {
 	// no more events...
-	return in;
+	return;
     }
-    b._parameter_value = value;
-    for (typename branch_basic<T,M>::size_type i = 0; in && i < b.size(); i++) {
-       in >> catchmark (b[i].first) >> b[i].second;
+    set_parameter(value);
+    for (typename branch_basic<T,M>::size_type i = 0; in && i < base::size(); i++) {
+       in >> catchmark (base::operator[](i).first) >> base::operator[](i).second;
     }
     // increment counter
-    if (b._count_value == numeric_limits<typename branch_basic<T,M>::size_type>::max()) {
-        b._count_value = 0;
+    if (_count_value == numeric_limits<typename branch_basic<T,M>::size_type>::max()) {
+        _count_value = 0;
     } else {
-        b._count_value++;
+        _count_value++;
     }
-    return in;
 }
 template <class T, class M>
 void 
@@ -192,9 +198,15 @@ branch_basic<T,M>::put_finalize (odiststream& out) const
 // ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
-template class branch_basic<Float,sequential>;
+#define rheolef_instanciate(T,M) \
+  template class branch_basic<T,M>; \
+  template odiststream& operator<< (odiststream&, const branch_basic<T,M>&); \
+  template idiststream& operator>> (idiststream&, branch_basic<T,M>&); 
+
+rheolef_instanciate(Float,sequential)
 #ifdef _RHEOLEF_HAVE_MPI
-template class branch_basic<Float,distributed>;
+rheolef_instanciate(Float,distributed)
 #endif // _RHEOLEF_HAVE_MPI
+#undef rheolef_instanciate
 
 }// namespace rheolef
