@@ -22,6 +22,7 @@
 /// =========================================================================
 
 #include "rheolef/field.h"
+
 namespace rheolef { 
 
 template <class T, class M> class __obranch;
@@ -79,10 +80,20 @@ public :
 // allocators:
 
   branch_basic ();
-  branch_basic (const std::string& parameter_name, const std::string& u0);
-  branch_basic (const std::string& parameter_name, const std::string& u0, const std::string& u1);
   ~branch_basic();
 
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+  template <typename... Args>
+  branch_basic(const std::string& parameter, Args... names);
+#else // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+  branch_basic (const std::string& parameter_name, const std::string& u0);
+  branch_basic (const std::string& parameter_name, const std::string& u0,
+                                                   const std::string& u1);
+  branch_basic (const std::string& parameter_name, const std::string& u0,
+                                                   const std::string& u1,
+                                                   const std::string& u2);
+#endif // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+
 // accessors:
 
   const T&  parameter () const;
@@ -99,11 +110,25 @@ public :
 // input/output:
 
   // get/set current value
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+  template <typename... Args>
+  __obranch<T,M>  operator()  (const T& t, const field_basic<T,M>& u0, Args... uk);
+  template <typename... Args>
+  __iobranch<T,M> operator()  (T& t,             field_basic<T,M>& u0, Args&... uk);
+#else // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
   __obranch<T,M> operator()  (const T& t, const field_basic<T,M>& u0);
-  __obranch<T,M> operator()  (const T& t, const field_basic<T,M>& u0, const field_basic<T,M>& u1);
-  
+  __obranch<T,M> operator()  (const T& t, const field_basic<T,M>& u0,
+                                          const field_basic<T,M>& u1);
+  __obranch<T,M> operator()  (const T& t, const field_basic<T,M>& u0,
+                                          const field_basic<T,M>& u1,
+                                          const field_basic<T,M>& u2);
   __iobranch<T,M> operator() (T& t, field_basic<T,M>& u0);
-  __iobranch<T,M> operator() (T& t, field_basic<T,M>& u0, field_basic<T,M>& u1);
+  __iobranch<T,M> operator() (T& t, field_basic<T,M>& u0,
+                                    field_basic<T,M>& u1);
+  __iobranch<T,M> operator() (T& t, field_basic<T,M>& u0,
+                                    field_basic<T,M>& u1,
+                                    field_basic<T,M>& u2);
+#endif // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
 
   __branch_header<T,M>         header ();
   __const_branch_header<T,M>   header () const;
@@ -114,13 +139,11 @@ public :
 // implementation
 
 public:
+  void get (idiststream&);
   void put_header (odiststream&) const;
   void get_header (idiststream&);
   void put_finalize (odiststream&) const;
 
-  template <class T1, class M1> friend idiststream& operator>> (idiststream&, branch_basic<T1,M1>&);
-  template <class T1, class M1> friend odiststream& operator<< (odiststream&, const branch_basic<T1,M1>&);
-
   template <class T1, class M1> friend class  __obranch;
   template <class T1, class M1> friend class __iobranch;
 
@@ -154,6 +177,15 @@ public:
   void put_header_paraview    (odiststream&) const;
   void put_event_paraview     (odiststream&) const;
   void put_finalize_paraview  (odiststream&) const;
+private:
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+  template <typename... Args>
+  void fill_field_names (const std::string &field_name, Args... others);
+  void fill_field_names () {}
+  template <typename... Args>
+  void fill_field_values (std::size_t i, const field_basic<T,M>& ui, Args... uk);
+  void fill_field_values (std::size_t i) {}
+#endif // _RHEOLEF_HAVE_VARIADIC_TEMPLATE
 // data:
 protected:
   std::string           _parameter_name;
@@ -165,8 +197,12 @@ protected:
   mutable bool          _header_in_done;
   mutable bool          _header_out_done;
   mutable bool          _finalize_out_done;
+  mutable std::string   _to_clean;
   std::pair<T,T>        _u_range;
 };
+template <class T, class M> idiststream& operator>> (idiststream&, branch_basic<T,M>&);
+template <class T, class M> odiststream& operator<< (odiststream&, const branch_basic<T,M>&);
+
 template <class T, class M>
 idiststream& operator>> (idiststream&, branch_basic<T,M>&);
 
@@ -191,13 +227,43 @@ branch_basic<T,M>::branch_basic ()
    _header_in_done     (false),
    _header_out_done    (false),
    _finalize_out_done  (false),
+   _to_clean(),
    _u_range(std::pair<T,T>(std::numeric_limits<T>::min(),
                            std::numeric_limits<T>::max()))
 {
 }
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+template <class T, class M>
+    template <typename... Args>
+inline branch_basic<T,M>::branch_basic(const std::string& parameter_name, Args... field_names) 
+ : std::vector<std::pair<std::string,field_basic<T,M> > >(0),
+   _parameter_name (parameter_name),
+   _parameter_value(std::numeric_limits<Float>::max()),
+   _n_value        (std::numeric_limits<size_type>::max()),
+   _count_value    (std::numeric_limits<size_type>::max()),
+   _p_data_out	   (0),
+   _p_ctrl_out	   (0),
+   _header_in_done     (false),
+   _header_out_done    (false),
+   _finalize_out_done  (false),
+   _to_clean(),
+   _u_range(std::pair<T,T>(std::numeric_limits<T>::min(),
+                           std::numeric_limits<T>::max()))
+{
+    check_macro (parameter_name.length() != 0, "empty parameter name not allowed");
+    fill_field_names (field_names...);
+}
+template <class T, class M>
+template <typename... Args>
+inline void branch_basic<T,M>::fill_field_names (const std::string& field_name, Args... others) {
+    check_macro (field_name.length() != 0, "empty field name not allowed");
+    base::push_back (std::make_pair(field_name, field_basic<T,M>()));
+    fill_field_names (others...);
+}
+#else // !_RHEOLEF_HAVE_VARIADIC_TEMPLATE
 template <class T, class M>
 inline
-branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::string& field_name)
+branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::string& u0)
  : std::vector<std::pair<std::string,field_basic<T,M> > >(1),
    _parameter_name (parameter_name),
    _parameter_value(std::numeric_limits<T>::max()),
@@ -208,16 +274,18 @@ branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::s
    _header_in_done     (false),
    _header_out_done    (false),
    _finalize_out_done  (false),
+   _to_clean(),
    _u_range(std::pair<T,T>(std::numeric_limits<T>::min(),
                            std::numeric_limits<T>::max()))
 {
     check_macro(parameter_name.length() != 0, "empty parameter name not allowed");
-    check_macro(field_name.length() != 0, "empty field name not allowed");
-    base::operator[](0).first = field_name;
+    check_macro(u0.length() != 0, "empty field name not allowed");
+    base::operator[](0).first = u0;
 }
 template <class T, class M>
 inline
-branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::string& u0, const std::string& u1)
+branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::string& u0,
+                                                                    const std::string& u1)
  : std::vector<std::pair<std::string,field_basic<T,M> > >(2),
    _parameter_name (parameter_name),
    _parameter_value(std::numeric_limits<T>::max()),
@@ -228,6 +296,7 @@ branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::s
    _header_in_done     (false),
    _header_out_done    (false),
    _finalize_out_done  (false),
+   _to_clean(),
    _u_range(std::pair<T,T>(std::numeric_limits<T>::min(),
                            std::numeric_limits<T>::max()))
 {
@@ -237,6 +306,34 @@ branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::s
     base::operator[](0).first = u0;
     base::operator[](1).first = u1;
 }
+template <class T, class M>
+inline
+branch_basic<T,M>::branch_basic (const std::string& parameter_name, const std::string& u0,
+                                                                    const std::string& u1,
+                                                                    const std::string& u2)
+ : std::vector<std::pair<std::string,field_basic<T,M> > >(3),
+   _parameter_name (parameter_name),
+   _parameter_value(std::numeric_limits<T>::max()),
+   _n_value        (std::numeric_limits<size_type>::max()),
+   _count_value    (std::numeric_limits<size_type>::max()),
+   _p_data_out	   (0),
+   _p_ctrl_out	   (0),
+   _header_in_done     (false),
+   _header_out_done    (false),
+   _finalize_out_done  (false),
+   _to_clean(),
+   _u_range(std::pair<T,T>(std::numeric_limits<T>::min(),
+                           std::numeric_limits<T>::max()))
+{
+    check_macro(parameter_name.length() != 0, "empty parameter name not allowed");
+    check_macro(u0.length() != 0, "empty #1 field name not allowed");
+    check_macro(u1.length() != 0, "empty #2 field name not allowed");
+    check_macro(u2.length() != 0, "empty #3 field name not allowed");
+    base::operator[](0).first = u0;
+    base::operator[](1).first = u1;
+    base::operator[](2).first = u2;
+}
+#endif // !_RHEOLEF_HAVE_VARIADIC_TEMPLATE
 // -----------------------------------------------------------------------------
 // accessors
 // -----------------------------------------------------------------------------
@@ -367,13 +464,37 @@ class __obranch {
 template <class T, class M>
 class __iobranch {
     public:
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+	template <typename... Args>
         __iobranch (odiststream& (*put)(odiststream&, const branch_basic<T,M>&), 
 	            idiststream& (*get)(idiststream&, branch_basic<T,M>&),
 		    branch_basic<T,M>& x,
-		    T*                 pt = 0, 
-		    field_basic<T,M>*  pu = 0, 
-		    field_basic<T,M>*  pp = 0)
-         : _put(put), _get(get), _px(&x), _pt(pt), _pu(pu), _pp(pp) {}
+		    T&                 t,
+		    field_basic<T,M>&  u0, 
+                    Args&...           uk)
+         : _put(put), _get(get), _px(&x), _pt(&t), _pu()
+         {
+           std::size_t n = sizeof...(Args)+1;
+           _pu.resize(n);
+	   _pu[0] = &u0;
+           fill_field_references (1, uk...);
+         }
+	template <typename... Args>
+        void fill_field_references (std::size_t i, field_basic<T,M>& ui, Args&... uk) {
+           _pu [i] = &ui;
+           fill_field_references (i+1, uk...);
+        }
+        void fill_field_references (std::size_t i) {}
+#else // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+        __iobranch (odiststream& (*put)(odiststream&, const branch_basic<T,M>&), 
+	            idiststream& (*get)(idiststream&, branch_basic<T,M>&),
+		    branch_basic<T,M>& x,
+		    T*                 pt  = 0, 
+		    field_basic<T,M>*  pu0 = 0, 
+		    field_basic<T,M>*  pu1 = 0,
+		    field_basic<T,M>*  pu2 = 0)
+         : _put(put), _get(get), _px(&x), _pt(pt), _pu0(pu0), _pu1(pu1), _pu2(pu2) {}
+#endif // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
         template <class T1, class M1>
 	friend odiststream& operator<< (odiststream& os, __iobranch<T1,M1> m);
         template <class T1, class M1>
@@ -383,8 +504,13 @@ class __iobranch {
 	idiststream&    (*_get) (idiststream&, branch_basic<T,M>&);
         branch_basic<T,M> *_px;
         T                 *_pt;
-        field_basic<T,M>  *_pu;
-        field_basic<T,M>  *_pp;
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+        std::vector<field_basic<T,M>*>  _pu;
+#else // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+        field_basic<T,M>  *_pu0;
+        field_basic<T,M>  *_pu1;
+        field_basic<T,M>  *_pu2;
+#endif // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
 };
 template <class T, class M>
 inline
@@ -392,61 +518,142 @@ odiststream& operator<< (odiststream& os, __iobranch<T,M> m) {
     m._put (os, *m._px);
     return os;
 }
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
 template <class T, class M>
 inline
 idiststream& operator>> (idiststream& is, __iobranch<T,M> m) { 
     m._get (is, *m._px); 
-    if (m._pt) { *m._pt = (*m._px).parameter(); }
-    if (m._pu) { *m._pu = (*m._px)[0].second; }
-    if (m._pp) { *m._pp = (*m._px)[1].second; }
+    if (m._pt)  { *m._pt  = (*m._px).parameter(); }
+    for (std::size_t i = 0, n = (*m._px).size(); i < n; ++i) {
+      *(m._pu[i]) = (*m._px)[i].second;
+    }
     return is;
 }
+#else // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+template <class T, class M>
+inline
+idiststream& operator>> (idiststream& is, __iobranch<T,M> m) { 
+    m._get (is, *m._px); 
+    if (m._pt)  { *m._pt  = (*m._px).parameter(); }
+    if (m._pu0) { *m._pu0 = (*m._px)[0].second; }
+    if (m._pu1) { *m._pu1 = (*m._px)[1].second; }
+    if (m._pu2) { *m._pu2 = (*m._px)[2].second; }
+    return is;
+}
+#endif // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+
+#ifdef _RHEOLEF_HAVE_VARIADIC_TEMPLATE
+template <class T, class M>
+template <typename... Args>
+inline 
+__obranch<T,M> 
+branch_basic<T,M>::operator()  (const T& t, const field_basic<T,M>& u0, Args... uk) {
+    check_macro (base::size() >= sizeof...(Args)+1, "attempt to output a " << (sizeof...(Args)+1) <<"-field branch when a " 
+            << base::size() << "-field one was supplied");
+    _parameter_value = t;
+    base::operator[](0).second = u0;
+    fill_field_values (1, uk...);
+    return __obranch<T,M> (operator<<, *this);
+}
+template <class T, class M>
+template <typename... Args>
+inline
+__iobranch<T,M>
+branch_basic<T,M>::operator() (T& t, field_basic<T,M>& u0, Args&... uk) {
+    check_macro (base::size() >= sizeof...(Args)+1, "attempt to output a " << (sizeof...(Args)+1) <<"-field branch when a " 
+            << base::size() << "-field one was supplied");
+    _parameter_value = t;
+    base::operator[](0).second = u0;
+    fill_field_values (1, uk...);
+    return __iobranch<T,M> (operator<<, operator>>, *this, t, u0, uk...);
+}
+template <class T, class M>
+template <typename... Args>
+inline
+void branch_basic<T,M>::fill_field_values(std::size_t id, const field_basic<T,M>& u_id, Args... uk) {
+    base::operator[](id++).second = u_id;
+    fill_field_values(id, uk...);
+}
+#else // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
 template <class T, class M>
 inline
 __obranch<T,M>
-branch_basic<T,M>::operator() (const T& t, const field_basic<T,M>& u)
+branch_basic<T,M>::operator() (const T& t, const field_basic<T,M>& u0)
 {
     check_macro (base::size() >= 1, "attempt to output a 1-field branch when a " 
 	<< base::size() << "-field one was supplied");
     _parameter_value = t;
-    base::operator[](0).second = u;
+    base::operator[](0).second = u0;
     return __obranch<T,M> (operator<<, *this);
 }
 template <class T, class M>
 inline
 __iobranch<T,M>
-branch_basic<T,M>::operator() (T& t, field_basic<T,M>& u)
+branch_basic<T,M>::operator() (T& t, field_basic<T,M>& u0)
 {
     check_macro (base::size() >= 1, "attempt to input/output a 1-field branch when a " 
 	<< base::size() << "-field one was supplied");
     _parameter_value = t;
-    base::operator[](0).second = u;
-    return __iobranch<T,M> (operator<<, operator>>, *this, &t, &u);
+    base::operator[](0).second = u0;
+    return __iobranch<T,M> (operator<<, operator>>, *this, &t, &u0);
 }
 template <class T, class M>
 inline
 __obranch<T,M>
-branch_basic<T,M>::operator() (const T& t, const field_basic<T,M>& u, const field_basic<T,M>& p)
+branch_basic<T,M>::operator() (const T& t, const field_basic<T,M>& u0,
+                                           const field_basic<T,M>& u1)
 {
     check_macro (base::size() >= 2, "attempt to output a 2-field branch when a " 
 	<< base::size() << "-field one was supplied");
     _parameter_value = t;
-    base::operator[](0).second = u;
-    base::operator[](1).second = p;
+    base::operator[](0).second = u0;
+    base::operator[](1).second = u1;
     return __obranch<T,M> (operator<<, *this);
 }
 template <class T, class M>
 inline
 __iobranch<T,M>
-branch_basic<T,M>::operator() (T& t, field_basic<T,M>& u, field_basic<T,M>& p)
+branch_basic<T,M>::operator() (T& t, field_basic<T,M>& u0,
+                                     field_basic<T,M>& u1)
 {
     check_macro (base::size() >= 2, "attempt to output a 2-field branch when a " 
 	<< base::size() << "-field one was supplied");
     _parameter_value = t;
-    base::operator[](0).second = u;
-    base::operator[](1).second = p;
-    return __iobranch<T,M> (operator<<, operator>>, *this, &t, &u, &p);
+    base::operator[](0).second = u0;
+    base::operator[](1).second = u1;
+    return __iobranch<T,M> (operator<<, operator>>, *this, &t, &u0, &u1);
 }
+template <class T, class M>
+inline
+__obranch<T,M>
+branch_basic<T,M>::operator() (const T& t, const field_basic<T,M>& u0,
+                                           const field_basic<T,M>& u1,
+                                           const field_basic<T,M>& u2)
+{
+    check_macro (base::size() >= 3, "attempt to output a 3-field branch when a " 
+	<< base::size() << "-field one was supplied");
+    _parameter_value = t;
+    base::operator[](0).second = u0;
+    base::operator[](1).second = u1;
+    base::operator[](2).second = u2;
+    return __obranch<T,M> (operator<<, *this);
+}
+template <class T, class M>
+inline
+__iobranch<T,M>
+branch_basic<T,M>::operator() (T& t, field_basic<T,M>& u0,
+                                     field_basic<T,M>& u1,
+                                     field_basic<T,M>& u2)
+{
+    check_macro (base::size() >= 3, "attempt to output a 3-field branch when a " 
+	<< base::size() << "-field one was supplied");
+    _parameter_value = t;
+    base::operator[](0).second = u0;
+    base::operator[](1).second = u1;
+    base::operator[](2).second = u2;
+    return __iobranch<T,M> (operator<<, operator>>, *this, &t, &u0, &u1, &u2);
+}
+#endif // ! _RHEOLEF_HAVE_VARIADIC_TEMPLATE
 
 }// namespace rheolef
 #endif // define_RHEO_BRANCH_H
