@@ -295,8 +295,7 @@ characteristic_rep<T,M>::get_pre_computed (
   // get a good quadrature if not set
   typedef typename space_basic<T,M>::size_type size_type;
   quadrature_option_type qopt = qopt1;
-  if (qopt.get_order() == 0 ||
-      qopt.get_order() == std::numeric_limits<quadrature_option_type::size_type>::max()) {
+  if (qopt.get_order() == std::numeric_limits<quadrature_option_type::size_type>::max()) {
     size_type quad_order = Xh.get_numbering().get_basis().degree();
     if (Xh.get_geo().coordinate_system() != space_constant::cartesian) quad_order++;
     qopt.set_order (quad_order);
