@@ -508,6 +508,8 @@ public:
     typedef domain_indirect_rep<distributed>  rep;
     typedef smart_pointer<rep>            base;
     typedef rep::size_type                size_type;
+    typedef rep::iterator_ioige           iterator_ioige;
+    typedef rep::const_iterator_ioige     const_iterator_ioige;
 
 // allocators:
 
@@ -535,6 +537,11 @@ public:
 
 // distributed specific acessors:
 
+    const_iterator_ioige ioige_begin() const;
+    const_iterator_ioige ioige_end()   const;
+          iterator_ioige ioige_begin();
+          iterator_ioige ioige_end();
+
     const distributor& ini_ownership() const;
     size_type ioige2ini_dis_ioige (size_type ioige) const;
     size_type ini_ioige2dis_ioige (size_type ini_ioige) const;
@@ -631,6 +638,30 @@ domain_indirect_basic<distributed>::oige (size_type ioige) const
   return base::data().oige (ioige);
 }
 inline
+domain_indirect_basic<distributed>::const_iterator_ioige
+domain_indirect_basic<distributed>::ioige_begin() const
+{
+  return base::data().ioige_begin();
+}
+inline
+domain_indirect_basic<distributed>::const_iterator_ioige
+domain_indirect_basic<distributed>::ioige_end() const
+{
+  return base::data().ioige_end();
+}
+inline
+domain_indirect_basic<distributed>::iterator_ioige
+domain_indirect_basic<distributed>::ioige_begin()
+{
+  return base::data().ioige_begin();
+}
+inline
+domain_indirect_basic<distributed>::iterator_ioige
+domain_indirect_basic<distributed>::ioige_end()
+{
+  return base::data().ioige_end();
+}
+inline
 const distributor&
 domain_indirect_basic<distributed>::ini_ownership() const
 {
