@@ -28,25 +28,19 @@
 namespace rheolef {
 
 // ----------------------------------------------------------------------------
-// @brief build a domain from a list of element indexes
+// @brief init domain ios from the whole mesh ios data
 // ----------------------------------------------------------------------------
-template<class T>
-domain_indirect_rep<distributed>::domain_indirect_rep (
-    const geo_basic<T,distributed>& omega,
-    const std::string&            name,
-    size_type                     map_dim,
-    const communicator&           comm,
-    const std::vector<size_type>& ie_list)
- : domain_indirect_base_rep<distributed>(name, map_dim, comm, ie_list),
-   _ioige2ini_dis_ioige(),
-   _ini_ioige2dis_ioige()
+template <class T>
+void
+domain_indirect_rep<distributed>::init_ios (
+    const geo_basic<T,distributed>& omega)
 {
   // compute i/o _ioige2ini_dis_ioige & _ioige2ini_dis_ioige numbering tables
   // --------------------------------------------
   // 1) mark used elements in omega ios numbering
   // --------------------------------------------
   const size_type unset = std::numeric_limits<size_type>::max();
-  distributor ios_ownership = omega.ios_sizes().ownership_by_dimension [map_dim];
+  distributor ios_ownership = omega.ios_sizes().ownership_by_dimension [base::_map_dim];
   array<size_type,distributed> ios_mark (ios_ownership, unset);
   distributor dom_ownership = base::ownership();
   size_type first_dom_dis_ie = dom_ownership.first_index();
@@ -54,7 +48,7 @@ domain_indirect_rep<distributed>::domain_indirect_rep (
   for (const_iterator_ioige iter = base::ioige_begin(), last = base::ioige_end();
 	iter != last; iter++, dom_ie++) {
     size_type ie = (*iter).index();
-    size_type ios_dis_ie = omega.ige2ios_dis_ige (map_dim, ie);
+    size_type ios_dis_ie = omega.ige2ios_dis_ige (base::_map_dim, ie);
     size_type dom_dis_ie = first_dom_dis_ie + dom_ie;
     ios_mark.dis_entry (ios_dis_ie) = dom_dis_ie;
   }
@@ -69,7 +63,7 @@ domain_indirect_rep<distributed>::domain_indirect_rep (
   // --------------------------------------------
   // 3) set _ini_ioige2dis_ioige
   // --------------------------------------------
-  distributor dom_ini_ownership (distributor::decide, comm, dom_ini_size);
+  distributor dom_ini_ownership (distributor::decide, base::comm(), dom_ini_size);
   _ini_ioige2dis_ioige.resize (dom_ini_ownership, unset);
   size_type dom_ini_ie = 0;
   for (size_type ios_ie = 0, ios_ne = ios_mark.size(); ios_ie < ios_ne; ios_ie++) {
@@ -339,12 +333,8 @@ domain_indirect_rep<distributed>::put (
 
 #define _RHEOLEF_instanciation(T,M)                                     \
 template                                                                \
-domain_indirect_rep<M>::domain_indirect_rep ( 	                	\
-    const geo_basic<T,M>&         omega,				\
-    const std::string&            name,                                 \
-    size_type                     map_dim,                              \
-    const communicator&           comm,                                 \
-    const std::vector<size_type>& ie_list);
+void									\
+domain_indirect_rep<M>::init_ios (const geo_basic<T,M>&);		\
 
 _RHEOLEF_instanciation(Float,distributed)
 
