@@ -281,6 +281,7 @@ field_basic<T,M>::put_field (odiststream& ods) const
 // here, we want to specialize for M=seq & M=dist since only seq graphic is vailable yet:
 template <class T> odiststream& visu_gnuplot      (odiststream&, const field_basic<T,sequential>&);
 template <class T> odiststream& visu_vtk_mayavi   (odiststream&, const field_basic<T,sequential>&);
+template <class T> odiststream& visu_vtk_paraview (odiststream&, const field_basic<T,sequential>&);
 template <class T> odiststream& field_put_gmsh    (odiststream&, const field_basic<T,sequential>&);
 template <class T> odiststream& field_put_gmsh_pos(odiststream&, const field_basic<T,sequential>&);
 template <class T> odiststream& field_put_bamg_bb (odiststream&, const field_basic<T,sequential>&);
@@ -299,6 +300,7 @@ struct field_put<T,sequential> {
     iorheo::flag_type format = iorheo::flags(ods.os()) & iorheo::format_field;
     if (format [iorheo::gnuplot]) { return visu_gnuplot      (ods,uh); }
     if (format [iorheo::mayavi])  { return visu_vtk_mayavi   (ods,uh); }
+    if (format [iorheo::paraview]){ return visu_vtk_paraview (ods,uh); }
     if (format [iorheo::gmsh])    { return field_put_gmsh    (ods,uh); }
     if (format [iorheo::gmsh_pos]){ return field_put_gmsh_pos(ods,uh); }
     if (format [iorheo::bamg])    { return field_put_bamg_bb (ods,uh); }
