@@ -66,22 +66,22 @@ public:
 
   space_constant::valued_type valued_tag() const { return _raw_expr.valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-    _raw_expr.initialize (omega, hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+    return _raw_expr.initialize (omega, hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
     return _raw_expr.initialize (Xh);
   }
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const {
-    _raw_expr.evaluate (K, value);
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const {
+    return _raw_expr.evaluate (K, value);
   }
   void evaluate_on_side (const geo_element& K, const side_information_type& sid, std::vector<result_type>& value) const {
     _raw_expr.evaluate_on_side (K, sid, value);
   }
   template<class Result>
-  void valued_check() const {
-    _raw_expr.valued_check<Result>();
+  bool valued_check() const {
+    return _raw_expr.valued_check<Result>();
   }
 
 protected:
@@ -120,8 +120,8 @@ public:
 
 // initializators:
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-    _expr.initialize (omega, hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+    return _expr.initialize (omega, hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
     return _expr.initialize (Xh);
@@ -203,7 +203,7 @@ public:
     }
   };
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const
   {
    typedef typename details::generic_unary_traits<UnaryFunction>::template hint<typename Expr::value_type,Result>::argument_type
                      A1;
@@ -211,6 +211,7 @@ public:
     static const space_constant::valued_type argument_tag = space_constant::valued_tag_traits<A1>::value;
     evaluate_switch <This, Result, A1, argument_tag> helper;
     helper.evaluate (*this, K, value);
+    return true;
   }
   template<class Result>
   void evaluate_on_side (const geo_element& K, const side_information_type& sid, std::vector<Result>& value) const { 
@@ -219,10 +220,11 @@ public:
   }
 
   template<class Result>
-  void valued_check() const {
+  bool valued_check() const {
     typedef typename details::generic_unary_traits<UnaryFunction>::template hint<typename Expr::value_type,Result>::argument_type
                      A1;
-    if (! is_undeterminated<A1>::value) _expr.valued_check<A1>();
+    if (! is_undeterminated<A1>::value) return _expr.valued_check<A1>();
+    return true;
   }
 
 protected:
@@ -264,15 +266,14 @@ public:
 
 // initializers:
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const
   {
-    _expr1.initialize (omega, hat_x); 
-    _expr2.initialize (omega, hat_x);
+    return _expr1.initialize (omega, hat_x) &&
+           _expr2.initialize (omega, hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
-    bool is_homogeneous1 = _expr1.initialize (Xh);
-    bool is_homogeneous2 = _expr2.initialize (Xh);
-    return is_homogeneous1 && is_homogeneous2;
+    return _expr1.initialize (Xh) &&
+           _expr2.initialize (Xh);
   }
 
 // evaluators:
@@ -455,7 +456,7 @@ public:
     }
   };
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const
   {
     typedef typename details::generic_binary_traits<BinaryFunction>::template hint<
           typename Expr1::value_type
@@ -470,9 +471,10 @@ public:
     typedef field_nonlinear_expr_bf<BinaryFunction, Expr1, Expr2> This;
     evaluate_switch <This, Result, A1, first_argument_tag, A2, second_argument_tag>   eval;
     eval (*this, K, value);
+    return true;
   }
   template<class Result>
-  void valued_check() const {
+  bool valued_check() const {
     typedef typename details::generic_binary_traits<BinaryFunction>::template hint<
           typename Expr1::value_type
          ,typename Expr2::value_type
@@ -481,8 +483,10 @@ public:
           typename Expr1::value_type
          ,typename Expr2::value_type
          ,Result>::second_argument_type A2;
-    if (! is_undeterminated<A1>::value) _expr1.valued_check<A1>();
-    if (! is_undeterminated<A2>::value) _expr2.valued_check<A2>();
+    bool status = true;
+    if (! is_undeterminated<A1>::value)  status &= _expr1.valued_check<A1>();
+    if (! is_undeterminated<A2>::value)  status &= _expr2.valued_check<A2>();
+    return status;
   }
 
 protected:
@@ -516,14 +520,14 @@ public:
 
   space_constant::valued_type valued_tag() const { return _expr0.valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
 
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const;
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const;
 
   template<class ValueType>
-  void valued_check() const;
+  bool valued_check() const;
 
 // data:
 protected:
@@ -545,14 +549,15 @@ field_nonlinear_expr_dg<Expr>::field_nonlinear_expr_dg (
 }
 template<class Expr>
 inline
-void
+bool
 field_nonlinear_expr_dg<Expr>::initialize (
     const geo_basic<float_type,memory_type>& side_dom,
     const quadrature<float_type>&            quad) const
 {
-  _expr0.initialize (side_dom, quad);
-  _expr1.initialize (side_dom, quad);
+  bool status = _expr0.initialize (side_dom, quad) &&
+                _expr1.initialize (side_dom, quad);
   _omega = side_dom.get_background_geo();
+  return status;
 }
 template<class Expr>
 inline
@@ -560,26 +565,28 @@ bool
 field_nonlinear_expr_dg<Expr>::initialize (
     const space_basic<float_type,memory_type>& Xh) const
 {
-  _expr0.initialize (Xh);
-  _expr1.initialize (Xh);
+  bool status = _expr0.initialize (Xh) &&
+                _expr1.initialize (Xh);
   _omega = Xh.get_geo().get_background_geo();
-  return true;
+  return status;
 }
 template<class Expr>
 template<class ValueType>
 inline
-void 
+bool
 field_nonlinear_expr_dg<Expr>::valued_check() const 
 {
   space_constant::valued_type valued_tag = space_constant::valued_tag_traits<ValueType>::value;
-  check_macro (_expr0.valued_tag() == valued_tag,
- 	              "unexpected "<<  space_constant::valued_name(_expr0.valued_tag())
+  bool status = (_expr0.valued_tag() == valued_tag);
+  check_macro (status,
+ 	"unexpected "<<  space_constant::valued_name(_expr0.valued_tag())
         << "-valued field while a " << space_constant::valued_name(valued_tag)
         << "-valued one is expected in expression");
+  return status;
 }
 template<class Expr>
 template<class Result>
-void
+bool
 field_nonlinear_expr_dg<Expr>::evaluate (
     const geo_element&   K,
     std::vector<Result>& value) const
@@ -591,7 +598,7 @@ field_nonlinear_expr_dg<Expr>::evaluate (
   L_dis_ie0 = K.master(0);
   L_dis_ie1 = K.master(1);
   check_macro (L_dis_ie0 != std::numeric_limits<size_type>::max(),
-      "unexpected isolated mesh side K="<<K);
+      "unexpected isolated mesh side K"<<K.dis_ie()<<"="<<K);
   if (L_dis_ie1 == std::numeric_limits<size_type>::max()) {
     // K is a boundary side
     const geo_element& L0 = _omega.dis_get_geo_element (L_map_d, L_dis_ie0);
@@ -602,7 +609,7 @@ field_nonlinear_expr_dg<Expr>::evaluate (
     for (size_type loc_idof = 0, loc_ndof = value.size(); loc_idof < loc_ndof; ++loc_idof) {
       value[loc_idof] = c0*value[loc_idof];
     }
-    return;
+    return true;
   }
   // K is an internal side
   std::vector<Result> value1;
@@ -615,6 +622,7 @@ field_nonlinear_expr_dg<Expr>::evaluate (
   for (size_type loc_idof = 0, loc_ndof = value.size(); loc_idof < loc_ndof; ++loc_idof) {
     value[loc_idof] = _c0*value[loc_idof] + _c1*value1[loc_idof];
   }
+  return true;
 }
 
 } // namespace rheolef
