@@ -33,19 +33,15 @@ OVERVIEW:
 
 TODO:
   - compose2: loop on Types instead of explicitely write "int" or "T" => add point & tensor
-  - field_functor : completer
-    vraie fonction a retour general :
-          template<class T, class R>
-          operator* (arg1.., R(*f)(const point_basic<T>&));
-
-    classe-fonction a retour scalaire et sans template dans l'arg :
-          if (o.have_arg(i,v::s)) :
+  - F=classe-fonction 
+    completer, comme pour field_functor et vraies fonctions
           operator* (arg1.., const function<Float(const point_basic<Float>&)>& f);
 	si on met un template : le compilateur ne sait pas instancier
 	si on met plusieurs types de retour : ambigu, le compilateur ne sait pas choisir
 
-    rq. ce n'est pas urgent: on utilise pas les vraires fct ni les classes-fct dans les exprs 
-        et les functor sont plus generaux : variables internes, possibilite de scalaires complexes, etc
+    rq. ce n'est pas urgent: on utilise plus les classes-fct dans les exprs 
+        * les functor sont plus generaux : variables internes, possibilite de scalaires complexes, etc
+        * quand on a pas de variables internes, on utilise une vraie fonction
 
 SUMMARY:
    mixt field and nonlinear functions and operators, as:
@@ -89,6 +85,7 @@ void g_unop_base (s root, const op& o, s details, s arg, s w_arg) {
   w_arg = (w_arg != "") ? w_arg : arg;
   s x  = (!is_functor(arg)) ? "x" : "x.get_ref()";
   x = (arg == w_arg) ? x : "arg_t("+x+")";
+  s arg_x = make_const_arg(arg,"x");
   cout << "inline" << endl
        << root << "_expr<" << endl
        << "  " << root << "_expr_uf<" << endl
@@ -96,7 +93,7 @@ void g_unop_base (s root, const op& o, s details, s arg, s w_arg) {
        << "     ," << w_arg << endl
        << "    >" << endl
        << "  >" << endl
-       << add_operator(o.name) << " (const " << arg << "& x)" << endl
+       << add_operator(o.name) << " (" << arg_x << ")" << endl
        << "{" << endl
        << "  typedef " << details << "::" << o.tag << "  op_t;" << endl
        << "  typedef " << w_arg                    << "  arg_t;" << endl
@@ -144,6 +141,24 @@ void nl_unop (const op& o, const vector<s>& Class, s details) {
   nl_unop_base (o, details,
 	"field_functor<Function,Result>", 
 	"field_expr_terminal_function<Function>"); 
+
+  // op(F), F=true function, 
+  if (!o.is_operator()) {
+    // +f or -f : requires a true class instead of a function pointer (compile error)
+    cout << "template<class Result>" << endl;
+    nl_unop_base (o, details,
+	"Result(&)(const point_basic<typename float_traits<Result>::type>&)", 
+	"field_expr_terminal_function<Result(const point_basic<typename float_traits<Result>::type>&)>"); 
+  }
+  // op(F), F=function<R(Arg)>
+  bool arg_is_unique = (o.arg(1) == v::s || o.arg(1) == v::p || o.arg(1) == v::t);
+  v::type t = arg_is_unique ? o.arg(1) : v::s;
+  if (o.have_arg (1, t)) {
+    s r = valued(t,"Float");
+    nl_unop_base (o, details,
+	"function<"+r+"(const point&)>", 
+	"field_expr_terminal_function<function<"+r+"(const point&)> >"); 
+  }
 }
 // -----------------------------------------------------------------------------
 void nl_all_unops (const vector<op>& Op, const vector<s>& Class, s details) {
@@ -154,12 +169,12 @@ void nl_all_unops (const vector<op>& Op, const vector<s>& Class, s details) {
 // =====================================================================================
 // chap 1.2 : unary compose (f,expr) <==> f(expr)
 // =====================================================================================
-// TODO: use function<F> instead of function_wrapper<F> ?
 // unary compose
 void nl_compose1_base (s details, s arg, s w_arg="") {
   w_arg = (w_arg != "") ? w_arg : arg;
   s x = (!is_functor(arg)) ? "x" : "x.get_ref()";
   x   = (arg == w_arg) ? x : "arg_t("+x+")";
+  s arg_x = make_const_arg(arg,"x");
   cout << "inline" << endl
        << "field_nonlinear_expr<" << endl
        << "  field_nonlinear_expr_uf<" << endl
@@ -167,7 +182,7 @@ void nl_compose1_base (s details, s arg, s w_arg="") {
        << "   ," << w_arg << endl
        << "  >" << endl
        << ">" << endl
-       << "compose (const Function& f, const " << arg << "& x) {" << endl
+       << "compose (const Function& f, " << arg_x << ") {" << endl
        << "  typedef typename " << details << "::function_wrapper<Function>::type func_t;" << endl
        << "  typedef " << w_arg << "                        arg_t;" << endl
        << "  typedef field_nonlinear_expr_uf<func_t, arg_t> expr_t;" << endl
@@ -199,59 +214,15 @@ void nl_compose1 (const vector<s>& Class, s details) {
   nl_compose1_base (details,
         "field_functor<Function1,Result1>",
         "field_expr_terminal_function<Function1>");
-}
-#ifdef TO_CLEAN
-// =====================================================================================
-// chap 1.3 : T val = integrate (omega,f,qopt) with various f : numeric integration
-// =====================================================================================
-// TODO: use a direct procedure without P0-field allocation and without riesz that is obsolete !
-void
-numeric_integrate_init (s details) {
-  cout << "namespace " << details << "{" << endl
-       << "template <class T, class M, class Expr>" << endl
-       << "T" << endl
-       << "integrate_internal (const geo_basic<T,M>& omega, const Expr& expr, const quadrature_option_type& qopt)" << endl
-       << "{" << endl
-       << "  space_basic<T,M> Xh (omega, \"P0\");" << endl
-       << "  field_basic<T,M> lh = riesz (Xh, expr, qopt);" << endl
-       << "  return dual (1, lh);" << endl
-       << "}" << endl
-       << "} // namespace " << details << endl
-    ;
-}
-void
-numeric_integrate_base (s f, s details) {
-  cout << "T integrate (const geo_basic<T,M>& dom, " << f <<"," 
-                   << " const quadrature_option_type& qopt)" << endl
-       << "{" << endl
-       << "  return " << details << "::integrate_internal (dom, f, qopt);" << endl
-       << "}" << endl
-   ;
-}
-void
-numeric_integrate (const vector<s>& Class, s details)
-{
-  numeric_integrate_init (details);
 
-  // pointer to function:
-  // TODO: function<Float(point)> that merge true functions and class-functions, but with T=Float
-  cout << "template <class T, class M>" << endl;
-  numeric_integrate_base ("T (*f)(const point_basic<T>&)", details);
+  // f(F), F=true function, 
+  cout << "template<class Function, class Result1>" << endl;
+  nl_compose1_base (details,
+	"Result1(&)(const point_basic<typename float_traits<Result1>::type>&)", 
+	"field_expr_terminal_function<Result1(const point_basic<typename float_traits<Result1>::type>&)>"); 
 
-  // field_functor
-  cout << "template <class T, class M, class Function>" << endl;
-  numeric_integrate_base ("const field_functor<Function,T>& f", details);
-
-  // terminal class
-  for (size_t i = 0, nc = Class.size(); i < nc; i++) {
-    cout << "template <class T, class M>" << endl;
-    numeric_integrate_base ("const "+Class[i]+"<T,M>& f", details);
-  }
-  // expression:
-  cout << "template <class T, class M, class E>" << endl;
-  numeric_integrate_base ("const field_nonlinear_expr<E>& f", details);
+  // f(F), F=function<R(Arg)> : cannot be solved with templates
 }
-#endif // TO_CLEAN
 // *************************************************************************************
 // part 2 : binary operations
 // *************************************************************************************
@@ -266,6 +237,8 @@ void g_binop_base (s root, const op& o, s details, s arg1, s w_arg1, s arg2, s w
   s y  = (!is_functor(arg2)) ? "y" : "y.get_ref()";
   x = (arg1 == w_arg1) ? x : "arg1_t("+x+")";
   y = (arg2 == w_arg2) ? y : "arg2_t("+y+")";
+  s arg_x = make_const_arg(arg1,"x");
+  s arg_y = make_const_arg(arg2,"y");
   cout << "inline" << endl
        << root << "_expr<" << endl
        << "  " << root << "_expr_bf<" << endl
@@ -274,7 +247,7 @@ void g_binop_base (s root, const op& o, s details, s arg1, s w_arg1, s arg2, s w
        << "     ," << w_arg2 << endl
        << "    >" << endl
        << "  >" << endl
-       << add_operator(o.name) <<" (const " << arg1 << "& x, const " << arg2 << "& y)" << endl
+       << add_operator(o.name) <<" (" << arg_x << ", " << arg_y << ")" << endl
        << "{" << endl
        << "  typedef " << w_arg1 << " arg1_t;" << endl
        << "  typedef " << w_arg2 << " arg2_t;" << endl
@@ -291,6 +264,12 @@ void vf_binop_base (const op& o, s details, s arg1, s w_arg1, s arg2, s w_arg2="
 void vf2_binop_base (const op& o, s details, s arg1, s w_arg1, s arg2, s w_arg2="") {
        g_binop_base ("form_vf", o, details, arg1, w_arg1, arg2, w_arg2);
 }
+void nl_binop_base_sym (const op& o, s details, s tmpl, s arg1, s w_arg1, s arg2, s w_arg2="") {
+     cout << tmpl << endl;
+     nl_binop_base (o, details, arg1, w_arg1, arg2, w_arg2);
+     cout << tmpl << endl;
+     nl_binop_base (o, details, arg2, w_arg2, arg1, w_arg1);
+}
 // -----------------------------------------------------------------------------
 void g_binop_by_constant_bind (s root, const op& o, s details, size_t i, s constant, s arg, s w_arg="") {
   w_arg = (w_arg != "") ? w_arg : arg;
@@ -301,6 +280,8 @@ void g_binop_by_constant_bind (s root, const op& o, s details, size_t i, s const
   s expr = (i == 2) ? "x" : "y";
   expr   = (!is_functor(arg)) ? expr : expr+".get_ref()";
   expr   = (arg == w_arg) ? expr : "arg_t("+expr+")";
+  s arg_x = make_const_arg(arg1,"x");
+  s arg_y = make_const_arg(arg2,"y");
   cout << "inline" << endl
        << root << "_expr<" << endl
        << "  " << root << "_expr_uf<" << endl
@@ -308,7 +289,7 @@ void g_binop_by_constant_bind (s root, const op& o, s details, size_t i, s const
        << "   ," << w_arg << endl
        << "  >" << endl
        << ">" << endl
-       << add_operator(o.name) << " (const " << arg1 << "& x, const " << arg2 << "& y)" << endl
+       << add_operator(o.name) << " (" << arg_x << ", " << arg_y << ")" << endl
        << "{" << endl
        << "  typedef details::" << o.tag << "                op_t;" << endl
        << "  typedef " << constant << "                      ct_t;" << endl
@@ -397,6 +378,24 @@ void nl_binop_by_constant (const op& o, const vector<s>& Class, const vector<v::
   	valued(t,"typename scalar_traits<Result>::type"),
   	"field_functor<Function,Result>", 
   	"field_expr_terminal_function<Function>"); 
+
+      // op(F), F=true function, 
+      cout << "template<class Result>" << endl;
+      nl_binop_by_constant_bind (o, details, i,
+  	valued(t,"typename scalar_traits<Result>::type"),
+	"Result(&)(const point_basic<typename float_traits<Result>::type>&)", 
+	"field_expr_terminal_function<Result(const point_basic<typename float_traits<Result>::type>&)>"); 
+
+      // op(F), F=function<R(Arg)>
+      bool arg_is_unique = (o.arg(1) == v::s || o.arg(1) == v::p || o.arg(1) == v::t);
+      if (!o.is_operator() && (arg_is_unique || t == v::s)) {
+        // f/2 : ambiguous with ginac::ex (compile error)
+        s r = valued(t,"Float");
+        nl_binop_by_constant_bind (o, details, i,
+  	  r,
+	  "function<"+r+"(const point&)>", 
+	  "field_expr_terminal_function<function<"+r+"(const point&)> >"); 
+      }
     }
   }
 }
@@ -421,50 +420,54 @@ void nl_binop_by_field (const op& o, const vector<s>& Class, s details) {
   // -------------
   for (size_t i = 0, nc = Class.size(); i < nc; i++) {
     if (!o.is_linear) {
-      // op(CL)
-      cout << "template<class Expr, class T, class M>" << endl;
-      nl_binop_base (o, details,
+      // op(CL), op(LC)
+      nl_binop_base_sym (o, details,
+        "template<class Expr, class T, class M>",
    	Class[i]+"<T,M>", 
 	"field_expr_terminal_field<T,M>",
         "field_expr<Expr>",
         "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>");
-      // op(LC)
-      cout << "template<class Expr, class T, class M>" << endl;
-      nl_binop_base (o, details,
-        "field_expr<Expr>",
-        "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>",
-   	Class[i]+"<T,M>", 
-	"field_expr_terminal_field<T,M>");
     }
-    // op(CF)
-    cout << "template<class Function, class Result, class T, class M>" << endl;
-    nl_binop_base (o, details,
+    // op(CF), op(FC), F=field_functor
+    nl_binop_base_sym (o, details,
+        "template<class Function, class Result, class T, class M>",
    	Class[i]+"<T,M>", 
 	"field_expr_terminal_field<T,M>",
 	"field_functor<Function,Result>", 
 	"field_expr_terminal_function<Function>"); 
 
-    // op(FC)
-    cout << "template<class Function, class Result, class T, class M>" << endl;
-    nl_binop_base (o, details,
-	"field_functor<Function,Result>", 
-        "field_expr_terminal_function<Function>", 
-        Class[i]+"<T,M>",
-        "field_expr_terminal_field<T,M>");
+    // op(CF), op(FC), F=true function
+    nl_binop_base_sym (o, details,
+        "template<class Result, class T, class M>",
+   	Class[i]+"<T,M>", 
+	"field_expr_terminal_field<T,M>",
+	"Result(&)(const point_basic<T>&)", 
+	"field_expr_terminal_function<Result(const point_basic<T>&)>"); 
 
-    // op(NC)
-    cout << "template<class Expr, class T, class M>" << endl;
-    nl_binop_base (o, details,
+#ifdef TODO
+    // op(CF), F=function<R(Arg)>
+    // TODO: use arg(j), j=1,2 and check for j-th arg of op
+    bool arg_is_unique = (o.arg(1) == v::s || o.arg(1) == v::p || o.arg(1) == v::t);
+    v::type t = arg_is_unique ? o.arg(1) : v::s;
+    if (o.have_arg (1, t)) {
+      s r = valued(t,"Float");
+
+      // op(CF), F=function<R(Arg)>
+      nl_binop_base_sym (o, details,
+        "template<class M>",
+   	Class[i]+"<Float,M>", 
+	"field_expr_terminal_field<Float,M>",
+	r+"(&)(const point&)", 
+	"field_expr_terminal_function<"+r+"(const point&)>"); 
+    }
+#endif // TODO
+    // op(NC), op(CN)
+    nl_binop_base_sym (o, details,
+        "template<class Expr, class T, class M>",
 	"field_nonlinear_expr<Expr>", 
 	"", 
 	Class[i]+"<T,M>",
 	"field_expr_terminal_field<T,M>");
-    // op(CN)
-    cout << "template<class Expr, class T, class M>" << endl;
-    nl_binop_base (o, details,
-	Class[i]+"<T,M>", 
-	"field_expr_terminal_field<T,M>",
-	"field_nonlinear_expr<Expr>");
   }
   if (!o.is_linear) {
     // op(LL)
@@ -478,52 +481,45 @@ void nl_binop_by_field (const op& o, const vector<s>& Class, s details) {
   // ----------------------
   // op(XY), X,Y=L,F,N
   // ----------------------
-  // op(LF)
-  cout << "template<class Expr, class Function, class Result>" << endl;
-  nl_binop_base (o, details,
+  // op(LF), op(FL), F=field_functor
+  nl_binop_base_sym (o, details,
+        "template<class Expr, class Function, class Result>",
         "field_expr<Expr>",
         "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>",
         "field_functor<Function,Result>", 
         "field_expr_terminal_function<Function>");
 
-  // op(FL)
-  cout << "template<class Expr, class Function, class Result>" << endl;
-  nl_binop_base (o, details,
-        "field_functor<Function,Result>", 
-        "field_expr_terminal_function<Function>",
+  // op(LF), op(FL), F=true function
+  nl_binop_base_sym (o, details,
+        "template<class Expr, class Result>",
         "field_expr<Expr>",
-        "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>");
+        "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>",
+        "Result(&)(const point_basic<typename float_traits<Result>::type>&)", 
+        "field_expr_terminal_function<Result(const point_basic<typename float_traits<Result>::type>&)>");
 
-  // op(NF)
-  cout << "template<class Expr, class Function, class Result>" << endl;
-  nl_binop_base (o, details,
+  // op(NF), op(FN), F=field_functor
+  nl_binop_base_sym (o, details,
+        "template<class Expr, class Function, class Result>",
 	"field_nonlinear_expr<Expr>",
         "", 
         "field_functor<Function,Result>", 
         "field_expr_terminal_function<Function>");
 
-  // op(FN)
-  cout << "template<class Expr, class Function, class Result>" << endl;
-  nl_binop_base (o, details,
-	"field_functor<Function,Result>", 
-	"field_expr_terminal_function<Function>", 
-	"field_nonlinear_expr<Expr>");
+  // op(NF), op(FN), F=true function
+  nl_binop_base_sym (o, details,
+        "template<class Expr, class Result>",
+	"field_nonlinear_expr<Expr>",
+        "", 
+        "Result(&)(const point_basic<typename float_traits<Result>::type>&)", 
+        "field_expr_terminal_function<Result(const point_basic<typename float_traits<Result>::type>&)>");
 
-  // op(LN)
-  cout << "template<class Expr1, class Expr2>" << endl;
-  nl_binop_base (o, details,
+  // op(LN), op(NL)
+  nl_binop_base_sym (o, details,
+        "template<class Expr1, class Expr2>",
         "field_expr<Expr1>",
         "field_expr_terminal_field<typename field_expr<Expr1>::scalar_type, typename field_expr<Expr1>::memory_type>",
 	"field_nonlinear_expr<Expr2>");
 
-  // op(NL)
-  cout << "template<class Expr1, class Expr2>" << endl;
-  nl_binop_base (o, details,
-	"field_nonlinear_expr<Expr1>",
-	"",
-        "field_expr<Expr2>",
-        "field_expr_terminal_field<typename field_expr<Expr2>::scalar_type, typename field_expr<Expr2>::memory_type>");
-
   // op(FF)
   cout << "template<class Function1, class Result1, class Function2, class Result2>" << endl;
   nl_binop_base (o, details,
@@ -556,6 +552,8 @@ void nl_compose2_base (s details, s arg1, s w_arg1, s arg2, s w_arg2="") {
   s y = (!is_functor(arg2)) ? "y" : "y.get_ref()";
   x = (arg1 == w_arg1) ? x : "arg1_t("+x+")";
   y = (arg2 == w_arg2) ? y : "arg2_t("+y+")";
+  s arg_x = make_const_arg(arg1,"x");
+  s arg_y = make_const_arg(arg2,"y");
   cout << "inline" << endl
        << "field_nonlinear_expr<" << endl
        << "  field_nonlinear_expr_bf<" << endl
@@ -564,7 +562,7 @@ void nl_compose2_base (s details, s arg1, s w_arg1, s arg2, s w_arg2="") {
        << "   ," << w_arg2 << endl
        << "  >" << endl
        << ">" << endl
-       << "compose (const Function& f, const " << arg1 << "& x, const " << arg2 << "& y) {" << endl
+       << "compose (const Function& f, " << arg_x << ", " << arg_y << ") {" << endl
        << "  typedef typename " << details << "::function_wrapper<Function>::type func_t;" << endl
        << "  typedef " << w_arg1 << " arg1_t;" << endl
        << "  typedef " << w_arg2 << " arg2_t;" << endl
@@ -573,6 +571,12 @@ void nl_compose2_base (s details, s arg1, s w_arg1, s arg2, s w_arg2="") {
 			<< x << ", " << y << "));" << endl
        << "}" << endl;
 }
+void nl_compose2_base_sym (s details, s tmpl, s arg1, s w_arg1, s arg2, s w_arg2="") {
+     cout << tmpl << endl;
+     nl_compose2_base (details, arg1, w_arg1, arg2, w_arg2);
+     cout << tmpl << endl;
+     nl_compose2_base (details, arg2, w_arg2, arg1, w_arg1);
+}
 void nl_compose_bind_scalar (s details, size_t i, s scalar, s arg, s w_arg="") {
   w_arg = (w_arg != "") ? w_arg : arg;
   s binder = (i == 1) ? "binder1st" : "binder2nd";
@@ -582,6 +586,8 @@ void nl_compose_bind_scalar (s details, size_t i, s scalar, s arg, s w_arg="") {
   s expr = (i == 2) ? "x" : "y";
   expr = (!is_functor(arg)) ? expr : expr+".get_ref()";
   expr = (arg == w_arg) ? expr : "arg_t("+expr+")";
+  s arg_x = make_const_arg(arg1,"x");
+  s arg_y = make_const_arg(arg2,"y");
   cout << "inline" << endl
        << "field_nonlinear_expr<" << endl
        << "  field_nonlinear_expr_uf<" << endl
@@ -589,7 +595,7 @@ void nl_compose_bind_scalar (s details, size_t i, s scalar, s arg, s w_arg="") {
        << "   ," << w_arg << endl
        << "  >" << endl
        << ">" << endl
-       << "compose (const Function& f, const " << arg1 << "& x, const " << arg2 << "& y) {" << endl
+       << "compose (const Function& f, " << arg_x << ", " << arg_y << ") {" << endl
        << "  typedef " << w_arg << "                                      arg_t;" << endl
        << "  typedef typename promote<" << scalar << ",typename arg_t::scalar_type>::type scalar_t;" << endl
        << "  typedef Function                                             funb_t;" << endl
@@ -621,13 +627,20 @@ void nl_compose2 (const vector<s>& Class, const vector<v::type>& Types, s detail
           Class[j]+"<T,M>",
           "field_expr_terminal_field<T,M>");
       }
-      // f(sF), f(Fs)
+      // f(sF), f(Fs), F=field_functor
       cout << "template<class Function, class Function1, class Result1>" << endl;
       nl_compose_bind_scalar (details, i,
           valued(t,"typename field_expr_terminal_function<Function1>::scalar_type"),
           "field_functor<Function1,Result1>",
           "field_expr_terminal_function<Function1>");
   
+      // f(sF), f(Fs), F=true function
+      cout << "template<class Function, class Result1>" << endl;
+      nl_compose_bind_scalar (details, i,
+          valued(t,"typename scalar_traits<Result1>::type"),
+          "Result1(&)(const point_basic<typename float_traits<Result1>::type>&)",
+          "field_expr_terminal_function<Result1(const point_basic<typename float_traits<Result1>::type>&)>");
+
       // f(sL), f(Ls)
       cout << "template<class Function, class Expr>" << endl;
       nl_compose_bind_scalar (details, i,
@@ -659,49 +672,28 @@ void nl_compose2 (const vector<s>& Class, const vector<v::type>& Types, s detail
   // f(CX), X=L,F,N
   // ---------------------------
   for (size_t i = 0, nc = Class.size(); i < nc; i++) {
-    // f(CF)
-    cout << "template<class Function, class Function1, class Result1, class T, class M>" << endl;
-    nl_compose2_base (details,
+    // f(CF), f(FC)
+    nl_compose2_base_sym (details,
+        "template<class Function, class Function1, class Result1, class T, class M>",
         Class[i]+"<T,M>",
         "field_expr_terminal_field<T,M>",
         "field_functor<Function1,Result1>",
         "field_expr_terminal_function<Function1>");
-    // f(FC)
-    cout << "template<class Function, class Function1, class Result1, class T, class M>" << endl;
-    nl_compose2_base (details,
-        "field_functor<Function1,Result1>",
-        "field_expr_terminal_function<Function1>",
-        Class[i]+"<T,M>",
-        "field_expr_terminal_field<T,M>");
 
-    // f(CL)
-    cout << "template<class Function, class Expr, class T, class M>" << endl;
-    nl_compose2_base (details,
+    // f(CL), f(LC)
+    nl_compose2_base_sym (details,
+        "template<class Function, class Expr, class T, class M>",
         Class[i]+"<T,M>",
         "field_expr_terminal_field<T,M>",
         "field_expr<Expr>",
         "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>");
-    // f(LC)
-    cout << "template<class Function, class Expr, class T, class M>" << endl;
-    nl_compose2_base (details,
-        "field_expr<Expr>",
-        "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>",
-        Class[i]+"<T,M>",
-        "field_expr_terminal_field<T,M>");
 
-    // f(CN)
-    cout << "template<class Function, class Expr, class T, class M>" << endl;
-    nl_compose2_base (details,
+    // f(CN), f(NC)
+    nl_compose2_base_sym (details,
+        "template<class Function, class Expr, class T, class M>",
         Class[i]+"<T,M>",
         "field_expr_terminal_field<T,M>",
         "field_nonlinear_expr<Expr>");
-    // f(NC)
-    cout << "template<class Function, class Expr, class T, class M>" << endl;
-    nl_compose2_base (details,
-        "field_nonlinear_expr<Expr>",
-        "",
-        Class[i]+"<T,M>",
-        "field_expr_terminal_field<T,M>");
   }
   // ---------------------------
   // f(XY), X,Y=L,F,N
@@ -721,57 +713,68 @@ void nl_compose2 (const vector<s>& Class, const vector<v::type>& Types, s detail
         "",
         "field_nonlinear_expr<Expr2>");
 
-  // f(LN)
-  cout << "template<class Function, class Expr1, class Expr2>" << endl;
-  nl_compose2_base (details,
+  // f(LN), f(NC)
+  nl_compose2_base_sym (details,
+        "template<class Function, class Expr1, class Expr2>",
         "field_expr<Expr1>",
         "field_expr_terminal_field<typename field_expr<Expr1>::scalar_type, typename field_expr<Expr1>::memory_type>",
         "field_nonlinear_expr<Expr2>");
-  // f(NL)
-  cout << "template<class Function, class Expr1, class Expr2>" << endl;
-  nl_compose2_base (details,
-        "field_nonlinear_expr<Expr1>",
-        "",
-        "field_expr<Expr2>",
-        "field_expr_terminal_field<typename field_expr<Expr2>::scalar_type, typename field_expr<Expr2>::memory_type>");
 
-  // f(FF)
-  cout << "template<class Function, class Function1, class Result1, class Function2, class Result2>" << endl;
-  nl_compose2_base (details,
-        "field_functor<Function1,Result2>",
-        "field_expr_terminal_function<Function1>",
-        "field_functor<Function2,Result2>",
-        "field_expr_terminal_function<Function2>");
-
-  // f(FL)
-  cout << "template<class Function, class Function1, class Result1, class Expr>" << endl;
-  nl_compose2_base (details,
+  // f(FL), f(LF), F=field_functor
+  nl_compose2_base_sym (details,
+        "template<class Function, class Function1, class Result1, class Expr>",
         "field_functor<Function1,Result1>",
         "field_expr_terminal_function<Function1>",
         "field_expr<Expr>",
         "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>");
-  // f(LF)
-  cout << "template<class Function, class Function1, class Result1, class Expr>" << endl;
-  nl_compose2_base (details,
-        "field_expr<Expr>",
-        "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>",
-        "field_functor<Function1,Result1>",
-        "field_expr_terminal_function<Function1>");
 
-  // f(FN)
-  cout << "template<class Function, class Function1, class Result1, class Expr>" << endl;
-  nl_compose2_base (details,
+  // f(FL), f(LF), F=true function
+  nl_compose2_base_sym (details,
+        "template<class Function, class Result1, class Expr>",
+        "Result1(&)(const point_basic<typename float_traits<Result1>::type>&)",
+        "field_expr_terminal_function<Result1(const point_basic<typename float_traits<Result1>::type>&)>",
+        "field_expr<Expr>",
+        "field_expr_terminal_field<typename field_expr<Expr>::scalar_type, typename field_expr<Expr>::memory_type>");
+
+  // f(FN), f(NF), F=field_functor
+  nl_compose2_base_sym (details,
+        "template<class Function, class Function1, class Result1, class Expr>",
         "field_functor<Function1,Result1>",
         "field_expr_terminal_function<Function1>",
         "field_nonlinear_expr<Expr>",
         "field_nonlinear_expr<Expr>");
-  // f(NF)
-  cout << "template<class Function, class Function1, class Result1, class Expr>" << endl;
+
+  // f(FN), f(NF), F=true function
+  nl_compose2_base_sym (details,
+        "template<class Function, class Result1, class Expr>",
+        "Result1(&)(const point_basic<typename float_traits<Result1>::type>&)",
+        "field_expr_terminal_function<Result1(const point_basic<typename float_traits<Result1>::type>&)>",
+        "field_nonlinear_expr<Expr>",
+        "field_nonlinear_expr<Expr>");
+
+  // f(FF), F=field_functor
+  cout << "template<class Function, class Function1, class Result1, class Function2, class Result2>" << endl;
   nl_compose2_base (details,
-        "field_nonlinear_expr<Expr>",
-        "field_nonlinear_expr<Expr>",
         "field_functor<Function1,Result1>",
-        "field_expr_terminal_function<Function1>");
+        "field_expr_terminal_function<Function1>",
+        "field_functor<Function2,Result2>",
+        "field_expr_terminal_function<Function2>");
+
+  // f(FF), F=true function
+  cout << "template<class Function, class Result1, class Result2>" << endl;
+  nl_compose2_base (details,
+        "Result1(&)(const point_basic<typename float_traits<Result1>::type>&)",
+        "field_expr_terminal_function<Result1(const point_basic<typename float_traits<Result1>::type>&)>",
+        "Result2(&)(const point_basic<typename float_traits<Result2>::type>&)",
+        "field_expr_terminal_function<Result2(const point_basic<typename float_traits<Result2>::type>&)>");
+
+  // f(FG), f(GF), F=field_functor, G=true function
+  nl_compose2_base_sym (details,
+        "template<class Function, class Function1, class Result1, class Result2>",
+        "field_functor<Function1,Result1>",
+        "field_expr_terminal_function<Function1>",
+        "Result2(&)(const point_basic<typename float_traits<Result2>::type>&)",
+        "field_expr_terminal_function<Result2(const point_basic<typename float_traits<Result2>::type>&)>");
 }
 #ifndef _NO_FIELD_NONLINEAR_MAIN
 // =====================================================================================
@@ -805,7 +808,7 @@ int main() {
   Unop.push_back (op("cosh",  v::s));
   Unop.push_back (op("sinh",  v::s));
   Unop.push_back (op("tanh",  v::s));
-  Unop.push_back (op("exp",   v::s));
+  Unop.push_back (op("exp",   v::st));
   Unop.push_back (op("log",   v::s));
   Unop.push_back (op("log10", v::s));
   Unop.push_back (op("sqrt",  v::s));
@@ -837,10 +840,6 @@ int main() {
   nl_compose1 (Class,        details);
   nl_compose2 (Class, Types, details);
 
-#ifdef TO_CLEAN
-  numeric_integrate (Class, details);
-#endif // TO_CLEAN
-
   footer("FIELD_NONLINEAR", "field_nonlinear");
 }
 #endif // _NO_FIELD_NONLINEAR_MAIN
