@@ -1,4 +1,4 @@
-///
+//
 /// This file is part of Rheolef.
 ///
 /// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
@@ -59,21 +59,23 @@ field_expr_terminal_function_base_rep<T>::field_expr_terminal_function_base_rep
 {
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_base_rep<T>::initialize (
   const geo_basic<float_type,memory_type>& omega,
   const quadrature<float_type>&            quad) const
 {
   _omega = omega;
   _piola_ops.set (quad, omega.get_piola_basis());
+  return true;
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_base_rep<T>::initialize (
   const space_basic<float_type,memory_type>& Xh) const
 {
   _omega = Xh.get_geo();
   _piola_ops.set (Xh.get_numbering().get_basis(), _omega.get_piola_basis());
+  return true;
 }
 template<class T>
 void
@@ -135,7 +137,7 @@ field_expr_terminal_function_rep<normal_pseudo_function<T> >::field_expr_termina
 {
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_rep<normal_pseudo_function<T> >::initialize (
     const geo_basic<float_type,memory_type>& omega, 
     const quadrature<float_type>&            quad) const
@@ -145,6 +147,7 @@ field_expr_terminal_function_rep<normal_pseudo_function<T> >::initialize (
     // normal computation on 0D sides requires 1D bgd mesh with neighbours
     base::_omega.get_background_geo().neighbour_guard();
   }
+  return true;
 }
 template<class T>
 bool
@@ -159,7 +162,7 @@ field_expr_terminal_function_rep<normal_pseudo_function<T> >::initialize (
   return true;
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_rep<normal_pseudo_function<T> >::evaluate (
     const geo_element&        K,
     std::vector<result_type>& value) const
@@ -176,6 +179,7 @@ field_expr_terminal_function_rep<normal_pseudo_function<T> >::evaluate (
     point_basic<T> nq = normal_from_piola_transformation (base::_omega, K, DF, base::_omega.dimension());
     *iter = nq;
   }
+  return true;
 }
 template<class T>
 void
@@ -202,23 +206,22 @@ field_expr_terminal_function_rep<h_local_pseudo_function<T> >::field_expr_termin
 {
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_rep<h_local_pseudo_function<T> >::initialize (
     const geo_basic<float_type,memory_type>& omega, 
     const quadrature<float_type>&            quad) const
 {
-  base::initialize (omega, quad);
+  return base::initialize (omega, quad);
 }
 template<class T>
 bool
 field_expr_terminal_function_rep<h_local_pseudo_function<T> >::initialize (
     const space_basic<float_type,memory_type>& Xh) const
 {
-  base::initialize (Xh);
-  return true;
+  return base::initialize (Xh);
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_rep<h_local_pseudo_function<T> >::evaluate (
     const geo_element&        K,
     std::vector<result_type>& value) const
@@ -238,6 +241,7 @@ field_expr_terminal_function_rep<h_local_pseudo_function<T> >::evaluate (
     T h_loc = pow (abs(det_DF)/measure(hat_K), 1./map_d);
     *iter = h_loc;
   }
+  return true;
 }
 template<class T>
 void
@@ -264,20 +268,19 @@ field_expr_terminal_function_rep<penalty_pseudo_function<T> >::field_expr_termin
 {
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_rep<penalty_pseudo_function<T> >::initialize (
     const geo_basic<float_type,memory_type>& omega, 
     const quadrature<float_type>&            quad) const
 {
-  base::initialize (omega, quad);
+  return base::initialize (omega, quad);
 }
 template<class T>
 bool
 field_expr_terminal_function_rep<penalty_pseudo_function<T> >::initialize (
     const space_basic<float_type,memory_type>& Xh) const
 {
-  base::initialize (Xh);
-  return true;
+  return base::initialize (Xh);
 }
 template<class T>
 T
@@ -318,7 +321,7 @@ field_expr_terminal_function_rep<penalty_pseudo_function<T> >::evaluate_internal
   }
 }
 template<class T>
-void
+bool
 field_expr_terminal_function_rep<penalty_pseudo_function<T> >::evaluate (
     const geo_element&           K,
     std::vector<result_type>&    value) const
@@ -332,7 +335,7 @@ field_expr_terminal_function_rep<penalty_pseudo_function<T> >::evaluate (
     // K is a boundary side
     const geo_element& L0 = base::_omega.dis_get_geo_element (L_map_d, L_dis_ie0);
     evaluate_internal (K, L0, value);
-    return;
+    return true;
   }
   // K is an internal side
   const geo_element& L0 = base::_omega.dis_get_geo_element (L_map_d, L_dis_ie0);
@@ -343,6 +346,7 @@ field_expr_terminal_function_rep<penalty_pseudo_function<T> >::evaluate (
   for (size_type loc_idof = 0, loc_ndof = value.size(); loc_idof < loc_ndof; ++loc_idof) {
     value[loc_idof] = std::max (value [loc_idof], value1 [loc_idof]);
   }
+  return true;
 }
 template<class T>
 void
@@ -360,7 +364,7 @@ field_expr_terminal_function_rep<penalty_pseudo_function<T> >::evaluate_on_side
 // 2.1) base class for the field family
 // ----------------------------------------------------------------------------
 template<class T, class M>
-void
+bool
 field_expr_terminal_field_base_rep<T,M>::initialize (
     const geo_basic<T,M>& omega,
     const quadrature<T>&  quad) const
@@ -377,6 +381,7 @@ field_expr_terminal_field_base_rep<T,M>::initialize (
     _have_dg_on_sides = _uh.get_space().get_numbering().is_discontinuous()
                       && _omega.variant() == geo_abstract_base_rep<T>::geo_domain_indirect
                       && 1 + omega.map_dimension() == omega.get_background_geo().map_dimension();
+    return true;
 }
 template<class T, class M>
 bool
@@ -394,7 +399,7 @@ field_expr_terminal_field_base_rep<T,M>::initialize (
     _use_bgd2dom = (_omega.variant()        != geo_abstract_base_rep<T>::geo_domain &&
                     _uh.get_geo().variant() == geo_abstract_base_rep<T>::geo_domain);
     _have_dg_on_sides = _uh.get_space().get_numbering().is_discontinuous()
-                      && _omega.variant() == geo_abstract_base_rep<T>::geo_domain_indirect
+                      && (_use_dom2bgd || _omega.variant() == geo_abstract_base_rep<T>::geo_domain_indirect)
                       && 1 + _omega.map_dimension() == _omega.get_background_geo().map_dimension();
     return true;
 }
@@ -403,32 +408,43 @@ void
 field_expr_terminal_field_base_rep<T,M>::evaluate_initialize (
   const geo_element& K) const
 {
-    if (_use_bgd2dom) {        // get dom_K from background mesh:
+    if (_have_dg_on_sides && K.dimension() == _omega.map_dimension()) {
+      // omits "inner(u)" on a sides-domain => assume a boundary side
+      const geo_element* bgd_K_ptr = 0;
+      if (_use_dom2bgd) {
+        // get bgd_K on background mesh: for DG-interpolate on a boundary domain
+        bgd_K_ptr = &(_omega.dom2bgd_geo_element (K));
+      } else {
+        bgd_K_ptr = &K;
+      }
+      const geo_element& bgd_K = *bgd_K_ptr;
+      size_type L_map_d = bgd_K.dimension() + 1;
+      check_macro (bgd_K.master(1) == std::numeric_limits<size_type>::max(),
+          "unexpected non-boundary side K="<<bgd_K);
+      size_type L_dis_ie = bgd_K.master(0);
+      check_macro (L_dis_ie != std::numeric_limits<size_type>::max(),
+          "unexpected isolated side K="<<bgd_K);
+      const geo_element* L_ptr = 0;
+      if (_use_dom2bgd) {
+        L_ptr = &(_omega.get_background_geo().dis_get_geo_element (L_map_d, L_dis_ie));
+      } else {
+        L_ptr = &(_omega.dis_get_geo_element (L_map_d, L_dis_ie));
+      }
+      const geo_element& L = *L_ptr;
+      _tilde_L = L.variant();
+      side_information_type sid;
+      L.get_side_informations (bgd_K, sid);
+      _bops.restrict_on_side (_tilde_L, sid);
+      _uh.get_space().dis_idof (L, _dis_idof);
+    } else if (_use_bgd2dom) {        // get dom_K from background mesh:
       const geo_element& dom_K = _uh.get_geo().bgd2dom_geo_element (K);
       _uh.get_space().dis_idof (dom_K, _dis_idof);
     } else if (_use_dom2bgd) { // get bgd_K on background mesh:
       const geo_element& bgd_K = _omega.dom2bgd_geo_element (K);
       _uh.get_space().dis_idof (bgd_K, _dis_idof);
     } else {
-      // use K directly
-      if (_have_dg_on_sides && K.dimension() == _omega.map_dimension()) {
-        // omits "inner(u)" on a sides-domain => assume a boundary side
-        size_type L_map_d = K.dimension() + 1;
-        check_macro (K.master(1) == std::numeric_limits<size_type>::max(),
-          "unexpected non-boundary side K="<<K);
-        size_type L_dis_ie = K.master(0);
-        check_macro (L_dis_ie != std::numeric_limits<size_type>::max(),
-          "unexpected isolated side K="<<K);
-        const geo_element& L = _omega.dis_get_geo_element (L_map_d, L_dis_ie);
-        _tilde_L = L.variant();
-        side_information_type sid;
-        L.get_side_informations (K, sid);
-        _bops.restrict_on_side (_tilde_L, sid);
-        _uh.get_space().dis_idof (L, _dis_idof);
-      } else {
-        // usual case:
+      // usual case: use K directly
         _uh.get_space().dis_idof (K, _dis_idof);
-      }
     }
 }
 template<class T, class M>
@@ -453,12 +469,12 @@ field_expr_terminal_field_base_rep<T,M>::get_side (
 // 2.2) field
 // ----------------------------------------------------------------------------
 template<class T, class M>
-void
+bool
 field_expr_terminal_field_rep<T,M>::initialize (
     const geo_basic<float_type,memory_type>& omega,
     const quadrature<float_type>&            quad) const
 {
-  base::initialize (omega, quad);
+  return base::initialize (omega, quad);
 }
 template<class T, class M>
 bool
@@ -469,7 +485,7 @@ field_expr_terminal_field_rep<T,M>::initialize (
 }
 template<class T, class M>
 template<class Result>
-void
+bool
 field_expr_terminal_field_rep<T,M>::evaluate ( 
     const geo_element&   K,
     std::vector<Result>& value) const
@@ -486,6 +502,7 @@ field_expr_terminal_field_rep<T,M>::evaluate (
 	last = value.end(); iter != last; ++iter, ++q) {
       general_field_evaluate (base::_uh, base::_bops, hat_K, base::_dis_idof, q, *iter);
     }
+    return true;
 }
 template<class T, class M>
 template<class Result>
@@ -518,7 +535,7 @@ field_expr_terminal_field_grad_rep<T,M>::valued_tag() const
   } 
 }
 template<class T, class M>
-void
+bool
 field_expr_terminal_field_grad_rep<T,M>::initialize (
     const geo_basic<float_type,memory_type>& omega,
     const quadrature<float_type>&            quad) const
@@ -526,6 +543,7 @@ field_expr_terminal_field_grad_rep<T,M>::initialize (
     base::initialize (omega, quad);
     bool ignore_sys_coord = false;
     _u_test.initialize (omega, quad, ignore_sys_coord);
+    return true;
 }
 template<class T, class M>
 bool
@@ -537,10 +555,9 @@ field_expr_terminal_field_grad_rep<T,M>::initialize (
     _u_test.initialize (Xh, ignore_sys_coord);
     return true;
 }
-// works only when Result=point_basic<T> ?
 template<class T, class M>
 template<class ValueType>
-void
+bool
 field_expr_terminal_field_grad_rep<T,M>::evaluate ( 
     const geo_element&      K,
     std::vector<ValueType>& value) const
@@ -549,16 +566,21 @@ field_expr_terminal_field_grad_rep<T,M>::evaluate (
     reference_element hat_K = K.variant();
     size_type nq = _u_test.element_initialize (K);
     value.resize (nq);
-    std::vector<ValueType> grad_basis (base::_dis_idof.size());
+    size_type loc_ndof = base::_dis_idof.size();
+    std::vector<T> udof (loc_ndof);
+    for (size_type loc_idof = 0; loc_idof < loc_ndof; ++loc_idof) {
+      udof[loc_idof] = base::_uh.dis_dof (base::_dis_idof[loc_idof]);
+    }
+    std::vector<ValueType> grad_basis (loc_ndof);
     for (size_type q = 0; q != nq; ++q) {
       _u_test.grad_basis_evaluate (hat_K, q, _opt, grad_basis);
-      ValueType sum (0.);
-      for (size_type loc_idof = 0, loc_ndof = grad_basis.size(); loc_idof < loc_ndof; ++loc_idof) {
-        const T& udof = base::_uh.dis_dof (base::_dis_idof[loc_idof]);
-        sum = sum + udof*grad_basis[loc_idof];
+      ValueType sum (T(0.));
+      for (size_type loc_idof = 0; loc_idof < loc_ndof; ++loc_idof) {
+        sum = sum + udof[loc_idof]*grad_basis[loc_idof];
       }
       value[q] = sum;
     }
+    return true;
 }
 // ---------------------------------------------------------------------------
 // 2.4) div of a field
@@ -579,7 +601,7 @@ field_expr_terminal_field_div_rep<T,M>::valued_tag() const
   } 
 }
 template<class T, class M>
-void
+bool
 field_expr_terminal_field_div_rep<T,M>::initialize (
     const geo_basic<float_type,memory_type>& omega,
     const quadrature<float_type>&            quad) const
@@ -587,6 +609,7 @@ field_expr_terminal_field_div_rep<T,M>::initialize (
     base::initialize (omega, quad);
     bool ignore_sys_coord = false;
     _u_test.initialize (omega, quad, ignore_sys_coord);
+    return true;
 }
 template<class T, class M>
 bool
@@ -656,12 +679,13 @@ div_evaluate (
 }
 template<class T, class M>
 template<class ValueType>
-void
+bool
 field_expr_terminal_field_div_rep<T,M>::evaluate ( 
     const geo_element&      K,
     std::vector<ValueType>& value) const
 {
   div_evaluate (*this, K, value);
+  return true;
 }
 // ---------------------------------------------------------------------------
 // 2.5) curl of a field
@@ -683,7 +707,7 @@ field_expr_terminal_field_curl_rep<T,M>::valued_tag() const
   } 
 }
 template<class T, class M>
-void
+bool
 field_expr_terminal_field_curl_rep<T,M>::initialize (
     const geo_basic<float_type,memory_type>& omega,
     const quadrature<float_type>&            quad) const
@@ -691,6 +715,7 @@ field_expr_terminal_field_curl_rep<T,M>::initialize (
     base::initialize (omega, quad);
     bool ignore_sys_coord = false;
     _u_test.initialize (omega, quad, ignore_sys_coord);
+    return true;
 }
 template<class T, class M>
 bool
@@ -707,7 +732,7 @@ field_expr_terminal_field_curl_rep<T,M>::initialize (
 // curl(vector) -> vector, 3D
 template<class T, class M, class ValueType>
 static
-void
+bool
 curl_evaluate_aux (
     const field_expr_terminal_field_curl_rep<T,M>& obj,
     const geo_element&                             K,
@@ -728,6 +753,7 @@ curl_evaluate_aux (
       }
       value[q] = sum;
     }
+    return true;
 }
 template<class T, class M>
 static
@@ -760,39 +786,61 @@ curl_evaluate (
 {
   fatal_macro ("curl() -> tensor: unsupported");
 }
+// curl(tensor4) -> tensor3
+template<class T, class M>
+static
+void
+curl_evaluate (
+    const field_expr_terminal_field_curl_rep<T,M>& obj,
+    const geo_element&                            K,
+    std::vector<tensor3_basic<T> >&               value)
+{
+  fatal_macro ("curl() -> tensor3: unsupported");
+}
+// curl(tensor5) -> tensor4
+template<class T, class M>
+static
+void
+curl_evaluate (
+    const field_expr_terminal_field_curl_rep<T,M>& obj,
+    const geo_element&                            K,
+    std::vector<tensor4_basic<T> >&               value)
+{
+  fatal_macro ("curl() -> tensor3: unsupported");
+}
 template<class T, class M>
 template<class ValueType>
-void
+bool
 field_expr_terminal_field_curl_rep<T,M>::evaluate ( 
     const geo_element&      K,
     std::vector<ValueType>& value) const
 {
   curl_evaluate (*this, K, value);
+  return true;
 }
 // ----------------------------------------------------------------------------
 // 2.6) field jump
 // ----------------------------------------------------------------------------
 template<class T, class M>
-void
+bool
 field_expr_terminal_field_dg_rep<T,M>::initialize (
     const geo_basic<float_type,memory_type>& omega,
     const quadrature<float_type>&            quad) const
 {
-  _expr0.initialize (omega, quad);
-  _expr1.initialize (omega, quad);
+  return _expr0.initialize (omega, quad) &&
+         _expr1.initialize (omega, quad);
 }
 template<class T, class M>
 bool
 field_expr_terminal_field_dg_rep<T,M>::initialize (
     const space_basic<float_type,memory_type>& Xh) const 
 {
-  _expr0.initialize (Xh);
-  _expr1.initialize (Xh);
-  return true;
+  return _expr0.initialize (Xh) &&
+         _expr1.initialize (Xh);
 }
 template<class T, class M>
 template<class Result>
-void
+bool
 field_expr_terminal_field_dg_rep<T,M>::evaluate ( 
     const geo_element&   K,
     std::vector<Result>& value) const
@@ -815,7 +863,7 @@ field_expr_terminal_field_dg_rep<T,M>::evaluate (
     for (size_type loc_idof = 0, loc_ndof = value.size(); loc_idof < loc_ndof; ++loc_idof) {
       value[loc_idof] = c0*value[loc_idof];
     }
-    return;
+    return true;
   }
   // K is an internal side
   std::vector<Result> value1;
@@ -828,6 +876,7 @@ field_expr_terminal_field_dg_rep<T,M>::evaluate (
   for (size_type loc_idof = 0, loc_ndof = value.size(); loc_idof < loc_ndof; ++loc_idof) {
     value[loc_idof] = _c0*value[loc_idof] + _c1*value1[loc_idof];
   }
+  return true;
 }
 // ---------------------------------------------------------------------------
 // 3) contains a convected field, as compose(uh,X) where X is a characteristic
@@ -867,7 +916,7 @@ interpolate_pass2_valued (
           array<T,M>&                      ux);
 
 template<class T, class M>
-void 
+bool
 field_expr_terminal_field_o_characteristic_rep<T,M>::initialize (
     const geo_basic<float_type,memory_type>& omega,
     const quadrature<float_type>&            quad) const
@@ -895,6 +944,7 @@ field_expr_terminal_field_o_characteristic_rep<T,M>::initialize (
     _quad = coq_r._quad;
     _bops.set (coq_r._quad, _uh.get_space().get_numbering().get_basis());
     _is_initialized = true;
+    return true;
 }
 template<class T, class M>
 bool
@@ -982,7 +1032,23 @@ set_value (
   const geo_element&                  K, 
   std::vector<tensor_basic<T> >&      value)
 {
-    fatal_macro ("characteristic: not yet for tensor-valued fields");
+  typedef geo_element::size_type size_type;
+  const geo_basic<T,M>& omega = obj._uh.get_geo(); // TODO: when another destination mesh omega, different from _uh.geo
+  size_type nloc_comp_q    = obj._quad.size(K);
+  size_type comp_nq        = omega.size()*nloc_comp_q;
+  size_type n_comp         = obj._uh.size();
+  space_constant::coordinate_type sys_coord = omega.coordinate_system();
+  for (geo_element::size_type loc_comp_idof = 0, loc_comp_ndof = value.size(); loc_comp_idof < loc_comp_ndof; loc_comp_idof++) {
+    tensor_basic<T>& vi = value [loc_comp_idof];
+    for (size_type ij_comp = 0; ij_comp < n_comp; ij_comp++) {
+      size_type q = obj._start_q + loc_comp_idof + ij_comp*comp_nq;
+      std::pair<size_type,size_type> ij
+          = space_constant::tensor_subscript (space_constant::tensor, sys_coord, ij_comp);
+      vi(ij.first,ij.second) = obj._uq_K [q];
+      if (ij.first != ij.second) { vi(ij.second,ij.first) = vi(ij.first,ij.second); }
+    }
+  }
+  obj._start_q += nloc_comp_q;
 }
 // tensor3-valued case:
 template<class T, class M>
@@ -1008,7 +1074,7 @@ set_value (
 }
 template<class T, class M>
 template<class Result>
-void
+bool
 field_expr_terminal_field_o_characteristic_rep<T,M>::evaluate (
     const geo_element&   K, 
     std::vector<Result>& value) const
@@ -1016,6 +1082,7 @@ field_expr_terminal_field_o_characteristic_rep<T,M>::evaluate (
     reference_element hat_K = K.variant();
     value.resize (_bops.size(hat_K));
     set_value (*this, K, value);
+    return true;
 }
 // ----------------------------------------------------------------------------
 // instanciation in library
@@ -1028,23 +1095,23 @@ template class field_expr_terminal_function_rep<penalty_pseudo_function<T> >; \
 
 // extra when Result=tensor3, tensor4
 #define _RHEOLEF_instanciation_base_both_members_full(T,M,Result) 	\
-template void field_expr_terminal_field_rep<T,M>::evaluate<Result > ( 	\
+template bool field_expr_terminal_field_rep<T,M>::evaluate<Result > ( 	\
     		const geo_element&, std::vector<Result >&) const;	\
-template void field_expr_terminal_field_grad_rep<T,M>::evaluate<Result > ( \
+template bool field_expr_terminal_field_grad_rep<T,M>::evaluate<Result > ( \
     		const geo_element&, std::vector<Result >&) const;	\
 template void field_expr_terminal_field_rep<T,M>::evaluate_on_side<Result > ( 	\
     		const geo_element& K, const side_information_type& sid, 	\
 		std::vector<Result>& value) const;			\
-template void field_expr_terminal_field_o_characteristic_rep<T,M>::evaluate<Result > ( 	\
+template bool field_expr_terminal_field_o_characteristic_rep<T,M>::evaluate<Result > ( 	\
+    		const geo_element&, std::vector<Result >&) const;	\
+template bool field_expr_terminal_field_curl_rep<T,M>::evaluate<Result > ( \
     		const geo_element&, std::vector<Result >&) const;	\
 
 #define _RHEOLEF_instanciation_base_both_members_mini(T,M,Result) 	\
 	_RHEOLEF_instanciation_base_both_members_full(T,M,Result) 	\
-template void field_expr_terminal_field_div_rep<T,M>::evaluate<Result > ( \
+template bool field_expr_terminal_field_div_rep<T,M>::evaluate<Result > ( \
     		const geo_element&, std::vector<Result >&) const;	\
-template void field_expr_terminal_field_curl_rep<T,M>::evaluate<Result > ( \
-    		const geo_element&, std::vector<Result >&) const;	\
-template void field_expr_terminal_field_dg_rep<T,M>::evaluate<Result > ( \
+template bool field_expr_terminal_field_dg_rep<T,M>::evaluate<Result > ( \
     		const geo_element&, std::vector<Result >&) const;	\
 
 #define _RHEOLEF_instanciation_base_both(T,M) 				\
