@@ -77,8 +77,8 @@ public:
 
 // accessors:
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
-  void initialize (const space_basic<float_type,memory_type>& Xh) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
+  bool initialize (const space_basic<float_type,memory_type>& Xh) const;
   void evaluate_initialize (const geo_element& K) const;
   const geo_element& get_side (const geo_element& K, const side_information_type& sid) const;
 
@@ -119,14 +119,13 @@ public:
   space_constant::valued_type valued_tag() const {
       return space_constant::valued_tag_traits<result_type>::value; }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const {
-    base::initialize (omega, quad);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const {
+    return base::initialize (omega, quad);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
-    base::initialize (Xh);
-    return true;
+    return base::initialize (Xh);
   }
-  void evaluate (const geo_element& K, std::vector<result_type>& value) const {
+  bool evaluate (const geo_element& K, std::vector<result_type>& value) const {
     base::evaluate_initialize (K);
     reference_element hat_K = K.variant();
     value.resize (base::_piola_ops.size(hat_K));
@@ -137,14 +136,16 @@ public:
       point_basic<float_type> xq = piola_transformation (base::_omega, base::_piola_ops, K, base::_dis_inod, q);
       *iter = _f (xq);
     }
+    return true;
   }
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<result_type>& value) const {
     evaluate (base::get_side(L,sid), value);
   }
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     static const bool status = details::is_equal<ValueType,result_type>::value;
     check_macro (status, "unexpected result_type");
+    return status;
   }
 // data:
 protected:
@@ -169,25 +170,31 @@ public:
 
 // alocators:
 
-  explicit field_expr_terminal_function (const Function& f) : base(new_macro(rep(f))) {} 
+  explicit field_expr_terminal_function (const Function& f) : base(new_macro(rep(f))) {}
+
+  template<class TrueFunction, class Check = typename std::enable_if<std::is_function<TrueFunction>::value, TrueFunction>::type>
+  explicit field_expr_terminal_function (TrueFunction f) : base(new_macro(rep(std::ptr_fun(f)))) {} 
+
+  template <typename Constant, class Check = typename std::enable_if <std::is_arithmetic<Constant>::value, Constant>::type>
+  explicit field_expr_terminal_function (const Constant& c) : base(new_macro(rep(details::f_constant<point_basic<float_type>,result_type>(c)))) {}
 
 // accessors:
 
   space_constant::valued_type valued_tag() const { return base::data().valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
   }
-  void evaluate (const geo_element& K, std::vector<result_type>& value) const {
-  	base::data().evaluate (K, value); }
+  bool evaluate (const geo_element& K, std::vector<result_type>& value) const {
+  	return base::data().evaluate (K, value); }
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<result_type>& value) const {
   	base::data().evaluate_on_side (L, sid, value); }
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>(); }
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>(); }
 };
 // ---------------------------------------------------------------------------
 // 1.3) normal to a surface
@@ -228,16 +235,17 @@ public:
 
   space_constant::valued_type valued_tag() const { return space_constant::vector; }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
 
-  void evaluate         (const geo_element& K,                                   std::vector<point_basic<T> >& value) const;
+  bool evaluate         (const geo_element& K,                                   std::vector<point_basic<T> >& value) const;
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<point_basic<T> >& value) const;
 
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     static const bool status = details::is_equal<ValueType,result_type>::value;
     check_macro (status, "unexpected result_type");
+    return status;
   }
 };
 // ---------------------------------------------------------------------------
@@ -279,16 +287,17 @@ public:
 
   space_constant::valued_type valued_tag() const { return space_constant::scalar; }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
 
-  void evaluate         (const geo_element& K,                                   std::vector<result_type>& value) const;
+  bool evaluate         (const geo_element& K,                                   std::vector<result_type>& value) const;
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<result_type>& value) const;
 
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     static const bool status = details::is_equal<ValueType,result_type>::value;
     check_macro (status, "unexpected result_type");
+    return status;
   }
 };
 // ---------------------------------------------------------------------------
@@ -330,16 +339,17 @@ public:
 
   space_constant::valued_type valued_tag() const { return space_constant::scalar; }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
 
-  void evaluate         (const geo_element& K,                                   std::vector<result_type>& value) const;
+  bool evaluate         (const geo_element& K,                                   std::vector<result_type>& value) const;
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<result_type>& value) const;
 
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     static const bool status = details::is_equal<ValueType,result_type>::value;
     check_macro (status, "unexpected result_type");
+    return status;
   }
 protected:
 // internal:
@@ -375,7 +385,7 @@ public:
   const geo_basic<T,M>& get_geo() const { return _omega; }
   const geo_element& get_side (const geo_element& K, const side_information_type& sid) const;
 
-  void initialize (const geo_basic<T,M>& omega, const quadrature<T>& quad) const;
+  bool initialize (const geo_basic<T,M>& omega, const quadrature<T>& quad) const;
   bool initialize (const space_basic<T,M>& Xh) const;
   void evaluate_initialize (const geo_element& K) const;
   void evaluate_on_side_initialize (const geo_element& L, const side_information_type& sid) const;
@@ -424,21 +434,23 @@ public:
   space_constant::valued_type valued_tag() const { return base::_uh.valued_tag(); }
 
   const geo_basic<T,M>& get_geo() const { return base::get_geo(); }
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const;
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const;
   template<class ValueType>
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<ValueType>& value) const;
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     space_constant::valued_type valued_tag = space_constant::valued_tag_traits<ValueType>::value;
-    check_macro ((base::_uh.valued_tag() == valued_tag) ||
-                 (base::_uh.valued_tag() == space_constant::unsymmetric_tensor &&
-                  valued_tag             == space_constant::tensor),
+    bool status = (base::_uh.valued_tag() == valued_tag) ||
+                  (base::_uh.valued_tag() == space_constant::unsymmetric_tensor &&
+                   valued_tag             == space_constant::tensor);
+    check_macro (status,
            "unexpected "<< base::_uh.valued()
         << "-valued field while a " << space_constant::valued_name(valued_tag)
         << "-valued one is expected in expression");
+    return status;
   }
 };
 template<class T, class M>
@@ -472,19 +484,19 @@ public:
 
   const geo_basic<T,M>& get_geo() const { return base::data().get_geo(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
   }
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>();
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>();
   }
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const {
-  	base::data().evaluate(K,value);
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const {
+  	return base::data().evaluate(K,value);
   } 
   template<class ValueType>
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<ValueType>& value) const {
@@ -527,12 +539,15 @@ public:
 
   space_constant::valued_type valued_tag() const;
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const;
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const;
   template<class ValueType>
-  void valued_check() const { _u_test.template grad_valued_check<ValueType>(); }
+  bool valued_check() const {
+	_u_test.template grad_valued_check<ValueType>(); 
+	return true;
+  }
 protected:
 // data:
   details::grad_option_type          _opt;
@@ -569,18 +584,18 @@ public:
   space_constant::valued_type valued_tag() const { return base::data().valued_tag(); }
 
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const {
-  	base::data().evaluate(K,value); }
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const {
+  	return base::data().evaluate(K,value); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
   }
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>();
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>();
   }
 };
 // ---------------------------------------------------------------------------
@@ -624,10 +639,10 @@ public:
 
   space_constant::valued_type valued_tag() const;
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const;
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const;
 
   template<class ValueType>
   void evaluate_on_side (const geo_element& L, const side_information_type& sid, std::vector<ValueType>& value) const {
@@ -635,7 +650,10 @@ public:
     evaluate (L, value);
   }
   template<class ValueType>
-  void valued_check() const { _u_test.template div_valued_check<ValueType>(); }
+  bool valued_check() const { 
+	_u_test.template div_valued_check<ValueType>();
+	return true;
+  }
 // data:
   details::grad_option_type          _opt;
   test_basic<T,M,details::vf_tag_01> _u_test;
@@ -671,11 +689,11 @@ public:
   space_constant::valued_type valued_tag() const { return base::data().valued_tag(); }
 
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const {
-  	base::data().evaluate(K,value); }
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const {
+  	return base::data().evaluate(K,value); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
@@ -685,8 +703,8 @@ public:
   	base::data().evaluate_on_side (L, sid, value);
   }
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>();
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>();
   }
 };
 // ---------------------------------------------------------------------------
@@ -726,12 +744,15 @@ public:
 
   space_constant::valued_type valued_tag() const;
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const;
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const;
   template<class ValueType>
-  void valued_check() const { _u_test.template curl_valued_check<ValueType>(); }
+  bool valued_check() const { 
+	_u_test.template curl_valued_check<ValueType>();
+	return true;
+  }
 // data:
   details::grad_option_type          _opt;
   test_basic<T,M,details::vf_tag_01> _u_test;
@@ -767,18 +788,18 @@ public:
   space_constant::valued_type valued_tag() const { return base::data().valued_tag(); }
 
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const {
-  	base::data().evaluate(K,value); }
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const {
+  	return base::data().evaluate(K,value); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
   }
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>();
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>();
   }
 };
 // ----------------------------------------------------------------------------
@@ -815,17 +836,19 @@ public:
 
   space_constant::valued_type valued_tag() const { return _expr0.valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& quad) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
   template<class ValueType>
-  void evaluate (const geo_element& K, std::vector<ValueType>& value) const;
+  bool evaluate (const geo_element& K, std::vector<ValueType>& value) const;
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     space_constant::valued_type valued_tag = space_constant::valued_tag_traits<ValueType>::value;
-    check_macro (_expr0.valued_tag() == valued_tag,
+    bool status = _expr0.valued_tag() == valued_tag;
+    check_macro (status,
  	              "unexpected "<<  space_constant::valued_name(_expr0.valued_tag())
         << "-valued field while a " << space_constant::valued_name(valued_tag)
         << "-valued one is expected in expression");
+    return status;
   }
 // data:
 protected:
@@ -862,19 +885,19 @@ public:
 
   space_constant::valued_type valued_tag() const { return base::data().valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
   }
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>();
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>();
   }
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const {
-  	base::data().evaluate(K,value);
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const {
+  	return base::data().evaluate(K,value);
   } 
 };
 // ---------------------------------------------------------------------------
@@ -906,18 +929,21 @@ public:
 
   space_constant::valued_type valued_tag() const { return _uh.valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const;
   bool initialize (const space_basic<float_type,memory_type>& Xh) const;
 
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const;
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const;
 
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     space_constant::valued_type valued_tag = space_constant::valued_tag_traits<ValueType>::value;
-    check_macro (_uh.valued_tag() == valued_tag, "unexpected "<<_uh.valued()
+    bool status = (_uh.valued_tag() == valued_tag);
+    check_macro (status, 
+        "unexpected "<<_uh.valued()
         << "-valued field while a " << space_constant::valued_name(valued_tag)
         << "-valued one is expected in expression");
+    return status;
   }
 
 // internal:
@@ -964,19 +990,19 @@ public:
 
   space_constant::valued_type valued_tag() const { return base::data().valued_tag(); }
 
-  void initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
-  	base::data().initialize(omega,hat_x);
+  bool initialize (const geo_basic<float_type,memory_type>& omega, const quadrature<float_type>& hat_x) const {
+  	return base::data().initialize(omega,hat_x);
   }
   bool initialize (const space_basic<float_type,memory_type>& Xh) const {
   	return base::data().initialize(Xh);
   }
   template<class Result>
-  void evaluate (const geo_element& K, std::vector<Result>& value) const {
-  	base::data().evaluate(K,value); }
+  bool evaluate (const geo_element& K, std::vector<Result>& value) const {
+  	return base::data().evaluate(K,value); }
 
   template<class ValueType>
-  void valued_check() const {
-  	base::data().valued_check<ValueType>();
+  bool valued_check() const {
+  	return base::data().valued_check<ValueType>();
   }
 };
 
