@@ -36,29 +36,13 @@
 # include "rheolef/rounder.h"
 # include "rheolef/rheostream.h"
 
+# include "field_seq_visu_gnuplot_internal.h"
+
 namespace rheolef {
 
 template <class T>
 odiststream& visu_gnuplot (odiststream& ops, const geo_basic<T,sequential>& omega);
 
-// ----------------------------------------------------------------------------
-// bounds: for curved geometries, need to commpute it on the fly
-// ----------------------------------------------------------------------------
-template <class T>
-struct bound_type {
-// data:
-  point_basic<T> xmin, xmax;
-  T              umin, umax;
-// modifier:
-  void update (const point_basic<T>& x, const T& u) {
-    for (size_t i = 0; i < 3; i++) {
-      xmin[i] = std::min(xmin[i], x[i]);
-      xmax[i] = std::max(xmax[i], x[i]);
-    }
-    umin = std::min(umin, u);
-    umax = std::max(umax, u);
-  }
-};
 // ----------------------------------------------------------------------------
 // puts for one element
 // ----------------------------------------------------------------------------
@@ -156,7 +140,6 @@ put_quadrangle (std::ostream& gdat, const geo_element& F, const geo_basic<T,sequ
   gdat << endl << endl;
 }
 template<class T>
-static
 void
 put (std::ostream& gdat, const geo_element& K, const geo_basic<T,sequential>& omega,
 	const field_basic<T,sequential>& uh,
@@ -254,7 +237,7 @@ visu_gnuplot_scalar (odiststream& ods, const field_basic<T,sequential>& uh)
   //
   // rounding bounds
   //
-  Float eps = 1e-7;
+  T eps = 1e-7;
   bbox.umin = floorer(eps) (bbox.umin);
   bbox.umax = ceiler(eps)  (bbox.umax);
   //
@@ -427,13 +410,13 @@ visu_gnuplot_vector (odiststream& ods, const field_basic<T,sequential>& uh)
 	"gnuplot vector: unsupported non-isoparametric approx " << uh.get_space().get_numbering().name());
   size_type n_elt = omega.size();
   size_type d     = omega.dimension();
-  Float diam_omega = norm (omega.xmax() - omega.xmin()); // characteristic length in omega
-  Float h_moy = diam_omega/pow(n_elt,1./d);
+  T diam_omega = norm (omega.xmax() - omega.xmin()); // characteristic length in omega
+  T h_moy = diam_omega/pow(n_elt,1./d);
   space_basic<T,sequential> Xh (uh.get_geo(), uh.get_approx());
   field_basic<T,sequential> norm_uh = interpolate(Xh, norm(uh));
-  Float norm_max_uh = norm_uh.max_abs();      // get max vector length
+  T norm_max_uh = norm_uh.max_abs();      // get max vector length
   if (norm_max_uh + 1 == 1) norm_max_uh = 1;
-  Float scale = h_moy/norm_max_uh;
+  T scale = h_moy/norm_max_uh;
   size_type n_comp = uh.size();
   std::vector<field_component_const<T,sequential> > uh_comp (n_comp);
   for (size_type i_comp = 0; i_comp < n_comp; i_comp++) {
@@ -472,6 +455,9 @@ visu_gnuplot (odiststream& ods, const field_basic<T,sequential>& uh)
 // ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
-template odiststream& visu_gnuplot<Float> (odiststream&, const field_basic<Float,sequential>&);
+#define _RHEOLEF_instanciate(T) \
+template odiststream& visu_gnuplot<T> (odiststream&, const field_basic<T,sequential>&);
 
+_RHEOLEF_instanciate(Float)
+#undef _RHEOLEF_instanciate
 } // rheolef namespace
