@@ -64,7 +64,7 @@ public:
 
   static bool have_test_space() { return RawExpr::have_test_space(); }
   const space_type& get_vf_space() const { return _raw_expr.get_vf_space(); }
-  static const space_constant::valued_type valued_hint = RawExpr::last_valued;
+  static const space_constant::valued_type valued_hint = RawExpr::valued_hint;
   space_constant::valued_type valued_tag() const { return _raw_expr.valued_tag(); } 
   size_type n_derivative() const { return _raw_expr.n_derivative(); }
 
@@ -107,12 +107,8 @@ public:
   typedef typename Expr::memory_type                    memory_type;
   typedef typename scalar_traits<typename Expr::value_type>::type 		
 	  						scalar_type;
-#ifndef TODO
   typedef typename space_constant::rank_up<typename Expr::value_type>::type
                                                         value_type;
-#else // TODO
-  typedef point_basic<scalar_type>                      value_type;
-#endif // TODO
   typedef typename  float_traits<scalar_type>::type 	float_type;
   typedef space_basic<scalar_type,memory_type>		space_type;
   typedef typename Expr::vf_tag_type                    vf_tag_type;
@@ -148,6 +144,7 @@ public:
     switch (v) {
       case space_constant::scalar: return space_constant::vector;
       case space_constant::vector: return space_constant::unsymmetric_tensor;
+      case space_constant::tensor: return space_constant::tensor3;
       default:
         fatal_macro ("unexpected " << space_constant::valued_name(v) << "-valued argument for grad() operator");
         return space_constant::last_valued;
@@ -443,7 +440,10 @@ public:
   };
   template<class ValueType, class Arg>
   void evaluate_call (const reference_element& hat_K, size_type q, std::vector<ValueType>& value) const {
-    typedef typename details::generic_unary_traits<UnaryFunction>::template hint<Arg,ValueType>::result_type result_type;
+    // check if ValueType is a valid return_type for this function:
+    typedef typename scalar_traits<ValueType>::type S;
+    typedef undeterminated_basic<S> undet;
+    typedef typename details::generic_unary_traits<UnaryFunction>::template hint<Arg,undet>::result_type result_type;
     // TODO: instead of is_equal, could have compatible scalars T1,T2 ?
     typedef typename details::is_equal<ValueType,result_type>::type status_t;
     evaluate_call_check<ValueType,Arg,status_t> eval;
@@ -736,7 +736,11 @@ protected:
         case space_constant::tensor:
         case space_constant::unsymmetric_tensor:
           obj._nl_expr.evaluate (K, obj._tensor_nl_value_quad); break;
-        default: error_macro ("unexpected first argument valued tag="<<nl_arg_valued_tag);
+        case space_constant::tensor3:
+          obj._nl_expr.evaluate (K, obj._tensor3_nl_value_quad); break;
+        case space_constant::tensor4:
+          obj._nl_expr.evaluate (K, obj._tensor4_nl_value_quad); break;
+        default: error_macro ("unexpected first argument valued tag="<<nl_arg_valued_tag); // ICI
       }
     }
     void element_initialize_on_side (const This& obj, const geo_element& K, const side_information_type& sid) const {
@@ -749,6 +753,10 @@ protected:
         case space_constant::tensor:
         case space_constant::unsymmetric_tensor:
           obj._nl_expr.evaluate_on_side (K, sid, obj._tensor_nl_value_quad); break;
+        case space_constant::tensor3:
+          obj._nl_expr.evaluate_on_side (K, sid, obj._tensor3_nl_value_quad); break;
+        case space_constant::tensor4:
+          obj._nl_expr.evaluate_on_side (K, sid, obj._tensor4_nl_value_quad); break;
         default: error_macro ("unexpected first argument valued tag="<<nl_arg_valued_tag);
       }
     }
@@ -788,6 +796,26 @@ protected:
     const tensor_basic<scalar_type>& get_nl_value (const This& obj, size_type q) const {
       return obj._tensor_nl_value_quad[q]; }
   };
+  template<class This> struct nl_switch<This,tensor3_basic<typename This::scalar_type> > {
+    typedef typename This::size_type size_type;
+    typedef typename This::scalar_type scalar_type;
+    void element_initialize (const This& obj, const geo_element& K) const {
+      obj._nl_expr.evaluate (K, obj._tensor3_nl_value_quad); }
+    void element_initialize_on_side (const This& obj, const geo_element& K, const side_information_type& sid) const {
+      obj._nl_expr.evaluate_on_side (K, sid, obj._tensor3_nl_value_quad); }
+    const tensor3_basic<scalar_type>& get_nl_value (const This& obj, size_type q) const {
+      return obj._tensor3_nl_value_quad[q]; }
+  };
+  template<class This> struct nl_switch<This,tensor4_basic<typename This::scalar_type> > {
+    typedef typename This::size_type size_type;
+    typedef typename This::scalar_type scalar_type;
+    void element_initialize (const This& obj, const geo_element& K) const {
+      obj._nl_expr.evaluate (K, obj._tensor4_nl_value_quad); }
+    void element_initialize_on_side (const This& obj, const geo_element& K, const side_information_type& sid) const {
+      obj._nl_expr.evaluate_on_side (K, sid, obj._tensor4_nl_value_quad); }
+    const tensor4_basic<scalar_type>& get_nl_value (const This& obj, size_type q) const {
+      return obj._tensor4_nl_value_quad[q]; }
+  };
 // ---------------------------------------------------------------------------
 // binary function call: (f nl_expr vf_expr)
 // examples: f = operator*, operator/
@@ -848,7 +876,8 @@ public:
       _vf_expr(vf_expr),
       _scalar_nl_value_quad(),
       _vector_nl_value_quad(),
-      _tensor_nl_value_quad()
+      _tensor_nl_value_quad(),
+      _tensor4_nl_value_quad()
     {}
 
   field_vf_expr_binded_bf (const field_vf_expr_binded_bf<BinaryFunction,NLExpr,VFExpr>& x)
@@ -857,7 +886,8 @@ public:
       _vf_expr(x._vf_expr),
       _scalar_nl_value_quad(x._scalar_nl_value_quad),
       _vector_nl_value_quad(x._vector_nl_value_quad),
-      _tensor_nl_value_quad(x._tensor_nl_value_quad)
+      _tensor_nl_value_quad(x._tensor_nl_value_quad),
+      _tensor4_nl_value_quad(x._tensor4_nl_value_quad)
     {}
 
 // accessors:
@@ -962,6 +992,10 @@ public:
         case space_constant::tensor:
         case space_constant::unsymmetric_tensor:
 	  obj.template evaluate_call<ValueType, tensor_basic<T1>, Arg2> (hat_K, q, value); break;
+        case space_constant::tensor3:
+	  obj.template evaluate_call<ValueType, tensor3_basic<T1>, Arg2> (hat_K, q, value); break;
+        case space_constant::tensor4:
+	  obj.template evaluate_call<ValueType, tensor4_basic<T1>, Arg2> (hat_K, q, value); break;
         default: error_macro ("unexpected first argument valued tag="<<arg1_valued_tag);
       }
     }
@@ -980,6 +1014,10 @@ public:
         case space_constant::tensor:
         case space_constant::unsymmetric_tensor:
 	  obj.template evaluate_call<ValueType, Arg1, tensor_basic<T2> > (hat_K, q, value); break;
+        case space_constant::tensor3:
+	  obj.template evaluate_call<ValueType, Arg1, tensor3_basic<T2> > (hat_K, q, value); break;
+        case space_constant::tensor4:
+	  obj.template evaluate_call<ValueType, Arg1, tensor4_basic<T2> > (hat_K, q, value); break;
         default: error_macro ("unexpected second argument valued tag="<<arg2_valued_tag);
       }
     }
@@ -1003,6 +1041,10 @@ public:
             case space_constant::tensor:
             case space_constant::unsymmetric_tensor:
 	      obj.template evaluate_call<ValueType, T1, tensor_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor3:
+	      obj.template evaluate_call<ValueType, T1, tensor3_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor4:
+	      obj.template evaluate_call<ValueType, T1, tensor4_basic<T2> > (hat_K, q, value); break;
             default: error_macro ("unexpected second argument valued tag="<<arg2_valued_tag);
           }
           break;
@@ -1016,6 +1058,10 @@ public:
             case space_constant::tensor:
             case space_constant::unsymmetric_tensor:
 	      obj.template evaluate_call<ValueType, point_basic<T1>, tensor_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor3:
+	      obj.template evaluate_call<ValueType, point_basic<T1>, tensor3_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor4:
+	      obj.template evaluate_call<ValueType, point_basic<T1>, tensor4_basic<T2> > (hat_K, q, value); break;
             default: error_macro ("unexpected second argument valued tag="<<arg2_valued_tag);
           }
           break;
@@ -1030,6 +1076,44 @@ public:
             case space_constant::tensor:
             case space_constant::unsymmetric_tensor:
 	      obj.template evaluate_call<ValueType, tensor_basic<T1>, tensor_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor3:
+	      obj.template evaluate_call<ValueType, tensor_basic<T1>, tensor3_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor4:
+	      obj.template evaluate_call<ValueType, tensor_basic<T1>, tensor4_basic<T2> > (hat_K, q, value); break;
+            default: error_macro ("unexpected second argument valued tag="<<arg2_valued_tag);
+          }
+          break;
+        }
+        case space_constant::tensor3: {
+          switch (arg2_valued_tag) {
+            case space_constant::scalar:
+	      obj.template evaluate_call<ValueType, tensor3_basic<T1>, T2>                (hat_K, q, value); break;
+            case space_constant::vector:
+	      obj.template evaluate_call<ValueType, tensor3_basic<T1>, point_basic<T2> >  (hat_K, q, value); break;
+            case space_constant::tensor:
+            case space_constant::unsymmetric_tensor:
+	      obj.template evaluate_call<ValueType, tensor3_basic<T1>, tensor_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor3:
+	      obj.template evaluate_call<ValueType, tensor3_basic<T1>, tensor3_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor4:
+	      obj.template evaluate_call<ValueType, tensor3_basic<T1>, tensor4_basic<T2> > (hat_K, q, value); break;
+            default: error_macro ("unexpected second argument valued tag="<<arg2_valued_tag);
+          }
+          break;
+        }
+        case space_constant::tensor4: {
+          switch (arg2_valued_tag) {
+            case space_constant::scalar:
+	      obj.template evaluate_call<ValueType, tensor4_basic<T1>, T2>                (hat_K, q, value); break;
+            case space_constant::vector:
+	      obj.template evaluate_call<ValueType, tensor4_basic<T1>, point_basic<T2> >  (hat_K, q, value); break;
+            case space_constant::tensor:
+            case space_constant::unsymmetric_tensor:
+	      obj.template evaluate_call<ValueType, tensor4_basic<T1>, tensor_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor3:
+	      obj.template evaluate_call<ValueType, tensor4_basic<T1>, tensor3_basic<T2> > (hat_K, q, value); break;
+            case space_constant::tensor4:
+	      obj.template evaluate_call<ValueType, tensor4_basic<T1>, tensor4_basic<T2> > (hat_K, q, value); break;
             default: error_macro ("unexpected second argument valued tag="<<arg2_valued_tag);
           }
           break;
@@ -1079,29 +1163,6 @@ public:
 	 ,ValueType>::second_argument_type
        >::type A2;
 
-#ifdef TO_CLEAN
-    typedef
-       typename details::generic_binary_traits<BinaryFunction>::template hint<
-	  typename NLExpr::value_type
-	 ,typename VFExpr::value_type
-	 ,ValueType>::second_argument_type
-       HINT_A2;
-    typedef
-       typename details::generic_binary_traits<details::multiplies>::template hint<
-	  undeterminated_basic<double>
-	 ,point_basic<double>
-	 ,point_basic<double> 
-       >::first_argument_type
-       HINT_DIR_A2;
-    typedef
-       typename details::generic_binary_traits<details::swapper<details::multiplies> >::template hint<
-	  point_basic<double>
-	 ,undeterminated_basic<double>
-	 ,point_basic<double> 
-       >::second_argument_type
-       HINT_SWP_A2;
-#endif // TO_CLEAN
-
     if (! is_undeterminated<A1>::value) _nl_expr.valued_check<A1>();
     if (! is_undeterminated<A2>::value) _vf_expr.valued_check<A2>();
   }
@@ -1113,6 +1174,8 @@ public:
   mutable std::vector<scalar_type>                 _scalar_nl_value_quad;
   mutable std::vector<point_basic<scalar_type> >   _vector_nl_value_quad;
   mutable std::vector<tensor_basic<scalar_type> >  _tensor_nl_value_quad;
+  mutable std::vector<tensor3_basic<scalar_type> > _tensor3_nl_value_quad;
+  mutable std::vector<tensor4_basic<scalar_type> > _tensor4_nl_value_quad;
 };
 
 } // namespace rheolef
