@@ -205,29 +205,29 @@ public:
     // backward compat: named forms
     form_basic (const space_type& X, const space_type& Y,
 	const std::string& name = "",
-	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family));
 
     form_basic (const space_type& X, const space_type& Y,
 	const std::string& name,
 	const field_basic<T,M>& weight, 
-	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family));
 
     template<class Function>
     form_basic (const space_type& X, const space_type& Y,
 	const std::string& name,
 	Function weight,
-	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family));
 
     form_basic (const space_type& X, const space_type& Y,
 	const std::string& name,
 	const geo_basic<T,M>& gamma,
-	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family));
 
     form_basic (const space_type& X, const space_type& Y,
 	const std::string& name,
 	const geo_basic<T,M>& gamma,
 	const field_basic<T,M>& weight,
-	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family));
 
     template<class Function>
     form_basic (
@@ -236,7 +236,7 @@ public:
         const std::string& name,
         const geo_basic<T,M>& gamma,
         Function weight,
-	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+	const quadrature_option_type& qopt = quadrature_option_type(quadrature_option_type::max_family));
 protected: 
     // backward compat: named forms (cont.)
     template<class WeightFunction>
