@@ -64,8 +64,8 @@ public:
   size_type n_derivative() const            { return _raw_expr.n_derivative(); }
 
 // mutable modifiers:
-  void initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const {
-    _raw_expr.initialize (dom, quad, ignore_sys_coord);
+  bool initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const {
+    return _raw_expr.initialize (dom, quad, ignore_sys_coord);
   }
   void initialize (const band_basic<float_type,memory_type>& gh, const quadrature<float_type>& quad, bool ignore_sys_coord) const {  
     _raw_expr.initialize (gh, quad, ignore_sys_coord);
@@ -77,10 +77,10 @@ public:
     _raw_expr.basis_evaluate (hat_K, q, value);
   }
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     static const bool status = details::is_equal<ValueType,scalar_type>::value;
     check_macro (status, "unexpected result_type"); 
-    _raw_expr.valued_check<ValueType>();
+    return _raw_expr.valued_check<ValueType>();
   }
 
 protected:
@@ -147,13 +147,14 @@ public:
 
 // mutable modifiers:
 
-  void initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
-    _expr1.initialize (dom, quad, ignore_sys_coord);
-    _expr2.initialize (dom, quad, ignore_sys_coord);
+  bool initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
+       	_expr1.initialize (dom, quad, ignore_sys_coord);
+        _expr2.initialize (dom, quad, ignore_sys_coord);
+	return true;
   }
   void initialize (const band_basic<float_type,memory_type>& gh, const quadrature<float_type>& quad, bool ignore_sys_coord) const {  
-    _expr1.initialize (gh, quad, ignore_sys_coord);
-    _expr2.initialize (gh, quad, ignore_sys_coord);
+           _expr1.initialize (gh, quad, ignore_sys_coord);
+           _expr2.initialize (gh, quad, ignore_sys_coord);
   }
   void element_initialize (const geo_element& K) const {
     _expr1.element_initialize (K);
@@ -206,7 +207,7 @@ public:
     eval (*this, hat_K, q, value);
   }
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     typedef typename details::generic_binary_traits<BinaryFunction>::template hint<
 	  typename Expr1::value_type
 	 ,typename Expr2::value_type
@@ -215,8 +216,9 @@ public:
 	  typename Expr1::value_type
 	 ,typename Expr2::value_type
 	 ,ValueType>::second_argument_type A2;
-    if (! is_undeterminated<A1>::value) _expr1.valued_check<A1>();
-    if (! is_undeterminated<A2>::value) _expr2.valued_check<A2>();
+    if (! is_undeterminated<A1>::value)  _expr1.valued_check<A1>();
+    if (! is_undeterminated<A2>::value)  _expr2.valued_check<A2>();
+    return true;
   }
 protected:
 // data:
@@ -266,8 +268,8 @@ public:
 
 // mutable modifiers:
 
-  void initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
-    _expr.initialize (dom, quad, ignore_sys_coord);
+  bool initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
+    return _expr.initialize (dom, quad, ignore_sys_coord);
   }
   void initialize (const band_basic<float_type,memory_type>& gh, const quadrature<float_type>& quad, bool ignore_sys_coord) const {  
     _expr.initialize (gh, quad, ignore_sys_coord);
@@ -287,9 +289,10 @@ public:
     }}
   }
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     typedef ValueType A1;
-    if (! is_undeterminated<A1>::value) _expr.valued_check<A1>();
+    if (! is_undeterminated<A1>::value) return _expr.valued_check<A1>();
+    return true;
   }
 protected:
 // data:
@@ -350,9 +353,9 @@ public:
 // mutable modifiers:
 
   // TODO: at init, check that exp1 & expr2 has the same test & trial spaces
-  void initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
-    _expr1.initialize (dom, quad, ignore_sys_coord);
-    _expr2.initialize (dom, quad, ignore_sys_coord);
+  bool initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
+    return _expr1.initialize (dom, quad, ignore_sys_coord) &&
+           _expr2.initialize (dom, quad, ignore_sys_coord);
   }
   void initialize (const band_basic<float_type,memory_type>& gh, const quadrature<float_type>& quad, bool ignore_sys_coord) const {  
     _expr1.initialize (gh, quad, ignore_sys_coord);
@@ -379,11 +382,13 @@ public:
     }}
   }
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     typedef ValueType A1;
     typedef ValueType A2;
-    if (! is_undeterminated<A1>::value) _expr1.valued_check<A1>();
-    if (! is_undeterminated<A2>::value) _expr2.valued_check<A2>();
+    bool status = true;
+    if (! is_undeterminated<A1>::value)  status &= _expr1.valued_check<A1>();
+    if (! is_undeterminated<A2>::value)  status &= _expr2.valued_check<A2>();
+    return status;
   }
 protected:
 // data:
@@ -451,9 +456,10 @@ public:
 
 // mutable modifiers:
 
-  void initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
+  bool initialize (const geo_basic<float_type,memory_type>& dom, const quadrature<float_type>& quad, bool ignore_sys_coord) const { 
     _nl_expr.initialize (dom, quad);
     _vf_expr.initialize (dom, quad, ignore_sys_coord);
+    return true;
   }
   void initialize (const band_basic<float_type,memory_type>& gh, const quadrature<float_type>& quad, bool ignore_sys_coord) const {  
     _nl_expr.initialize (gh.level_set(), quad);
@@ -485,11 +491,13 @@ public:
     }}
   }
   template<class ValueType>
-  void valued_check() const {
+  bool valued_check() const {
     typedef ValueType A1;
     typedef ValueType A2;
-    if (! is_undeterminated<A1>::value) _nl_expr.valued_check<A1>();
-    if (! is_undeterminated<A2>::value) _vf_expr.valued_check<A2>();
+    bool status = true;
+    if (! is_undeterminated<A1>::value)  status &= _nl_expr.valued_check<A1>();
+    if (! is_undeterminated<A2>::value)  status &= _vf_expr.valued_check<A2>();
+    return status;
   }
 //protected:
 // data:
