@@ -47,9 +47,12 @@ geo_base_rep<T,M>::geo_base_rep ()
    _sys_coord(space_constant::cartesian),
    _xmin(),
    _xmax(),
+   _hmin(),
+   _hmax(),
    _numbering("P1"),
    _locator(),
-   _tracer_ray_boundary()
+   _tracer_ray_boundary(),
+   _nearestor()
 {
 }
 template <class T, class M>
@@ -69,9 +72,12 @@ geo_base_rep<T,M>::geo_base_rep (const geo_base_rep<T,M>& o)
    _sys_coord           (o._sys_coord),
    _xmin                (o._xmin),
    _xmax                (o._xmax),
+   _hmin		(o._hmin),
+   _hmax                (o._hmax),
    _numbering           (o._numbering),
    _locator             (o._locator),
-   _tracer_ray_boundary(o._tracer_ray_boundary)
+   _tracer_ray_boundary (o._tracer_ray_boundary),
+   _nearestor           (o._nearestor)
 {
   trace_macro ("*** PHYSICAL COPY OF GEO_BASE_REP ***");
 }
@@ -331,6 +337,24 @@ geo_base_rep<T,M>::compute_bbox()
     }
   }
 #endif // _RHEOLEF_HAVE_MPI
+
+  // next, compute hmin & hmax
+  _hmin = std::numeric_limits<T>::max();
+  _hmax = 0;
+  for (size_t iedg = 0, nedg = geo_element_ownership(1).size(); iedg < nedg; ++iedg) {
+    const geo_element& E = get_geo_element (1, iedg);
+    const point_basic<T>& x0 = dis_node(E[0]);
+    const point_basic<T>& x1 = dis_node(E[1]);
+    T hloc = dist(x0,x1);
+    _hmin = min(_hmin, hloc);
+    _hmax = max(_hmax, hloc);
+  }
+#ifdef _RHEOLEF_HAVE_MPI
+  if (is_distributed<M>::value) {
+    _hmin = mpi::all_reduce (comm(), _hmin, mpi::minimum<T>());
+    _hmax = mpi::all_reduce (comm(), _hmax, mpi::maximum<T>());
+  }
+#endif // _RHEOLEF_HAVE_MPI
 }
 template <class T, class M>
 typename geo_base_rep<T,M>::const_reference
