@@ -65,6 +65,7 @@
 #include "rheolef/space_constant.h"
 #include "rheolef/geo_locate.h"
 #include "rheolef/geo_trace_ray_boundary.h"
+#include "rheolef/geo_nearest.h"
 
 namespace rheolef {
 
@@ -197,6 +198,8 @@ public:
     virtual const basis_basic<T>& get_piola_basis() const = 0;
     virtual const node_type& xmin() const = 0;
     virtual const node_type& xmax() const = 0;
+    virtual const T&         hmin() const = 0;
+    virtual const T&         hmax() const = 0;
     virtual const geo_size&  sizes() const = 0;
     virtual const geo_size&  ios_sizes() const = 0;
 
@@ -230,14 +233,22 @@ public:
     virtual size_type dis_locate (
 	const point_basic<T>& x,
 	size_type dis_ie_guest = std::numeric_limits<size_type>::max()) const = 0;
+
     virtual size_type seq_trace_move (
-                const point_basic<T>&     x,
-                const point_basic<T>&     v,
-                      point_basic<T>&     y) const = 0;
+        const point_basic<T>&     x,
+        const point_basic<T>&     v,
+              point_basic<T>&     y) const = 0;
     virtual size_type dis_trace_move (
-                const point_basic<T>&     x,
-                const point_basic<T>&     v,
-                      point_basic<T>&     y) const = 0;
+        const point_basic<T>&     x,
+        const point_basic<T>&     v,
+              point_basic<T>&     y) const = 0;
+
+    virtual size_type seq_nearest (
+        const point_basic<T>&    x,
+              point_basic<T>&    x_nearest) const = 0;
+    virtual size_type dis_nearest (
+        const point_basic<T>&    x,
+              point_basic<T>&    x_nearest) const = 0;
 
 // virtual i/o:
 
@@ -333,7 +344,10 @@ public:
                 const array<point_basic<T>,sequential>&     v,
                       array<size_type, sequential>&         dis_ie,
                       array<point_basic<T>,sequential>&     y) const = 0;
-
+    virtual void nearest (
+                const array<point_basic<T>,sequential>&     x,
+                      array<point_basic<T>,sequential>&     x_nearest,
+                      array<size_type, sequential>&         dis_ie) const = 0;
 };
 #ifdef _RHEOLEF_HAVE_MPI
 template <class T>
@@ -385,6 +399,10 @@ public:
                 const array<point_basic<T>,distributed>&     v,
                       array<size_type, distributed>&         dis_ie,
                       array<point_basic<T>,distributed>&     y) const = 0;
+    virtual void nearest (
+                const array<point_basic<T>,distributed>&     x,
+                      array<point_basic<T>,distributed>&     x_nearest,
+                      array<size_type, distributed>&         dis_ie) const = 0;
 
 // utility:
 
@@ -446,6 +464,8 @@ public:
     const basis_basic<T>& get_piola_basis() const { return _numbering.get_basis(); }
     const node_type&      xmin() const { return _xmin; }
     const node_type&      xmax() const { return _xmax; }
+    const T&              hmin() const { return _hmin; }
+    const T&              hmax() const { return _hmax; }
     const geo_size&      sizes() const { return _gs; }
     const geo_size&  ios_sizes() const { return _gs; }
 
@@ -487,6 +507,12 @@ public:
                 const point_basic<T>&     x,
                 const point_basic<T>&     v,
                       point_basic<T>&     y) const;
+    size_type seq_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const;
+    size_type dis_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const;
 
     size_type neighbour (size_type ie, size_type loc_isid) const;
     void neighbour_guard() const;
@@ -554,9 +580,12 @@ protected:
     coordinate_type                          _sys_coord;
     node_type				     _xmin; // bounding box
     node_type				     _xmax;
+    T					     _hmin;
+    T					     _hmax;
     numbering<T,M>                           _numbering;
     geo_locate<T,M>                          _locator;
     geo_trace_ray_boundary<T,M>              _tracer_ray_boundary;
+    geo_nearest<T,M>                         _nearestor;
 };
 template <class T, class M>
 inline
@@ -687,6 +716,10 @@ public:
                 const array<point_basic<T>,sequential>&     v,
                       array<size_type, sequential>&         dis_ie,
                       array<point_basic<T>,sequential>&     y) const;
+    void nearest (
+                const array<point_basic<T>,sequential>&     x,
+                      array<point_basic<T>,sequential>&     x_nearest,
+                      array<size_type, sequential>&         dis_ie) const;
 
 // herited accessors:
 
@@ -825,6 +858,10 @@ public:
                 const array<point_basic<T>,distributed>&     v,
                       array<size_type, distributed>&         dis_ie,
                       array<point_basic<T>,distributed>&     y) const;
+    void nearest (
+                const array<point_basic<T>,distributed>&     x,
+                      array<point_basic<T>,distributed>&     x_nearest,
+                      array<size_type, distributed>&         dis_ie) const;
 
 // herited accessors:
 
@@ -982,6 +1019,8 @@ public:
     size_type                     order() const { return base::data().get_piola_basis().degree(); }
     const node_type&               xmin() const { return base::data().xmin(); }
     const node_type&               xmax() const { return base::data().xmax(); }
+    const T&           		   hmin() const { return base::data().hmin(); }
+    const T&	                   hmax() const { return base::data().hmax(); }
     const distributor& geo_element_ownership(size_type dim) const { return base::data().geo_element_ownership(dim); }
     const geo_size&      sizes()             const { return base::data().sizes(); }
     const geo_size&  ios_sizes()             const { return base::data().ios_sizes(); }
@@ -1053,6 +1092,19 @@ public:
                       array<size_type, sequential>&         dis_ie,
                       array<point_basic<T>,sequential>&     y) const
     					{ return base::data().trace_move (x,v,dis_ie,y); }
+    size_type seq_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const
+    					{ return base::data().seq_nearest (x, x_nearest); }
+    size_type dis_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const
+    					{ return base::data().dis_nearest (x, x_nearest); }
+    void nearest (
+                const array<point_basic<T>,sequential>&     x,
+                      array<point_basic<T>,sequential>&     x_nearest,
+                      array<size_type, sequential>&         dis_ie) const
+    					{ base::data().nearest (x, x_nearest, dis_ie); }
 
 // modifiers:
 
@@ -1213,6 +1265,8 @@ public:
     size_type                     order() const { return base::data().get_piola_basis().degree(); }
     const node_type&               xmin() const { return base::data().xmin(); }
     const node_type&               xmax() const { return base::data().xmax(); }
+    const T&           		   hmin() const { return base::data().hmin(); }
+    const T&	                   hmax() const { return base::data().hmax(); }
     const distributor& geo_element_ownership(size_type dim) const
 			{ return base::data().geo_element_ownership (dim); }
     const geo_size&      sizes()             const { return base::data().sizes(); }
@@ -1300,6 +1354,19 @@ public:
                       array<size_type, distributed>&         dis_ie,
                       array<point_basic<T>,distributed>&     y) const
     					{ return base::data().trace_move (x,v,dis_ie,y); }
+    size_type seq_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const
+    					{ return base::data().seq_nearest (x, x_nearest); }
+    size_type dis_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const
+    					{ return base::data().dis_nearest (x, x_nearest); }
+    void nearest (
+                const array<point_basic<T>,distributed>&     x,
+                      array<point_basic<T>,distributed>&     x_nearest,
+                      array<size_type, distributed>&         dis_ie) const
+    					{ base::data().nearest (x, x_nearest, dis_ie); }
 
 // extended accessors:
 
