@@ -57,11 +57,15 @@ public:
 
 // allocators:
 
+    geo_domain_rep (const geo_domain_rep<T,M>&);
     geo_domain_rep (const geo_domain_indirect_rep<T,M>& dom);
+    geo_abstract_rep<T,M>* clone() const;
 
 // accessors: the geo_abstract interface overloaded here:
 
     size_type variant() const { return geo_abstract_base_rep<T>::geo_domain; }
+    const geo_element& bgd2dom_geo_element (const geo_element& bgd_K) const;
+    const geo_element& dom2bgd_geo_element (const geo_element& dom_K) const;
 
     size_type                     n_domain_indirect() const { return 0; }
     const domain_indirect_basic<M>& get_domain_indirect (size_type i) const;
@@ -72,23 +76,21 @@ public:
    const geo_basic<T,M>& get_background_geo()    const { return _dom.get_background_geo(); }
          geo_basic<T,M>  get_background_domain() const;
 
+   size_type bgd_ie2dom_ie (size_type bgd_ie) const {
+        typename std::map<size_type,size_type>::const_iterator iter = _bgd_ie2dom_ie.find (bgd_ie);
+        return (iter != _bgd_ie2dom_ie.end()) ? (*iter).second
+                                              : std::numeric_limits<size_type>::max();
+   }
 protected:
 // data:
-    geo_domain_indirect_rep<T,M>   _dom;
+    geo_domain_indirect_rep<T,M>          _dom;
+    std::map<size_type,size_type>         _bgd_ie2dom_ie;
 };
 // ------------------------------------------------------------------------
 // inlined 
 // ------------------------------------------------------------------------
 template <class T, class M>
 inline
-geo_domain_rep<T,M>::geo_domain_rep (const geo_domain_indirect_rep<T,M>& dom)
-  : geo_rep<T,M>(),
-    _dom (dom)
-{
-    geo_rep<T,M>::build_from_domain (_dom.get_indirect().data(), _dom.get_background_geo().data());
-}
-template <class T, class M>
-inline
 geo_basic<T,M>
 geo_domain_rep<T,M>::get_background_domain() const
 {
@@ -117,7 +119,7 @@ template <class T, class M>
 geo_basic<T,M>
 compact (const geo_basic<T,M>& gamma)
 {
-    if (gamma.data().variant() != geo_abstract_base_rep<T>::geo_domain_indirect) {
+    if (gamma.variant() != geo_abstract_base_rep<T>::geo_domain_indirect) {
         return gamma;
     }
     /* allocate a new geo_rep object */
@@ -135,12 +137,12 @@ template <class T>								\
 const geo_basic<T,M>&								\
 geo_basic<T,M>::get_background_geo() const 					\
 {										\
-    if (base::data().variant() == geo_abstract_base_rep<T>::geo_domain) {	\
+    if (variant() == geo_abstract_base_rep<T>::geo_domain) {			\
       const geo_domain_rep<T,M>& dom 						\
        = dynamic_cast<const geo_domain_rep<T,M>&>(base::data());		\
       return dom.get_background_geo();						\
     }										\
-    if (base::data().variant() == geo_abstract_base_rep<T>::geo_domain_indirect) { \
+    if (variant() == geo_abstract_base_rep<T>::geo_domain_indirect) { 		\
       const geo_domain_indirect_rep<T,M>& dom					\
        = dynamic_cast<const geo_domain_indirect_rep<T,M>&>(base::data());	\
       return dom.get_background_geo();						\
@@ -153,16 +155,16 @@ template <class T>								\
 geo_basic<T,M>									\
 geo_basic<T,M>::get_background_domain() const 					\
 {										\
-    if (base::data().variant() == geo_abstract_base_rep<T>::geo_domain) {	\
+    if (variant() == geo_abstract_base_rep<T>::geo_domain) {			\
       const geo_domain_rep<T,M>& dom						\
        = dynamic_cast<const geo_domain_rep<T,M>&>(base::data()); 		\
       return dom.get_background_domain();					\
     }										\
-    if (base::data().variant() == geo_abstract_base_rep<T>::geo_domain_indirect) { \
+    if (variant() == geo_abstract_base_rep<T>::geo_domain_indirect) { 		\
       return *this;								\
     }										\
-    error_macro ("geo `"<<name()<<"' has no background domain");		\
-    return *this; /* not reached */						\
+    /* else classic geo: is its own background_geo */				\
+    return *this;								\
 }
 
 _RHEOLEF_get_bgd_geo(sequential)
