@@ -100,6 +100,8 @@ public:
     const array<node_type,M>& get_nodes() const { return _omega.get_nodes(); }
     const node_type& xmin () const { return _omega.xmin (); }
     const node_type& xmax () const { return _omega.xmax (); }
+    const T& hmin () const { return _omega.hmin (); }
+    const T& hmax () const { return _omega.hmax (); }
 
 // partially implemented geo_abstract interface:
 
@@ -133,6 +135,10 @@ public:
     size_type dis_locate(
 		const point_basic<T>&     x,
 		size_type                 dis_ie_guest = std::numeric_limits<size_type>::max()) const;
+    void locate (
+                const array<point_basic<T>,M>& x,
+                      array<size_type,M>&      dis_ie,
+                bool do_check = false) const;
     size_type seq_trace_move (
                 const point_basic<T>&     x,
                 const point_basic<T>&     v,
@@ -141,22 +147,27 @@ public:
                 const point_basic<T>&     x,
                 const point_basic<T>&     v,
                       point_basic<T>&     y) const;
-    void locate (
-                const array<point_basic<T>,M>& x,
-                      array<size_type,M>&      dis_ie,
-                bool do_check = false) const;
+    void trace_move (
+                const array<point_basic<T>,M>&     x,
+                const array<point_basic<T>,M>&     v,
+                      array<size_type, M>&         dis_ie,
+                      array<point_basic<T>,M>&     y) const;
     void trace_ray_boundary (
                 const array<point_basic<T>,M>&     x,
                 const array<point_basic<T>,M>&     v,
                       array<size_type, M>&         dis_ie,
                       array<point_basic<T>,M>&     y,
                 bool do_check = false) const;
-    void trace_move (
+    size_type seq_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const;
+    size_type dis_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const;
+    void nearest (
                 const array<point_basic<T>,M>&     x,
-                const array<point_basic<T>,M>&     v,
-                      array<size_type, M>&         dis_ie,
-                      array<point_basic<T>,M>&     y) const;
-
+                      array<point_basic<T>,M>&     x_nearest,
+                      array<size_type, M>&         dis_ie) const;
 protected:
 // data:
     domain_indirect_basic<M>   _indirect;
