@@ -216,6 +216,33 @@ geo_domain_indirect_base_rep<T,M>::trace_move (
 }
 template <class T, class M>
 typename geo_domain_indirect_base_rep<T,M>::size_type
+geo_domain_indirect_base_rep<T,M>::seq_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const
+{
+  error_macro ("domain.seq_nearest: no implemented");
+  return std::numeric_limits<size_type>::max();
+}
+template <class T, class M>
+typename geo_domain_indirect_base_rep<T,M>::size_type
+geo_domain_indirect_base_rep<T,M>::dis_nearest (
+                const point_basic<T>&    x,
+                      point_basic<T>&    x_nearest) const
+{
+  error_macro ("domain.dis_nearest: no implemented");
+  return std::numeric_limits<size_type>::max();
+}
+template <class T, class M>
+void
+geo_domain_indirect_base_rep<T,M>::nearest (
+                const array<point_basic<T>,M>&     x,
+                      array<point_basic<T>,M>&     x_nearest,
+                      array<size_type, M>&         dis_ie) const
+{
+  error_macro ("domain.nearest: no implemented");
+}
+template <class T, class M>
+typename geo_domain_indirect_base_rep<T,M>::size_type
 geo_domain_indirect_base_rep<T,M>::neighbour (size_type ie, size_type loc_iside) const
 {
   error_macro ("domain.neighbour: no implemented");
