@@ -33,9 +33,11 @@
 #include "rheolef/point_util.h"
 
 // internal includes:
+#ifdef _RHEOLEF_HAVE_CGAL
 #include "rheolef/cgal_traits.h"
 #include <CGAL/Segment_tree_k.h>
 #include <CGAL/Range_segment_tree_traits.h>
+#endif // _RHEOLEF_HAVE_CGAL
 
 namespace rheolef {
 
@@ -62,6 +64,7 @@ compute_bbox (const geo_base_rep<T,M>& omega, const geo_element& K, point_basic<
     }
   }
 }
+#ifdef _RHEOLEF_HAVE_CGAL
 // -----------------------------------------------------------------------------------
 // 2) missing in cgal: inspirated from "examples/RangeSegmentTrees/include/Tree_Traits.h"
 // -----------------------------------------------------------------------------------
@@ -97,7 +100,7 @@ class Segment_tree_map_traits_1 {
   public:
     bool operator()(Key_1 k1, Key_1 k2)
     {
-      return std::less<double>()(k1,k2);
+      return std::less<Float>()(k1,k2);
     }
   };
 
@@ -509,6 +512,74 @@ trace_macro ("locate done(2)");
 }
 #endif // _RHEOLEF_HAVE_MPI
 // ----------------------------------------------------------------------------
+// no CGAL: no locators
+// ----------------------------------------------------------------------------
+#else // _RHEOLEF_HAVE_CGAL
+template <class T, class M>
+geo_locate<T,M>::~geo_locate()
+{
+}
+template <class T, class M>
+typename geo_locate<T,M>::size_type
+geo_locate<T,M>::seq_locate (
+    const geo_base_rep<T,M>& omega,
+    const point_basic<T>&    x,
+    size_type                dis_ie_guest) const
+{
+  fatal_macro ("geo: locator not available (HINT: recompile Rheolef with the CGAL library)");
+  return 0;
+}
+template <class T, class M>
+typename geo_locate<T,M>::size_type
+geo_locate<T,M>::dis_locate (
+    const geo_base_rep<T,M>& omega,
+    const point_basic<T>&    x,
+    size_type                dis_ie_guest) const
+{
+  fatal_macro ("geo: locator not available (HINT: recompile Rheolef with the CGAL library)");
+  return 0;
+}
+template <class T, class M>
+typename geo_base_rep<T,M>::size_type
+geo_base_rep<T,M>::seq_locate (
+  const point_basic<T>& x,
+  size_type             dis_ie_guest) const
+{
+  fatal_macro ("geo: locator not available (HINT: recompile Rheolef with the CGAL library)");
+  return 0;
+}
+template <class T, class M>
+typename geo_base_rep<T,M>::size_type
+geo_base_rep<T,M>::dis_locate (
+  const point_basic<T>& x,
+  size_type             dis_ie_guest) const
+{
+  fatal_macro ("geo: locator not available (HINT: recompile Rheolef with the CGAL library)");
+  return 0;
+}
+template <class T>
+void
+geo_rep<T,sequential>::locate (
+  const array<point_basic<T>, sequential>& x,
+        array<size_type, sequential>&      dis_ie,
+  bool do_check) const
+{
+  fatal_macro ("geo: locator not available (HINT: recompile Rheolef with the CGAL library)");
+}
+#ifdef _RHEOLEF_HAVE_MPI
+template <class T>
+void
+geo_rep<T,distributed>::locate (
+  const array<point_basic<T>, distributed>& x,
+        array<size_type, distributed>&      dis_ie,
+	// dis_ie(ix) can contains a guest for x in K=omega[dis_ie(ix)]
+  bool do_check) const
+{
+  fatal_macro ("geo: locator not available (HINT: recompile Rheolef with the CGAL library)");
+}
+#endif // _RHEOLEF_HAVE_MPI
+#endif // _RHEOLEF_HAVE_CGAL
+// ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
 #define _RHEOLEF_instanciation(T,M)                                     \
