@@ -170,7 +170,7 @@ geo_element_renumbering_part1_new (
       }
     }
   }
-  K_ball.dis_entry_assembly (index_set_add_op<index_set,index_set>());
+  K_ball.dis_entry_assembly();
   // ------------------------------------------------------------------------
   // 2) import ball[ios_dis_iv] when dis_iv is referenced by a side S
   // ------------------------------------------------------------------------
@@ -378,7 +378,7 @@ geo_rep<T,distributed>::build_external_entities ()
         ext_node_set.insert (dis_inod);
 	if (loc_inod >= K.size()) continue;
         // then reports vertex to node: same owner
-	size_type dis_iv = dis_inod2dis_iv (dis_inod);
+	size_type dis_iv = base::dis_inod2dis_iv (dis_inod);
         check_macro (dis_iv < dis_nv, "vertex index "<< dis_iv <<" out of range [0:"<<dis_nv<<"[");
         check_macro (!vertex_ownership.is_owned (dis_iv), "strange bug: not owned (nod) but is_owned(ver)");
         ext_vertex_set.insert (dis_iv);
@@ -415,7 +415,7 @@ geo_rep<T,distributed>::set_element_side_index (size_type side_dim)
         for (size_type iloc = 0, nloc = K.size(); iloc < nloc; iloc++) {
           size_type dis_inod = K[iloc];
           if (! node_ownership.is_owned(dis_inod)) { // vertex ownership follows node ownership
-            size_type dis_iv = dis_inod2dis_iv (dis_inod);
+            size_type dis_iv = base::dis_inod2dis_iv (dis_inod);
             ext_iv_set.insert (dis_iv);
           }
         }
@@ -438,7 +438,7 @@ geo_rep<T,distributed>::set_element_side_index (size_type side_dim)
       for (size_type iloc = 0, nloc = S.size(); iloc < nloc; iloc++) {
         size_type dis_inod = S[iloc];
 	assert_macro (dis_inod < dis_nnod, "S={"<<S<<"}: "<<iloc<<"-node index " << dis_inod << " out of range [0:"<<dis_nnod<<"[");
-        size_type dis_iv = dis_inod2dis_iv (dis_inod);
+        size_type dis_iv = base::dis_inod2dis_iv (dis_inod);
         if (vertex_ownership.is_owned(dis_iv)) {
 	  size_type iv = dis_iv - first_dis_iv;
 	  ball [variant][iv] += dis_igev;
@@ -449,7 +449,7 @@ geo_rep<T,distributed>::set_element_side_index (size_type side_dim)
         }
       }
     }
-    ball [variant].dis_entry_assembly (index_set_add_op<index_set,index_set>());
+    ball [variant].dis_entry_assembly();
     // ------------------------------------------------------------------------
     // for all the dis_iv that are not handled by the current process
     // but are referenced by at least a side, get ext_ball[dis_iv]
@@ -521,13 +521,13 @@ geo_rep<T,distributed>::set_element_side_index (size_type side_dim)
   	  size_type S_variant = reference_element::variant (S_size, side_dim);
           size_type loc_jv0 = K.subgeo_local_vertex (side_dim, loc_isid, 0);
           S_inod [0] = K[loc_jv0];
-          S_iv   [0] = dis_inod2dis_iv (S_inod[0]);
+          S_iv   [0] = base::dis_inod2dis_iv (S_inod[0]);
           const array<index_set,distributed>& ball_S_variant = ball [S_variant];
           index_set igev_set = ball_S_variant.dis_at (S_iv[0]);
   	  for (size_type sid_jloc = 1, sid_nloc = S_size; sid_jloc < sid_nloc; sid_jloc++) { 
             size_type loc_jv = K.subgeo_local_vertex (side_dim, loc_isid, sid_jloc);
             S_inod [sid_jloc] = K[loc_jv];
-            S_iv   [sid_jloc] = dis_inod2dis_iv (S_inod[sid_jloc]);
+            S_iv   [sid_jloc] = base::dis_inod2dis_iv (S_inod[sid_jloc]);
             const index_set& ball_jv = ball_S_variant.dis_at (S_iv[sid_jloc]);
             igev_set.inplace_intersection (ball_jv);
           }
@@ -986,7 +986,6 @@ geo_rep<T,distributed>::load (
 // instanciation in library
 // ----------------------------------------------------------------------------
 template class geo_rep<Float,distributed>;
-template geo_basic<Float,distributed> compact (const geo_basic<Float,distributed>&);
 
 } // namespace rheolef
 #endif // _RHEOLEF_HAVE_MPI
