@@ -29,6 +29,7 @@
 #include "geo_partition_scotch.h"
 #elif defined(_RHEOLEF_HAVE_PARMETIS)
 #include <parmetis.h>
+typedef idx_t idxtype; // change from previous parmetis versions
 #endif // _RHEOLEF_HAVE_PARMETIS
 
 namespace rheolef {
@@ -90,22 +91,22 @@ geo_mpi_partition (
       }
     }
   }
-  int wgtflag = 0;
-  int numflag = 0;
-  int mgcnum = map_dim;  
-  int ncon = 1;  
-  int nparts = nproc;  
+  idxtype wgtflag = 0;
+  idxtype numflag = 0;
+  idxtype mgcnum = map_dim;  
+  idxtype ncon = 1;  
+  idxtype nparts = nproc;  
   std::vector<float> tpwgts (nparts*ncon);
   std::fill (tpwgts.begin(), tpwgts.end(), 1./nparts);
   float ubvec [12];
   std::fill (ubvec, ubvec+ncon, 1.05);
-  int options [10];
+  idxtype options [10];
   options[0] = 1;
   const int pvm3_option_dbglvl = 1;
   const int pvm3_option_seed   = 2;
   options[pvm3_option_dbglvl] = 0; // otherwise: timming print to stdout...
   options[pvm3_option_seed]   = 0;
-  int edgecut;
+  idxtype edgecut;
   MPI_Comm raw_comm = comm;
   idxtype *elmwgt = 0; 
 #if defined(_RHEOLEF_HAVE_SCOTCH)
