@@ -41,6 +41,7 @@
 // intel C++ has less 2011 standard features
 // thus re-config it!
 // ------------------------------------------
+#ifdef _RHEOLEF_HAVE_CGAL
 #ifdef _RHEOLEF_HAVE_INTEL_CXX
 #include <CGAL/config.h>
 
@@ -60,9 +61,11 @@
 #include <CGAL/AABB_tree.h>
 #include <CGAL/AABB_traits.h>
 #include <CGAL/AABB_triangle_primitive.h>
+#endif // _RHEOLEF_HAVE_CGAL
 
 namespace rheolef {
 
+#ifdef _RHEOLEF_HAVE_CGAL
 // ---------------------------------------------------------------------
 // 0) utility: nearest hit over processors
 // ---------------------------------------------------------------------
@@ -622,7 +625,8 @@ geo_rep<T,distributed>::trace_ray_boundary (
         //  - this works in 1d when Omega is simply connected
         //  - this works in 2d & 3d when Omega is a rectangle or a paralellotope...
         // TODO: boundary projection in the multi-connected case with: y = omega.nearest(x+v)
-	T eps = 1e3*std::numeric_limits<T>::epsilon();
+	// T eps = 1e3*std::numeric_limits<T>::epsilon();
+	T eps = sqrt(std::numeric_limits<T>::epsilon());
         for (size_t k = 0; k < base::_dimension; k++) {
           y[i][k] = std::max (std::min (y[i][k]+eps, base::_xmax[k]-eps), base::_xmin[k]);
         }
@@ -725,6 +729,61 @@ geo_rep<T,distributed>::trace_ray_boundary (
 }
 #endif // _RHEOLEF_HAVE_MPI
 // ----------------------------------------------------------------------------
+// CGAL lib is missing
+// ----------------------------------------------------------------------------
+#else // _RHEOLEF_HAVE_CGAL
+template <class T, class M>
+geo_trace_ray_boundary<T,M>::~geo_trace_ray_boundary()
+{
+}
+template <class T, class M>
+bool
+geo_trace_ray_boundary<T,M>::seq_trace_ray_boundary (
+  const geo_base_rep<T,M>&  omega,
+  const point_basic<T>&     x,
+  const point_basic<T>&     v,
+        point_basic<T>&     y) const
+{
+  fatal_macro ("geo: trace_ray_boundary not available (HINT: recompile Rheolef with the CGAL library)");
+  return false;
+}
+template <class T, class M> 
+bool    
+geo_trace_ray_boundary<T,M>::dis_trace_ray_boundary (
+  const geo_base_rep<T,M>&  omega,
+  const point_basic<T>&     x, 
+  const point_basic<T>&     v,
+        point_basic<T>&     y) const
+{
+  fatal_macro ("geo: trace_ray_boundary not available (HINT: recompile Rheolef with the CGAL library)");
+  return false;
+}
+template <class T>
+void
+geo_rep<T,sequential>::trace_ray_boundary (
+                const array<point_basic<T>,sequential>&     x,
+                const array<point_basic<T>,sequential>&     v,
+                      array<size_type, sequential>&         dis_ie,
+                      array<point_basic<T>,sequential>&     y,
+		bool  do_check) const
+{
+  fatal_macro ("geo: trace_ray_boundary not available (HINT: recompile Rheolef with the CGAL library)");
+}
+#ifdef _RHEOLEF_HAVE_MPI
+template <class T>
+void
+geo_rep<T,distributed>::trace_ray_boundary (
+                const array<point_basic<T>,distributed>&     x,
+                const array<point_basic<T>,distributed>&     v,
+                      array<size_type, distributed>&         dis_ie,
+                      array<point_basic<T>,distributed>&     y,
+		bool  do_check) const
+{
+  fatal_macro ("geo: trace_ray_boundary not available (HINT: recompile Rheolef with the CGAL library)");
+}
+#endif // _RHEOLEF_HAVE_MPI
+#endif // _RHEOLEF_HAVE_CGAL
+// ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
 template class geo_rep<Float,sequential>;
