@@ -449,29 +449,32 @@ struct generic_binary_traits<minus> {
 // --------
 /*
     result: when arg1 & arg2 are given
-      1\2 s  v  t
-       s  s  v  t
-       v  v  E  E
-       t  t  v  t
+      1\2 s  v  t  t3
+       s  s  v  t  t3
+       v  v  E  E  E
+       t  t  v  t  E
+       t3 t3 t  t3 E
 
     arg1: deduced when arg2 & result are known
-      2\r s  v  t
-       s  s  v  t
-       v  E (st)E
-       t  E  E (st)  st: indetermine'
+      2\r s  v  t  t3
+       s  s  v  t  t3
+       v  E (st)t3 E
+       t  E  E (st)t3     st: indetermine'
+       t3 E  E  E  s
 
     arg2: deduced when arg1 & result are known
-      1\r s  v  t
-       s  s  v  t
-       v  E  s  E
-       t  E  v (st)  st: indeterminé
+      1\r s  v  t  t3
+       s  s  v  t  t3
+       v  E  s  E  E
+       t  E  v (st)E      st: indetermine'
+       t3 E  E  v (st)
 
-      il n'y a que deux cas ou l'arg1 est indeterminé :
+      il n'y a que deux cas ou l'arg1 est indetermine' :
             quand arg2=v et res_t=v alors arg1=s ou t
             quand arg2=t et res_t=t alors arg1=s ou t
       dans tous les autres cas, on deduit l'arg1
 
-      il n'y a qu'un cas ou l'arg2 est indeterminé :
+      il n'y a qu'un cas ou l'arg2 est indetermine' :
             quand arg1=t et res_t=t alors arg2=s ou t
       dans tous les autres cas, on deduit l'arg2
 */
@@ -528,6 +531,47 @@ struct multiplies_result<tensor_basic<S1>, tensor_basic<S2> > {
   typedef typename promote<S1,S2>::type S;
   typedef tensor_basic<S> type;
 };
+// s*t3 -> t3
+template <class S>
+struct multiplies_result<S, tensor3_basic<S> > {
+  typedef tensor3_basic<S> type;
+};
+// t*s -> t
+template <class S>
+struct multiplies_result<tensor3_basic<S>, S> {
+  typedef tensor3_basic<S> type;
+};
+// t3*v -> t
+template <class S1, class S2>
+struct multiplies_result<tensor3_basic<S1>, point_basic<S2> > {
+  typedef typename promote<S1,S2>::type S;
+  typedef tensor_basic<S> type;
+};
+// t3*t -> t3
+template <class S1, class S2>
+struct multiplies_result<tensor3_basic<S1>, tensor_basic<S2> > {
+  typedef typename promote<S1,S2>::type S;
+  typedef tensor3_basic<S> type;
+};
+// v*t3 -> err
+template <class S1, class S2>
+struct multiplies_result<point_basic<S1>, tensor3_basic<S2> > {
+  typedef typename promote<S1,S2>::type S;
+  typedef binop_error<details::multiplies,point_basic<S1>,tensor3_basic<S2>,undeterminated_basic<S> > type;
+};
+// t*t3 -> err
+template <class S1, class S2>
+struct multiplies_result<tensor_basic<S1>, tensor3_basic<S2> > {
+  typedef typename promote<S1,S2>::type S;
+  typedef binop_error<details::multiplies,tensor_basic<S1>,tensor3_basic<S2>,undeterminated_basic<S> > type;
+};
+// t3*t3 -> err
+template <class S1, class S2>
+struct multiplies_result<tensor3_basic<S1>, tensor3_basic<S2> > {
+  typedef typename promote<S1,S2>::type S;
+  typedef binop_error<details::multiplies,tensor3_basic<S1>,tensor3_basic<S2>,undeterminated_basic<S> > type;
+};
+
 struct multiplies {
   template <class T1, class T2>
   typename multiplies_result<T1,T2>::type
@@ -557,6 +601,11 @@ struct generic_binary_traits<multiplies> {
   struct first_argument_hint<A2,tensor_basic<R> > {
     typedef tensor_basic<typename promote<A2,R>::type> type;
   };
+  // ?*s=t3 => a1=t3
+  template <class A2, class R>
+  struct first_argument_hint<A2,tensor3_basic<R> > {
+    typedef tensor3_basic<typename promote<A2,R>::type> type;
+  };
   // ?*v=s => a1=err
   template <class A2, class R>
   struct first_argument_hint<point_basic<A2>,R> {
@@ -567,10 +616,15 @@ struct generic_binary_traits<multiplies> {
   struct first_argument_hint<point_basic<A2>,point_basic<R> > {
     typedef undeterminated_basic<typename promote<A2,R>::type> type;
   };
-  // ?*v=t => a1=err
+  // ?*v=t => a1=t3
   template <class A2, class R>
   struct first_argument_hint<point_basic<A2>,tensor_basic<R> > {
-    typedef binop_error<details::multiplies,undeterminated_basic<A2>,tensor_basic<A2>,tensor_basic<R> > type;
+    typedef tensor3_basic<typename promote<A2,R>::type> type;
+  };
+  // ?*v=t3 => a1=err
+  template <class A2, class R>
+  struct first_argument_hint<point_basic<A2>,tensor3_basic<R> > {
+    typedef binop_error<details::multiplies,undeterminated_basic<A2>,point_basic<A2>,tensor3_basic<R> > type;
   };
   // ?*t=s => a1=err
   template <class A2, class R>
@@ -587,6 +641,31 @@ struct generic_binary_traits<multiplies> {
   struct first_argument_hint<tensor_basic<A2>,tensor_basic<R> > {
     typedef undeterminated_basic<typename promote<A2,R>::type> type;
   };
+  // ?*t=t3 => a1=t3
+  template <class A2, class R>
+  struct first_argument_hint<tensor_basic<A2>,tensor3_basic<R> > {
+    typedef tensor3_basic<typename promote<A2,R>::type> type;
+  };
+  // ?*t3=s => a1=err
+  template <class A2, class R>
+  struct first_argument_hint<tensor3_basic<A2>,R> {
+    typedef binop_error<details::multiplies,undeterminated_basic<A2>,tensor3_basic<A2>,R> type;
+  };
+  // ?*t3=v => a1=err
+  template <class A2, class R>
+  struct first_argument_hint<tensor3_basic<A2>,point_basic<R> > {
+    typedef binop_error<details::multiplies,undeterminated_basic<A2>,tensor3_basic<A2>,point_basic<R> > type;
+  };
+  // ?*t3=t => a1=err
+  template <class A2, class R>
+  struct first_argument_hint<tensor3_basic<A2>,tensor_basic<R> > {
+    typedef binop_error<details::multiplies,undeterminated_basic<A2>,tensor3_basic<A2>,tensor_basic<R> > type;
+  };
+  // ?*t3=t3 => a1=s
+  template <class A2, class R>
+  struct first_argument_hint<tensor3_basic<A2>,tensor3_basic<R> > {
+    typedef typename promote<A2,R>::type type;
+  };
   // --------------------------
   // hint arg2 type
   // --------------------------
@@ -605,6 +684,11 @@ struct generic_binary_traits<multiplies> {
   struct second_argument_hint<A1,tensor_basic<R> > {
     typedef tensor_basic<typename promote<A1,R>::type> type;
   };
+  // s*?=t3 => a1=t3
+  template <class A1, class R>
+  struct second_argument_hint<A1,tensor3_basic<R> > {
+    typedef tensor3_basic<typename promote<A1,R>::type> type;
+  };
   // v*?=s => a1=err
   template <class A1, class R>
   struct second_argument_hint<point_basic<A1>,R> {
@@ -635,6 +719,36 @@ struct generic_binary_traits<multiplies> {
   struct second_argument_hint<tensor_basic<A1>,tensor_basic<R> > {
     typedef undeterminated_basic<typename promote<A1,R>::type> type;
   };
+  // v*?=t3 => a1=err
+  template <class A1, class R>
+  struct second_argument_hint<point_basic<A1>,tensor3_basic<R> > {
+    typedef binop_error<details::multiplies,point_basic<A1>,undeterminated_basic<A1>,tensor3_basic<R> > type;
+  };
+  // t*?=t3 => a1=err
+  template <class A1, class R>
+  struct second_argument_hint<tensor_basic<A1>,tensor3_basic<R> > {
+    typedef binop_error<details::multiplies,tensor_basic<A1>,undeterminated_basic<A1>,tensor3_basic<R> > type;
+  };
+  // t3*?=s => a1=err
+  template <class A1, class R>
+  struct second_argument_hint<tensor3_basic<A1>,R > {
+    typedef binop_error<details::multiplies,tensor_basic<A1>,undeterminated_basic<A1>,R> type;
+  };
+  // t3*?=v => a1=err
+  template <class A1, class R>
+  struct second_argument_hint<tensor3_basic<A1>,point_basic<R> > {
+    typedef binop_error<details::multiplies,tensor_basic<A1>,undeterminated_basic<A1>,point_basic<R> > type;
+  };
+  // t3*?=t => a1=v
+  template <class A1, class R>
+  struct second_argument_hint<tensor3_basic<A1>,tensor_basic<R> > {
+    typedef point_basic<typename promote<A1,R>::type> type;
+  };
+  // t3*?=t3 => a1={st}
+  template <class A1, class R>
+  struct second_argument_hint<tensor3_basic<A1>,tensor3_basic<R> > {
+    typedef undeterminated_basic<typename promote<A1,R>::type> type;
+  };
   // --------------------------
   // hint interface
   // --------------------------
@@ -693,6 +807,13 @@ struct generic_binary_traits<multiplies> {
     typedef undeterminated_basic<A2> second_argument_type;
     typedef tensor_basic<R>          result_type;
   };
+  // ?*? -> t3 : deduce (a1,a2)={(s,t3),(t3,s),(t3,t)}=?
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,undeterminated_basic<A2>,tensor3_basic<R> > {
+    typedef undeterminated_basic<A1> first_argument_type;
+    typedef undeterminated_basic<A2> second_argument_type;
+    typedef tensor3_basic<R>          result_type;
+  };
   // s*? -> ? : deduce (a2,r)={(s,s),(v,v),(t,t)}=?
   template <class A1, class A2, class R>
   struct hint<A1,undeterminated_basic<A2>,undeterminated_basic<R> > {
@@ -714,6 +835,13 @@ struct generic_binary_traits<multiplies> {
     typedef undeterminated_basic<A2> second_argument_type;
     typedef undeterminated_basic<R>  result_type;
   };
+  // t3*? -> ? : deduce (a2,r)={(s,t3),(v,t),(t3,t3)}=?
+  template <class A1, class A2, class R>
+  struct hint<tensor3_basic<A1>,undeterminated_basic<A2>,undeterminated_basic<R> > {
+    typedef tensor3_basic<A1>        first_argument_type;
+    typedef undeterminated_basic<A2> second_argument_type;
+    typedef undeterminated_basic<R>  result_type;
+  };
   // ?*s -> ? : deduce (a1,r)={(s,s),(v,v),(t,t)}=?
   template <class A1, class A2, class R>
   struct hint<undeterminated_basic<A1>,A2,undeterminated_basic<R> > {
@@ -735,6 +863,13 @@ struct generic_binary_traits<multiplies> {
     typedef tensor_basic<A2>         second_argument_type;
     typedef undeterminated_basic<R>  result_type;
   };
+  // ?*t3 -> ? : deduce (a1,r)=(s,t3)
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,tensor3_basic<A2>,undeterminated_basic<R> > {
+    typedef A1                       first_argument_type;
+    typedef tensor3_basic<A2>        second_argument_type;
+    typedef tensor3_basic<R>         result_type;
+  };
   // none types are known
   // -----------------------
   // ?*? -> ? : deduce (a1,a2,r)=?
@@ -772,7 +907,7 @@ template <class T>
 struct divides_result<point_basic<T>,T> {
   typedef point_basic<T> type;
 };
-template <class T> 
+template <class T>
 struct divides_result<tensor_basic<T>,T> {
   typedef tensor_basic<T> type;
 };
@@ -794,6 +929,34 @@ struct divides_result<tensor_basic<T1>, point_basic<T2> > {
   typedef typename promote<T1,T2>::type S;
   typedef binop_error<details::divides,tensor_basic<T1>,point_basic<T2>,undeterminated_basic<S> > type;
 };
+template <class T>
+struct divides_result<tensor3_basic<T>,T> {
+  typedef tensor3_basic<T> type;
+};
+template <class T>
+struct divides_result<T, tensor3_basic<T> > {
+  typedef binop_error<details::divides,T,tensor3_basic<T>,undeterminated_basic<T> > type;
+};
+template <class T1, class T2>
+struct divides_result<point_basic<T1>, tensor3_basic<T2> > {
+  typedef typename promote<T1,T2>::type S;
+  typedef binop_error<details::divides,point_basic<T1>,tensor3_basic<T2>,undeterminated_basic<S> > type;
+};
+template <class T1, class T2>
+struct divides_result<tensor_basic<T1>, tensor3_basic<T2> > {
+  typedef typename promote<T1,T2>::type S;
+  typedef binop_error<details::divides,tensor_basic<T1>,tensor3_basic<T2>,undeterminated_basic<S> > type;
+};
+template <class T1, class T2>
+struct divides_result<tensor3_basic<T1>, point_basic<T2> > {
+  typedef typename promote<T1,T2>::type S;
+  typedef binop_error<details::divides,tensor3_basic<T1>,point_basic<T2>,undeterminated_basic<S> > type;
+};
+template <class T1, class T2>
+struct divides_result<tensor3_basic<T1>, tensor_basic<T2> > {
+  typedef typename promote<T1,T2>::type S;
+  typedef binop_error<details::divides,tensor3_basic<T1>,tensor_basic<T2>,undeterminated_basic<S> > type;
+};
 
 struct divides {
   template <class T1, class T2>
@@ -1036,9 +1199,37 @@ struct generic_binary_traits<dot_> {
 // ------------
 // ddot(x,y)
 // ------------
+/*
+    result: when arg1 & arg2 are given
+      1\2 t  t4
+       t  s  t
+       t4 t  E
+*/
+struct ddot_;
+template <class A1, class A2>
+struct ddot_result {
+    typedef typename promote<typename float_traits<A1>::type, typename float_traits<A2>::type>::type S;
+    typedef binop_error<ddot_,A1,A2,undeterminated_basic<S> > type;
+};
+template <class A1, class A2>
+struct ddot_result<tensor_basic<A1>,tensor_basic<A2> > {
+    typedef typename promote<typename float_traits<A1>::type, typename float_traits<A2>::type>::type S;
+    typedef S type;
+};
+template <class A1, class A2>
+struct ddot_result<tensor4_basic<A1>,tensor_basic<A2> > {
+    typedef typename promote<typename float_traits<A1>::type, typename float_traits<A2>::type>::type S;
+    typedef tensor_basic<S> type;
+};
+template <class A1, class A2>
+struct ddot_result<tensor_basic<A1>,tensor4_basic<A2> > {
+    typedef typename promote<typename float_traits<A1>::type, typename float_traits<A2>::type>::type S;
+    typedef tensor_basic<S> type;
+};
 struct ddot_ {
-  template<class T>
-  T operator() (const tensor_basic<T>& x, const tensor_basic<T>& y) const { return ddot(x,y); }
+  template<class A1, class A2>
+  typename ddot_result<A1,A2>::type
+  operator() (const A1& x, const A2& y) const { return ddot(x,y); }
 };
 template<>
 struct generic_binary_traits<ddot_> {
@@ -1046,56 +1237,121 @@ struct generic_binary_traits<ddot_> {
   valued_tag (space_constant::valued_type tag1, space_constant::valued_type tag2) { 
     return space_constant::scalar;
   }
-  template <class Arg1, class Arg2>
-  struct result_hint {
-    typedef typename promote<typename float_traits<Arg1>::type, typename float_traits<Arg2>::type>::type type;
-  };
   template <class A1, class A2, class R>
   struct hint {
-    typedef typename scalar_traits<A1>::type  S1;
-    typedef typename scalar_traits<A2>::type  S2;
-    typedef typename scalar_traits<R>::type   S;
-    typedef typename mpl::and_<
-              mpl::or_<
-                details::is_tensor<A1>
-               ,is_undeterminated<A1>
-      	      >
-             ,mpl::or_<
-                details::is_tensor<A2>
-               ,is_undeterminated<A2>
-      	      >
-             ,mpl::or_<
-                details::is_scalar<R>
-               ,is_undeterminated<R>
-      	     >
-	   >::type  				is_good;
+    typedef binop_error<ddot_,A1,A2,R> T;
+    typedef typename mpl::if_<is_undeterminated<A1>,T,A1>::type first_argument_type;
+    typedef typename mpl::if_<is_undeterminated<A2>,T,A2>::type second_argument_type;
+    typedef typename mpl::if_<is_undeterminated<R>,T,R>::type   result_type;
+  };
+  // a:b -> ?
+  template <class A1, class A2, class R>
+  struct hint<A1,A2,undeterminated_basic<R> > {
+    typedef typename ddot_result<A1,A2>::type result_type;
     typedef typename mpl::if_<
-	     is_undeterminated<A1>
-	    ,typename mpl::if_<
-	       is_good
-              ,tensor_basic<S1>
-              ,binop_error<details::ddot_,A1,A2,R>
-             >::type
-            ,A1
-           >::type                              first_argument_type;
+	typename mpl::and_<is_undeterminated<A1>,is_error<result_type> >::type
+       ,result_type,A1>::type                 first_argument_type;
     typedef typename mpl::if_<
-	     is_undeterminated<A2>
-	    ,typename mpl::if_<
-	       is_good
-              ,tensor_basic<S2>
-              ,binop_error<details::ddot_,A1,A2,R>
-             >::type
-            ,A2
-           >::type                              second_argument_type;
-    typedef typename mpl::if_<
-	     is_undeterminated<R>
-	    ,typename mpl::if_<
-	       is_good
-              ,S
-              ,binop_error<details::ddot_,A1,A2,R>
-             >::type
-            ,R
-           >::type                              result_type;
+	typename mpl::and_<is_undeterminated<A2>,is_error<result_type> >::type
+       ,result_type,A2>::type                 second_argument_type;
+  };
+  // ====================
+  // A1 xor A2 is unknown
+  // ====================
+  // ?:t -> s => a=t
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,tensor_basic<A2>,R> {
+    typedef binop_error<ddot_,A1,A2,R> E;
+    typedef typename mpl::if_<is_scalar<R>,tensor_basic<A1>,E>::type
+                                              first_argument_type;
+    typedef tensor_basic<A2>                  second_argument_type;
+    typedef R                                 result_type;
+  };
+  // t:? -> s => b=t
+  template <class A1, class A2, class R>
+  struct hint<tensor_basic<A1>,undeterminated_basic<A2>,R> {
+    typedef binop_error<ddot_,A1,A2,R> E;
+    typedef tensor_basic<A1>                  first_argument_type;
+    typedef typename mpl::if_<is_scalar<R>,tensor_basic<A2>,E>::type
+                                              second_argument_type;
+    typedef R                                 result_type;
+  };
+  // ?:t4 -> t => a=t
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,tensor4_basic<A2>,tensor_basic<R> > {
+    typedef tensor_basic<A1>                  first_argument_type;
+    typedef tensor4_basic<A2>                 second_argument_type;
+    typedef tensor_basic<R>                   result_type;
+  };
+  // t4:? -> t => b=t
+  template <class A1, class A2, class R>
+  struct hint<tensor4_basic<A1>,undeterminated_basic<A2>,tensor_basic<R> > {
+    typedef tensor4_basic<A1>                 first_argument_type;
+    typedef tensor_basic<A2>                  second_argument_type;
+    typedef tensor_basic<R>                   result_type;
+  };
+  // =====================
+  // A1 and A2 are unknown
+  // =====================
+  // ?:? -> s => a=b=t
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,undeterminated_basic<A2>,R> {
+    typedef binop_error<ddot_,A1,A2,R> E;
+    typedef typename mpl::if_<is_scalar<R>,tensor_basic<A1>,E>::type
+                                              first_argument_type;
+    typedef typename mpl::if_<is_scalar<R>,tensor_basic<A2>,E>::type
+                                              second_argument_type;
+    typedef R                                 result_type;
+  };
+  // ?:? -> t => (a,b)={(t,t4),(t4,t)}
+  // =====================
+  // A1 and R are unknown
+  // =====================
+  // ?:t  -> ? => (a,r)={(t,s),(t4,t)}
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,tensor_basic<A2>,undeterminated_basic<R> > {
+    typedef undeterminated_basic<A1>          first_argument_type;
+    typedef tensor_basic<A2>                  second_argument_type;
+    typedef undeterminated_basic<R>           result_type;
+  };
+  // ?:t4 -> ? => (a,r)=(t,t)
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,tensor4_basic<A2>,undeterminated_basic<R> > {
+    typedef tensor_basic<A1>                  first_argument_type;
+    typedef tensor4_basic<A2>                 second_argument_type;
+    typedef tensor_basic<R>                   result_type;
+  };
+  // =====================
+  // A2 and R are unknown
+  // =====================
+  // t:?  -> ? => (b,r)={(t,s),(t4,t)}
+  template <class A1, class A2, class R>
+  struct hint<tensor_basic<A1>,undeterminated_basic<A2>,undeterminated_basic<R> > {
+    typedef tensor_basic<A1>                  first_argument_type;
+    typedef undeterminated_basic<A2>          second_argument_type;
+    typedef undeterminated_basic<R>           result_type;
+  };
+  // t4:? -> ? => (b,r)=(t,t)
+  template <class A1, class A2, class R>
+  struct hint<tensor4_basic<A1>,undeterminated_basic<A2>,undeterminated_basic<R> > {
+    typedef tensor4_basic<A1>                 first_argument_type;
+    typedef tensor_basic<A2>                  second_argument_type;
+    typedef tensor_basic<R>                   result_type;
+  };
+  // ========================
+  // A1, A2 and R are unknown
+  // ========================
+  // ?:? -> ? => a,b,r=?
+  template <class A1, class A2, class R>
+  struct hint<undeterminated_basic<A1>,undeterminated_basic<A2>,undeterminated_basic<R> > {
+    typedef undeterminated_basic<A1>          first_argument_type;
+    typedef undeterminated_basic<A2>          second_argument_type;
+    typedef undeterminated_basic<R>           result_type;
+  };
+  // short interface
+  template <class Arg1, class Arg2>
+  struct result_hint {
+    typedef typename hint<Arg1,Arg2,undeterminated_basic<Float> >::result_type type;
   };
   typedef mpl::true_ is_symmetric;
 };
