@@ -19,7 +19,8 @@
 /// 
 /// =========================================================================
 #include "rheolef/piola.h"
-
+#include "rheolef/inv_piola.h"
+#include "rheolef/damped_newton.h"
 namespace rheolef {
 
 // =========================================================================
@@ -394,26 +395,34 @@ template<class T, class M>
 point_basic<T>
 inverse_piola_transformation (
   const geo_basic<T,M>&         omega,
-  reference_element             hat_K,
+  const reference_element&      hat_K,
   const std::vector<size_t>&    dis_inod,
   const point_basic<T>&         x)
 {
   check_macro (omega.order() == 1, "inverse piola: mesh order > 1: not yet");
-  switch (hat_K.variant()) {
-   case reference_element::e: return inv_piola_e (x, omega.dis_node(dis_inod [0]),
-                                                     omega.dis_node(dis_inod [1])); 
-   case reference_element::t: return inv_piola_t (x, omega.dis_node(dis_inod [0]),
-                                                     omega.dis_node(dis_inod [1]),
-                                                     omega.dis_node(dis_inod [2]));
-   case reference_element::T: return inv_piola_T (x, omega.dis_node(dis_inod [0]),
-                                                     omega.dis_node(dis_inod [1]),
-                                                     omega.dis_node(dis_inod [2]),
-                                                     omega.dis_node(dis_inod [3]));
-   default: {
-	fatal_macro ("inverse piola: non-linear transformation on '"<<hat_K.name()<<"' element: not yet");
-        return point_basic<T>();
-   }
+  if (omega.order() == 1) {
+    switch (hat_K.variant()) {
+      case reference_element::e: return inv_piola_e (x, omega.dis_node(dis_inod [0]),
+                                                        omega.dis_node(dis_inod [1])); 
+      case reference_element::t: return inv_piola_t (x, omega.dis_node(dis_inod [0]),
+                                                        omega.dis_node(dis_inod [1]),
+                                                        omega.dis_node(dis_inod [2]));
+      case reference_element::T: return inv_piola_T (x, omega.dis_node(dis_inod [0]),
+                                                        omega.dis_node(dis_inod [1]),
+                                                        omega.dis_node(dis_inod [2]),
+                                                        omega.dis_node(dis_inod [3]));
+    }
   }
+  // non-linear transformation: q,P,H or high order > 1 => use Newton
+  inv_piola<T> F;
+  F.reset (omega, hat_K, dis_inod);
+  F.set_x (x);
+  point_basic<T> hat_x = F.initial();
+  size_t max_iter = 500, n_iter = max_iter;
+  T tol = std::numeric_limits<Float>::epsilon(), r = tol;
+  int status = damped_newton (F, hat_x, r, n_iter);
+  check_macro (status == 0, "inv_piola: newton failed (residue="<<r<<", n_iter="<<n_iter<<")");
+  return hat_x;
 }
 // ----------------------------------------------------------------------------
 // instanciation in library
@@ -458,7 +467,7 @@ template								\
 point_basic<T>								\
 inverse_piola_transformation (						\
   const geo_basic<T,M>&         omega,					\
-  reference_element             hat_K,					\
+  const reference_element&      hat_K,					\
   const std::vector<size_t>&    dis_inod,				\
   const point_basic<T>&         x);					\
 template								\
