@@ -36,70 +36,6 @@
 
 namespace rheolef { 
 
-// --------------------------------------------------------------------------
-// utility to determine whether a template arg is a function or a constant
-// --------------------------------------------------------------------------
-namespace details {
-  // build a function that returns a constant
-  template<class T1, class T2>
-  struct f_constant : field_functor<f_constant<T1,T2>,T2 > {
-    const T2& operator() (const T1& x) const { return c; }
-    f_constant (const T2& c0) : c(c0) {}
-    T2 c;
-  };
-  template<class F> struct is_vector_function :
-    boost::mpl::and_ <
-        boost::is_class<F>
-       ,is_point<typename F::result_type>
-    > {};
-  template<class T> struct is_vector_function<point_basic<T> (const point_basic<T>)>
-  	: boost::mpl::true_ {};
-
-  template<class E> struct is_expr                    : boost::mpl::false_ {};
-  template<class E> struct is_expr<field_nonlinear_expr<E> > : boost::mpl::true_ {};
-
-  template<class C>
-  struct is_constant :
-        boost::mpl::or_ <
-          is_scalar<C>
-         ,is_point<C>
-        > {};
-
-  template<class F>
-  struct is_function :
-    boost::mpl::and_ <
-      boost::mpl::or_ <
-        boost::is_class<F>
-       ,boost::is_pointer<F>
-       ,boost::is_function<F>
-      >
-     ,boost::mpl::not_ <
-        boost::mpl::or_ <
-          is_scalar<F>
-         ,is_point<F>
-         ,is_expr<F>
-        >
-      >
-    >
-  {};
-  template<class F>
-  struct result_type {
-    typedef typename F::result_type type;
-  };
-  template<class T, class R>
-  struct result_type<R(const point_basic<T>&)> {
-    typedef R type;
-  };
-  template<class T, class R>
-  struct result_type<R (*)(const point_basic<T>&)> {
-    typedef R type;
-  };
-  template<class Constant>
-  struct constant_promote { typedef Constant type; };
-  template<>        struct constant_promote<int>    { typedef Float type; };
-  template<>        struct constant_promote<size_t> { typedef Float type; };
-    
-} // namespace details
 // --------------------------------------------------------------------------
 // generic implementation
 // --------------------------------------------------------------------------
@@ -301,7 +237,7 @@ riesz (
     const space_basic<T,M>&       Xh,
     const Function&               f,
     const quadrature_option_type& qopt
-       = quadrature_option_type(quadrature_option_type::max_family,0))
+       = quadrature_option_type(quadrature_option_type::max_family))
 //>riesz:
 {
   return riesz_tag (Xh, f, qopt, Xh.get_geo(),
@@ -315,7 +251,7 @@ riesz (
     const Function&               f,
     const geo_basic<T,M>&         dom,
     const quadrature_option_type& qopt
-       = quadrature_option_type(quadrature_option_type::max_family,0))
+       = quadrature_option_type(quadrature_option_type::max_family))
 //>riesz:
 {
   return riesz_tag (Xh, f, qopt, dom,
@@ -329,7 +265,7 @@ riesz (
     const Function&               f,
     std::string                   dom_name,
     const quadrature_option_type& qopt
-       = quadrature_option_type(quadrature_option_type::max_family,0))
+       = quadrature_option_type(quadrature_option_type::max_family))
 //>riesz:
 {
   const geo_basic<T,M>& omega = Xh.get_geo();
@@ -343,7 +279,7 @@ riesz (
     const Function&               f,
     const band_basic<T,M>&        gh,
     const quadrature_option_type& qopt
-       = quadrature_option_type(quadrature_option_type::max_family,0))
+       = quadrature_option_type(quadrature_option_type::max_family))
 //>riesz:
 {
   return riesz_tag (Xh, f, qopt, gh,
