@@ -30,7 +30,7 @@ struct rounder_type : std::unary_function<T,T> {
   rounder_type (const T& prec) : _prec(prec) {}
   T operator() (const T& x) const {
      // use floor : std::round() is non-standard (INTEL C++)
-     T value = _prec*std::floor(x/_prec + 0.5);
+     T value = _prec*floor(x/_prec + 0.5);
      if (1+value == 1) value = 0;
      return value;
   }
@@ -40,7 +40,7 @@ template <class T>
 struct floorer_type : std::unary_function<T,T> {
   floorer_type (const T& prec) : _prec(prec) {}
   T operator() (const T& x) const { 
-     T value = _prec*std::floor(x/_prec);
+     T value = _prec*floor(x/_prec);
      if (1+value == 1) value = 0;
      return value;
   }
@@ -50,7 +50,7 @@ template <class T>
 struct ceiler_type : std::unary_function<T,T> {
   ceiler_type (const T& prec) : _prec(prec) {}
   T operator() (const T& x) const { 
-     T value = _prec*std::ceil(x/_prec);
+     T value = _prec*ceil(x/_prec);
      if (1+value == 1) value = 0;
      return value;
   }
