@@ -187,7 +187,8 @@ test_rep<T,M>::_basis_evaluate_init (const reference_element& hat_K, size_type q
     }
     check_macro (loc_ndof == get_vf_space().get_constitution().loc_ndof (ref_K), 
 	"invalid value local ndof="<<loc_ndof<<": expect loc_ndof="
-  	<<get_vf_space().get_constitution().loc_ndof (ref_K));
+  	<<get_vf_space().get_constitution().loc_ndof (ref_K)
+        <<" on element variant=" << ref_K.name());
     _basis_on_quad.evaluate (ref_K, q);
   } else {
     check_macro (loc_ndof == get_vf_space().get_constitution().loc_ndof (_tilde_L), 
@@ -440,12 +441,16 @@ test_rep<T,M>::_grad_basis_evaluate (const reference_element& hat_K_in, size_typ
   }
   if (opt.surfacic) {
     // surfacic grad, where P=I-nxn is the map projector
-    // grad_s(u) = P*grad(u)*P
-    // or Ds(u)  = P*D(u)*P
+    //    grad_s(u) =   grad(u)*P
+    // or    Ds(u)  = P*D(u)*P
     for (size_type loc_idof = 0, loc_ndof = value.size(); loc_idof < loc_ndof; ++loc_idof) {
       // TODO: reduce product to d <= 3. faster when d=2
       // TODO: pre-multiplies invDF by P, then right mult by P and last symmetrize: faster ?
-      value[loc_idof] = P*value[loc_idof]*P;
+      if (! opt.symmetrized) {
+        value[loc_idof] =   value[loc_idof]*P;
+      } else {
+        value[loc_idof] = P*value[loc_idof]*P;
+      }
     }
   }
   // set the specific value(theta,theta) = vr/r term in the axi case
