@@ -1,10 +1,10 @@
 #ifndef _RHEOLEF_DISTO_H
 #define _RHEOLEF_DISTO_H
-#include "rheolef/point.h"
+#include "rheolef/field_functor.h"
 namespace rheolef {
 // P_order piola-triangle transformation here
 template <class T>
-struct disto_t : std::unary_function<point_basic<T>,point_basic<T> > {
+struct disto_t : field_functor<disto_t<T>,point_basic<T> > {
   point_basic<T> operator() (const point_basic<T>& x) const {
     point_basic<T> Fx;
     Fx[0] = x[0] + a*pow(x[0]+  x[1], order);
