@@ -24,6 +24,11 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"2"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
+if test x"${QD_EXT}" != x""; then
+  echo "      not yet (skiped when QD lib is active)"
+  exit 0
+fi
+
 status=0
 
 L="
@@ -40,7 +45,7 @@ my_cube_TPH-5-v2
 "
 for geo in $L; do
  for Pk in P1 P2 P3; do
-  loop_mpirun "./field_expr2_tst ${SRCDIR}/$geo $Pk right 2>/dev/null | diff interpolate_dom-$geo-right-$Pk.field - >/dev/null"
+  loop_mpirun "./field_expr2_tst ${SRCDIR}/$geo $Pk right 2>/dev/null | diff $SRCDIR/interpolate_dom-$geo-right-$Pk.field - >/dev/null"
   if test $? -ne 0; then status=1; fi
  done
 done
