@@ -24,6 +24,11 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"2"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
+if test x"${QD_EXT}" != x""; then
+  echo "      not yet (skiped when QD lib is active)"
+  exit 0
+fi
+
 status=0
 
 L="
