@@ -24,6 +24,10 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"7"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
+if test x"${QD_EXT}" != x""; then
+  echo "      not yet (skiped when QD lib is active)"
+  exit 0
+fi
 
 status=0
 
@@ -41,7 +45,7 @@ my_cube_TPH-5-v2
 "
 for geo in $L; do
  for Pk in P1 P2 P3; do
-  loop_mpirun "./field_indirect2_tst ${SRCDIR}/$geo.geo $Pk left 1e-8 2>/dev/null | diff field_indirect2-$geo-$Pk.field - >/dev/null"
+  loop_mpirun "./field_indirect2_tst ${SRCDIR}/$geo.geo $Pk left 1e-8 2>/dev/null | diff $SRCDIR/field_indirect2-$geo-$Pk.field - >/dev/null"
   if test $? -ne 0; then status=1; fi
  done
 done
