@@ -29,9 +29,9 @@ Float u(const point& x) { return x[0]+x[1]+x[2]; }
 
 int main(int argc, char**argv) {
   environment rheolef (argc,argv);
-  Float tol = 1e-7;
   geo omega1 (argv[1]);
   geo omega2 (argv[2]);
+  Float tol = (argc > 3) ? atof(argv[3]) : 1e-7;
   space V1h (omega1, "P1");
   space V2h (omega2, "P1");
   field u1h     = interpolate (V1h, u);
