@@ -197,7 +197,7 @@ int main(int argc, char**argv)
 #endif // TO_CLEAN
     Float mes_omega = a(w1h, w2h);
     dout << setprecision(numeric_limits<Float>::digits10)
-         << "mes(omega," << approx1 << "," << approx2 << ") = " << double(mes_omega) << endl;
+         << "mes(omega," << approx1 << "," << approx2 << ") = " << mes_omega << endl;
     int status = 0;
     // TODO: ny+my
     bool is_boundary = (omega.dimension() > omega.map_dimension());
