@@ -82,7 +82,7 @@ compute_error_band (
     c[i] = phi_h_cci.u();
   }
   csr<Float> M = { { m.uu(), trans(c)},
-                   { c,         0    } };
+                   { c,      Float(0)} };
   M.set_symmetry(true);
   // M.set_pattern_dimension(m.uu().pattern_dimension());
   solver sm = ldlt (M);
