@@ -20,7 +20,7 @@
 #
 # -------------------------------------------------------------------------
 SRCDIR=${SRCDIR-"."}
-TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+TOP_SRCDIR=${TOP_SRCDIR-"../../../rheolef"}
 DATADIR=${SRCDIR}
 BINDIR="../pbin"
 SBINDIR="../sbin"
@@ -85,18 +85,29 @@ run "$SBINDIR/mkgeo_grid_3d -v4 -T 21 -a -2 -b 2 -c -2 -d 2 -f -2 -g 2  2>/dev/n
 # -------------------------------------------------------
 # geo			err_P1 
 L="
-mesh-2d-20		0.1
 mesh-2d-41		0.08
 mesh-3d-20		0.53  
 mesh-3d-21		0.6   
 "
+if test x"${QD_EXT}" == x""; then
+  # add 20x20 mesh that need some additional constraints with Lagrange multiplier:
+  # the linear system is symmetric but undefinite
+  # and, when using Float=float128, the direct solver ldlt is unable to solve it
+L="
+mesh-2d-20		0.1
+$L
+"
+else
+  echo "      mesh-2d-20: not yet (skiped when QD lib is active, since undefinite matrix solver is missing)"
+fi
+
 while test "$L" != ""; do
   geo=`echo $L | awk '{print $1}'`
   L=`echo $L | awk '{for (i=2; i <= NF; i++) print $i}'`
   for Pk in P1; do
     err=`echo $L | awk '{print $1}'`
     L=`echo $L | awk '{for (i=2; i <= NF; i++) print $i}'`
-    loop_mpirun "./form_div_s_tst $DATADIR/${geo} -normal $Pk $qorder $err >/dev/null 2>/dev/null"
+    loop_mpirun "./form_div_s_tst ${geo} -normal $Pk $qorder $err >/dev/null 2>/dev/null"
     if test $? -ne 0; then status=1; fi
   done
 done
