@@ -79,7 +79,7 @@ check_curved (const geo& omega, const Float& tol)
       Float expect = curved_Pk [order][nx+mx][ny+my];
       string msg = "ok";
       if (fabs(mes_omega - expect) > tol) { msg = "NO"; }
-      dout << msg << ": meas("<<omega.name()<<",x^" << nx << "y^" << ny << ",x^"<<mx<<"y^"<<my<<") = " << double(mes_omega) << endl;
+      dout << msg << ": meas("<<omega.name()<<",x^" << nx << "y^" << ny << ",x^"<<mx<<"y^"<<my<<") = " << mes_omega << endl;
       if (fabs(mes_omega - expect) > tol) {
         status = 1;
         dout << "NO: expected  = " << expect << endl
