@@ -80,7 +80,7 @@ check_one (const geo& omega, const Float& tol, size_t nx, size_t ny, size_t mx,
   dout << std::setprecision(std::numeric_limits<Float>::digits10);
   std::string msg = "ok";
   if (fabs(mes_omega - expect) > tol) { msg = "NO"; }
-  dout << msg << ": meas("<<omega.name()<<",x^" << nx << "y^" << ny << ",x^"<<mx<<"y^"<<my<<") = " << double(mes_omega) << std::endl;
+  dout << msg << ": meas("<<omega.name()<<",x^" << nx << "y^" << ny << ",x^"<<mx<<"y^"<<my<<") = " << mes_omega << std::endl;
   int status = 0;
   if (fabs(mes_omega - expect) > tol) {
     status = 1;
@@ -111,7 +111,7 @@ check_all (const geo& omega, const Float& tol)
       Float expect = disto_Pk [order][nx+mx][ny+my];
       std::string msg = "ok";
       if (fabs(mes_omega - expect) > tol) { msg = "NO"; }
-      derr << msg << ": meas("<<omega.name()<<",x^" << nx << "y^" << ny << ",x^"<<mx<<"y^"<<my<<") = " << double(mes_omega) << std::endl;
+      derr << msg << ": meas("<<omega.name()<<",x^" << nx << "y^" << ny << ",x^"<<mx<<"y^"<<my<<") = " << mes_omega << std::endl;
       if (fabs(mes_omega - expect) > tol) {
         status = 1;
         derr << "NO: expected  = " << expect << std::endl
