@@ -27,7 +27,7 @@ NPROC_MAX=${NPROC_MAX-"2"}
 
 status=0
 
-run "operators_hint_tst 2>/dev/null | diff -B -w operators_hint_tst.valid - >/dev/null"
+run "operators_hint_tst 2>/dev/null | diff -B -w $SRCDIR/operators_hint_tst.valid - >/dev/null"
 if test $? -ne 0; then status=1; fi
 
 exit $status
