@@ -39,14 +39,14 @@ Float f_y2 (const point& x) { return sqr(x[1]); }
 Float f_xy (const point& x) { return x[0]*x[1]; }
 
 template <class Function>
-int check (const space& Vh, Function f, string name, Float expected_value) {
+int check (const space& Vh, Function f, string name, Float expected_value, Float tol) {
   field mfh = riesz (Vh, f);
   field one (Vh, 1.0);
   Float int_f_dx = dual(mfh,one);
-  Float tol = 100*numeric_limits<Float>::epsilon();
   if (fabs(int_f_dx - expected_value) > tol) {
-    cout << "int " << name << " dx = " << int_f_dx << " but " << expected_value
-	<< " was expected" << endl;
+    cout << setprecision(numeric_limits<Float>::digits10)
+         << "int " << name << " dx = " << int_f_dx << " but " << expected_value
+         << " was expected (error = " << fabs(int_f_dx - expected_value) << ")" << endl;
     return 1;
   } else {
     cout << "int " << name << " dx = " << int_f_dx << " : ok" << endl;
@@ -56,20 +56,21 @@ int check (const space& Vh, Function f, string name, Float expected_value) {
 int main(int argc, char**argv) {
   environment rheolef(argc, argv);
   geo omega_h (argv[1]);
+  Float  tol    = (argc > 3) ? atof(argv[3]) : 100*numeric_limits<Float>::epsilon();
   check_macro (omega_h.coordinate_system() == space_constant::axisymmetric_rz,
 	"expect `rz' coordinate system");
   string approx = (argc > 2) ? argv[2] : "P1";
   space Vh (omega_h, approx);
   int status = 0;
-  status |= check (Vh, f_1, "1", 0.5);
+  status |= check (Vh, f_1, "1", 0.5, tol);
   if (approx == "P2") {
 #ifdef _RHEOLEF_HAVE_STD_INITIALIZER_LIST
-     status |= check (Vh, f_x(), "x", 1./3);
+     status |= check (Vh, f_x(), "x", Float(1)/3, tol);
 #endif // _RHEOLEF_HAVE_STD_INITIALIZER_LIST
-     status |= check (Vh, f_y,   "y", 1./4);
-     status |= check (Vh, f_x2, "x2", 1./4);
-     status |= check (Vh, f_y2, "y2", 1./6);
-     status |= check (Vh, f_xy, "xy", 1./6);
+     status |= check (Vh, f_y,   "y", Float(1)/4, tol);
+     status |= check (Vh, f_x2, "x2", Float(1)/4, tol);
+     status |= check (Vh, f_y2, "y2", Float(1)/6, tol);
+     status |= check (Vh, f_xy, "xy", Float(1)/6, tol);
   }
   return status;
 }
