@@ -22,7 +22,7 @@
 using namespace rheolef;
 using namespace std;
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   geo omega (argv[1]);
   space Vh (omega, argv[2]);
   return 0;
