@@ -19,18 +19,15 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # -------------------------------------------------------------------------
+TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+. "${TOP_SRCDIR}/config/loop_mpirun.sh"
+
 status=0
-
 for q in gauss gauss_radau gauss_lobatto; do
-    echo "    ./${q}_legendre_tst 1000"
-    ./${q}_legendre_tst 1000 >/dev/null 2>/dev/null
-    status=$?
-    if test $? -ne 0; then echo "      *NO*"; status=1; fi
+    run "./${q}_legendre_tst 1000 >/dev/null 2>/dev/null"
+    if test $? -ne 0; then status=1; fi
 
-    echo "    ./${q}_jacobi_tst 1000 3 1"
-    ./${q}_jacobi_tst 200 30 1 >/dev/null 2>/dev/null
-    status=$?
-    if test $? -ne 0; then echo "      *NO*"; status=1; fi
+    run "./${q}_jacobi_tst 200 30 1 1e-6 >/dev/null 2>/dev/null"
+    if test $? -ne 0; then status=1; fi
 done
-
 exit $status
