@@ -18,6 +18,7 @@
 /// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ///
 /// =========================================================================
+#include "rheolef/compiler.h"
 #include "rheolef/gauss_jacobi.h"
 #include "rheolef/gauss_chebyschev.h"
 #include "rheolef/gauss_radau_jacobi.h"
@@ -28,10 +29,14 @@
 using namespace rheolef;
 using namespace std;
 int main (int argc, char **argv) {
+#ifdef _RHEOLEF_HAVE_QD
+        unsigned int oldcw;
+        fpu_fix_start(&oldcw);
+#endif // _RHEOLEF_HAVE_QD
 	size_t R = argc > 1 ? atoi(argv[1]) : 5;
 	Float alpha = argc > 2 ? atof(argv[2]) : 0;
 	Float beta  = argc > 3 ? Float(atof(argv[3])) : alpha;
-	Float tol   = argc > 4 ? Float(atof(argv[4])) : 1e-7;
+        Float tol   = argc > 4 ? Float(atof(argv[4])) : sqrt(numeric_limits<Float>::epsilon());
 	if (R == 0) return 0;
 	vector<Float> zeta(R), omega(R);
 	gauss_radau_jacobi (R, alpha, beta, zeta.begin(), omega.begin());
@@ -44,17 +49,17 @@ int main (int argc, char **argv) {
 	for (size_t r = 0; r < R; r++)
 	    cout << r << "\t" << zeta[r] << "\t" << omega[r] << endl;
 	Float err = 0;
-	if (Float(int(alpha)) == alpha && Float(int(beta)) == beta) {
-	    size_t p = int(alpha);
-	    size_t q = int(beta);
+	if (floor(alpha) == alpha && floor(beta) == beta) {
+	    size_t p = details::convert_to_int(alpha);
+	    size_t q = details::convert_to_int(beta);
             size_t Rtilde = R + int((p+q+1)/2);
 	    vector<Float> x(Rtilde), w(Rtilde);
 	    gauss_jacobi (Rtilde, 0, 0, x.begin(), w.begin());
 	    err = gauss_jacobi_check (R, p, q, zeta.begin(), omega.begin(),
 			Rtilde, x.begin(), w.begin(), 2*R-2);
-	} else if (fabs(alpha-int(alpha)) == Float(0.5) && fabs(beta-int(beta)) == Float(0.5)) {
-	    size_t p = int(alpha+0.5);
-	    size_t q = int(beta+0.5);
+	} else if (fabs(alpha-floor(alpha)) == Float(0.5) && fabs(beta-floor(beta)) == Float(0.5)) {
+	    size_t p = details::convert_to_int(alpha+0.5);
+	    size_t q = details::convert_to_int(beta+0.5);
             size_t Rtilde = R + int((p+q+1)/2);
             vector<Float> x(Rtilde), w(Rtilde);
 	    gauss_chebyschev (Rtilde, x.begin(), w.begin());
@@ -65,5 +70,8 @@ int main (int argc, char **argv) {
 	    exit (1);
 	}
 	if (err >= tol) cerr << "err = " << err << endl;
+#ifdef _RHEOLEF_HAVE_QD
+        fpu_fix_end(&oldcw);
+#endif // _RHEOLEF_HAVE_QD
         return err < tol ? 0 : 1;
 }
