@@ -68,6 +68,10 @@ int main (int argc, char**argv)
     // machine-dependent precision
     int digits10 = numeric_limits<Float>::digits10;
     cout << setprecision(digits10);
+#ifdef _RHEOLEF_HAVE_QD
+    unsigned int oldcw;
+    fpu_fix_start(&oldcw);
+#endif // _RHEOLEF_HAVE_QD
 
     // default nb edge on x axis
 
@@ -166,7 +170,7 @@ int main (int argc, char**argv)
 		cout << "e " << no(i) << " " << no(i+1) << endl;
 	    cout << endl;
     }
-    return 0;
+#ifdef _RHEOLEF_HAVE_QD
+  fpu_fix_end(&oldcw);
+#endif // _RHEOLEF_HAVE_QD
 }
-
-
