@@ -30,25 +30,25 @@ To: Pierre Saramito <Pierre.Saramito@imag.fr>
 
 Salut Pierre,
 
-J'ai test le terme de bord sur un cas de transport simple (partial 
+J'ai teste' le terme de bord sur un cas de transport simple (partial 
 u/partial t + (v.grad)u = 0) : sur un canal  [0,10]x[0,1] avec v = 
 (1,0), je mets en terme de  bord  sin(t)*exp(-10*(y-0.5)*(y-0.5)), 
-c'est- dire un train de sinus moduls par une gaussienne, qui doit 
-atteindre l'extrmit du canal  t = 10. Comme on le craignait, pour 
-crer le terme de bord en second membre, l'interpolation d'une fonction 
-de bord en P1d ne marche pas, mais j'ai contourn le problme en 
-interpolant d'abord en P1 puis en projetant en P1d. Et a marche aprs 
-une correction mineure dans form.cc. Les modifs sont remontes sous cvs.
+c'est-a dire un train de sinus modules par une gaussienne, qui doit 
+atteindre l'extremite du canal a t = 10. Comme on le craignait, pour 
+creer le terme de bord en second membre, l'interpolation d'une fonction 
+de bord en P1d ne marche pas, mais j'ai contourne le probleme en 
+interpolant d'abord en P1 puis en projetant en P1d. Et ca marche apres 
+une correction mineure dans form.cc. Les modifs sont remontees sous cvs.
 Pour faire fonctionner le fichier d'exemple :
 maillage : mkgeo_grid -t 100 -t 10 -a 0 -b 10 -c 0 -d 1 > omega-10.geo
 puis : ./transport_pur omega-10.geo [N] > tp-10.branch, qui fait le 
-calcul jusqu' t=10, en N pas de temps
+calcul jusqu'a t=10, en N pas de temps
 
-a affiche l'erreur par rapport  la solution exacte tous les 10 pas de 
+ca affiche l'erreur par rapport a la solution exacte tous les 10 pas de 
 temps et le .branch contient 10 solutions en plus de la condition 
-initiale (prise  zro). Ca converge bien en Dx et en Dt.
+initiale (prise a zero). Ca converge bien en Dx et en Dt.
 
-Par ailleurs, pour le bug des produits de formes : voil un exemple
+Par ailleurs, pour le bug des produits de formes : voila un exemple
 
 space Vh(omega,"P2",vector);
 space Qh(omega,"P1");
@@ -58,7 +58,7 @@ form A = trans(b)*mq*b;
 field uh(Vh,0);
 cout << A*uh;
 
-a donne une seg fault alors que a devrait pas ! Et si on affiche le nb 
+ca donne une seg fault alors que ca devrait pas ! Et si on affiche le nb 
 de lignes et de colonnes des formes, on voit que ce n'est pas bon.
 
 Bon we,
