@@ -452,7 +452,7 @@ def build_isovalue_table (fmin, fmax, option):
 
 def mayavi2_field_scalar (view, basename, option):
 
-    print "ici scalaire..."
+    #print "ici scalaire..."
     scene = start_new_scene (view, option)
 
     data = VTKFileReader()
@@ -491,7 +491,7 @@ def mayavi2_field_scalar (view, basename, option):
 
     if (z_max - z_min == 0 and y_max - y_min == 0):
 	# 1D data:
-        print "1D data..."
+        #print "1D data..."
 
         elevation = WarpScalar()
         view.add_module(elevation)
@@ -504,7 +504,7 @@ def mayavi2_field_scalar (view, basename, option):
 
 	mm = surface.module_manager
 	ms = mm.scalar_lut_manager
-        print "ms=",ms
+        #print "ms=",ms
         ms.show_legend = False
         ms.render()
 	range = tuple(ms.data_range)
@@ -556,9 +556,9 @@ def mayavi2_field_scalar (view, basename, option):
         if option['approx'] != "P0" and not option['view_map']:
             mm.scalar_lut_manager.number_of_colors = option['n_isovalue']
             mm.scalar_lut_manager.number_of_labels = option['n_isovalue']+1
-        print "ici 2D..."
+        #print "ici 2D..."
         if option['elevation']:
-            print "ici 2D elevation..."
+            #print "ici 2D elevation..."
 	    if (f_max-f_min > 0):
                 scale = 0.5*numpy.sqrt((x_max-x_min)**2+(y_max-y_min)**2)/(f_max - f_min)
 	    else:
