@@ -51,7 +51,7 @@ DESCRIPTION:
  
  @noindent
  Input and output, as usual 
- (@pxref{iorheo class}):
+ (@pxref{iorheo ialgorithm}):
  @example
      	cin  >> a;
      	cout << a;
