@@ -26,8 +26,13 @@
 #include "rheolef/diststream.h"
 #include "rheolef/heap_allocator.h"
 #include "rheolef/msg_util.h"
+#include "rheolef/container_traits.h"
+#ifdef _RHEOLEF_HAVE_MPI
+#include "rheolef/mpi_pair_datatype.h"
+#endif // _RHEOLEF_HAVE_MPI
 
 #include <boost/mpl/bool.hpp>
+#include <boost/type_traits/remove_const.hpp>
 
 namespace rheolef {
 /// @brief array element output helper
@@ -35,68 +40,25 @@ template <class T>
 struct _array_put_element_type {
   std::ostream& operator() (std::ostream& os, const T& x) { return os << x; }
 };
+template <class T>
+struct _array_put_matlab_type {
+  std::ostream& operator() (std::ostream& os, const T& x) { return os << x << ";"; }
+};
 /// @brief array element input helper
 template <class T>
 struct _array_get_element_type {
   std::istream& operator() (std::istream& is, T& x) { return is >> x; }
 };
 } // namespace rheolef
-
-// -------------------------------------------------------------
-// send a pair<size_t,T> via mpi & serialization
-// -------------------------------------------------------------
-// non-intrusive version of serialization for pair<size_t,T> 
-#ifdef _RHEOLEF_HAVE_MPI
-namespace boost {
- namespace serialization {
-  template<class T, class Archive>
-  void serialize (Archive & ar, std::pair<size_t, T>& g, const unsigned int version) {
-    ar & g.first;
-    ar & g.second;
-  }
- } // namespace serialization
-} // namespace boost
-
-// Some serializable types, like pair<size_t,double>, have a fixed amount of data stored at fixed field positions.
-// When this is the case, boost::mpi can optimize their serialization and transmission to avoid extraneous copy operations.
-// To enable this optimization, we specialize the type trait is_mpi_datatype, e.g.:
-namespace boost {
- namespace mpi {
-  template <>
-  struct is_mpi_datatype<std::pair<size_t,double> > : boost::mpl::true_ { };
- } // namespace mpi
-} // namespace boost
-#endif // _RHEOLEF_HAVE_MPI
-
-#ifndef TO_CLEAN
-namespace rheolef {
-// for debug
-template<class T>
-inline
-std::ostream&
-operator<< (std::ostream& o, const std::pair<size_t,T>& x) {
-  return o << "pair{" << x.first << ", "<< x.second << "}";
-}
-} // namespace rheolef
-#endif // TO_CLEAN
-// -------------------------------------------------------------
-// support both array<T> with T=size_t and T=set<size_t>
-// but we declare whether
-// T is a simple mpi_datatype or a container of mpi_datatype
-// -------------------------------------------------------------
-namespace rheolef {
-  template <class T>
-  struct is_container_of_mpi_datatype : boost::mpl::false_ {
-    typedef boost::mpl::false_ type;
-  };
-} // namespace rheolef
 // -------------------------------------------------------------
 // the sequential representation
 // -------------------------------------------------------------
 namespace rheolef {
 
+template <class T, class M, class A> class array_rep {};
+
 template <class T, class A>
-class array_seq_rep : public std::vector<T,A> {
+class array_rep<T,sequential,A> : public std::vector<T,A> {
 public:
     typedef T                                         value_type;
     typedef A                                         allocator_type;
@@ -111,12 +73,12 @@ public:
     typedef distributor::communicator_type            communicator_type;
     typedef sequential                                memory_type;
 
-    explicit array_seq_rep (const A& alloc = A());
-    array_seq_rep (const distributor& ownership, const T& init_val = T(), const A& alloc = A());
+    explicit array_rep (const A& alloc = A());
+    array_rep (const distributor& ownership, const T& init_val = T(), const A& alloc = A());
     void resize   (const distributor& ownership, const T& init_val = T());
-    array_seq_rep (size_type loc_size = 0,       const T& init_val = T(), const A& alloc = A());
+    array_rep (size_type loc_size = 0,       const T& init_val = T(), const A& alloc = A());
     void resize   (size_type loc_size = 0,       const T& init_val = T());
-    array_seq_rep (const array_seq_rep<T,A>& x);
+    array_rep (const array_rep<T,sequential,A>& x);
 
     A get_allocator() const { return base::get_allocator(); }
     size_type size() const { return base::size(); }
@@ -133,26 +95,27 @@ public:
     size_type dis_size () const { return base::size(); }
     size_type first_index () const { return 0; }
     size_type last_index () const { return base::size(); }
-    reference dis_entry (size_type dis_i) { return operator[](dis_i); }
+    reference dis_entry     (size_type dis_i) { return operator[](dis_i); }
     void reset_dis_indexes() const {}
-    template<class SetOp> void dis_entry_assembly_begin (SetOp) {}
-    template<class SetOp> void dis_entry_assembly_end (SetOp) {}
-    void dis_entry_assembly_begin () {}
-    void dis_entry_assembly_end () {}
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_begin (SetOp = SetOp()) {}
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_end (SetOp = SetOp()) {}
     void repartition (					      // old_numbering for *this
-        const array_seq_rep<size_type,A>& partition,	      // old_ownership
-        array_seq_rep<T,A>&               new_array,	      // new_ownership (created)
-        array_seq_rep<size_type,A>&       old_numbering,      // new_ownership
-        array_seq_rep<size_type,A>&       new_numbering) const // old_ownership
+        const array_rep<size_type,sequential,A>& partition,	      // old_ownership
+        array_rep<T,sequential,A>&               new_array,	      // new_ownership (created)
+        array_rep<size_type,sequential,A>&       old_numbering,      // new_ownership
+        array_rep<size_type,sequential,A>&       new_numbering) const // old_ownership
     {
 	error_macro ("not yet");
     }
     template<class A2>
     void reverse_permutation (                                // old_ownership for *this=iold2dis_inew
-        array_seq_rep<size_type,A2>& inew2dis_iold) const;    // new_ownership
+        array_rep<size_type,sequential,A2>& inew2dis_iold) const;    // new_ownership
 
     idiststream& get_values (idiststream& s);
     odiststream& put_values (odiststream& s) const;
+    odiststream& put_matlab (odiststream& s) const;
     template <class GetFunction> idiststream& get_values (idiststream& ips, GetFunction get_element);
     template <class PutFunction> odiststream& put_values (odiststream& ops, PutFunction put_element) const;
     void dump (std::string name) const;
@@ -165,12 +128,12 @@ protected:
 // -------------------------------------------------------------
 #ifdef _RHEOLEF_HAVE_MPI
 template <class T, class A>
-class array_mpi_rep : public array_seq_rep<T,A> {
+class array_rep<T,distributed,A> : public array_rep<T,sequential,A> {
 public:
 
 // typedefs:
 
-    typedef array_seq_rep<T,A>         		base;
+    typedef array_rep<T,sequential,A>     	base;
     typedef typename base::value_type     	value_type;
     typedef typename base::size_type      	size_type;
     typedef typename base::difference_type      difference_type;
@@ -184,28 +147,29 @@ public:
 						scatter_map_type;
 
     struct dis_reference {
-      dis_reference (array_mpi_rep<T,A>& x, size_type dis_i)
+      dis_reference (array_rep<T,distributed,A>& x, size_type dis_i)
        : _x(x), _dis_i(dis_i) {}
 
       dis_reference& operator= (const T& value) {
         _x.set_dis_entry (_dis_i, value);
         return *this;
       }
-      dis_reference& operator+= (const T& value) {
+      template<class U>
+      dis_reference& operator+= (const U& value) {
         _x.set_add_dis_entry (_dis_i, value);
         return *this;
       }
     // data:
     protected:
-      array_mpi_rep<T,A>& _x;
+      array_rep<T,distributed,A>& _x;
       size_type           _dis_i;
     };
 
 // allocators:
 
-    array_mpi_rep (const distributor& ownership, const T&  init_val = T(), const A& alloc = A());
+    array_rep (const distributor& ownership, const T&  init_val = T(), const A& alloc = A());
     void resize   (const distributor& ownership, const T&  init_val = T());
-    array_mpi_rep (const array_mpi_rep<T,A>& x);
+    array_rep (const array_rep<T,distributed,A>& x);
 
     A get_allocator() const        { return base::get_allocator(); }
     size_type size() const         { return base::size(); }
@@ -222,15 +186,14 @@ public:
 
     dis_reference dis_entry (size_type dis_i) { return dis_reference (*this, dis_i); }
 
-    template<class SetOp> void dis_entry_assembly_begin (SetOp my_set_op);
-    template<class SetOp> void dis_entry_assembly_end   (SetOp my_set_op);
-    template<class SetOp> void dis_entry_assembly       (SetOp my_set_op)
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_begin (SetOp my_set_op = SetOp());
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_end   (SetOp my_set_op = SetOp());
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly       (SetOp my_set_op = SetOp())
    		{ dis_entry_assembly_begin (my_set_op); dis_entry_assembly_end (my_set_op); }
 
-    void dis_entry_assembly_begin () { dis_entry_assembly_begin (set_op<T,T>()); }
-    void dis_entry_assembly_end ()   { dis_entry_assembly_end   (set_op<T,T>()); }
-    void dis_entry_assembly ()       { dis_entry_assembly_begin (); dis_entry_assembly_end (); }
-
     template<class Set, class Map>
     void append_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const;
 
@@ -254,34 +217,45 @@ public:
 
     template<class A2>
     void repartition (					      // old_numbering for *this
-        const array_mpi_rep<size_type,A2>& partition,	      // old_ownership
-        array_mpi_rep<T,A>&                new_array,	      // new_ownership (created)
-        array_mpi_rep<size_type,A2>&       old_numbering,     // new_ownership
-        array_mpi_rep<size_type,A2>&       new_numbering) const; // old_ownership
+        const array_rep<size_type,distributed,A2>& partition,	      // old_ownership
+        array_rep<T,distributed,A>&                new_array,	      // new_ownership (created)
+        array_rep<size_type,distributed,A2>&       old_numbering,     // new_ownership
+        array_rep<size_type,distributed,A2>&       new_numbering) const; // old_ownership
 
     template<class A2>
     void permutation_apply (		 		       // old_numbering for *this
-        const array_mpi_rep<size_type,A2>& new_numbering,      // old_ownership
-        array_mpi_rep<T,A>&                new_array) const;   // new_ownership (already allocated)
+        const array_rep<size_type,distributed,A2>& new_numbering,      // old_ownership
+        array_rep<T,distributed,A>&                new_array) const;   // new_ownership (already allocated)
 
     template<class A2>
     void reverse_permutation (                                // old_ownership for *this=iold2dis_inew
-        array_mpi_rep<size_type,A2>& inew2dis_iold) const;    // new_ownership
+        array_rep<size_type,distributed,A2>& inew2dis_iold) const;    // new_ownership
 
     idiststream& get_values (idiststream& s);
     odiststream& put_values (odiststream& s) const;
+    odiststream& put_matlab (odiststream& s) const;
     template <class GetFunction> idiststream& get_values (idiststream& ips, GetFunction get_element);
     template <class PutFunction> odiststream& put_values (odiststream& ops, PutFunction put_element) const;
-    template <class PutFunction, class A2> odiststream& permuted_put_values (odiststream& ops, const array_mpi_rep<size_type,A2>& perm,
+    template <class PutFunction, class A2> odiststream& permuted_put_values (odiststream& ops, const array_rep<size_type,distributed,A2>& perm,
 		PutFunction put_element) const;
     void dump (std::string name) const;
 protected:
     void set_dis_entry (size_type dis_i, const T& val);
-    void set_add_dis_entry (size_type dis_i, const T& val);
+    template<class U>
+    void set_add_dis_entry (size_type dis_i, const U& val);
 // typedefs:
     /** 1) stash: store data before assembly() communications:
       *   select multimap<U> when T=set<U> and map<T> otherwise
       */
+    template<class Pair>
+    struct remove_const_in_pair {
+        typedef Pair type;
+    };
+    template<class T1, class T2>
+    struct remove_const_in_pair<std::pair<T1,T2> > {
+        typedef std::pair<typename boost::remove_const<T1>::type,
+                          typename boost::remove_const<T2>::type> type;
+    };
     template<class U, class IsContainer> struct stash_traits {};
     template<class U>
     struct stash_traits<U,boost::mpl::false_> {
@@ -290,7 +264,7 @@ protected:
     };
     template<class U>
     struct stash_traits<U,boost::mpl::true_> {
-        typedef typename U::value_type mapped_type;
+        typedef typename remove_const_in_pair<typename U::value_type>::type mapped_type;
         typedef std::multimap <size_type, mapped_type, std::less<size_type>, heap_allocator<std::pair<size_type,mapped_type> > >   map_type;
     };
     typedef typename is_container_of_mpi_datatype<T>::type is_container;
@@ -410,12 +384,12 @@ public:
 };
 //<verbatim:
 template <class T, class A>
-class array<T,sequential,A> : public smart_pointer<array_seq_rep<T,A> > {
+class array<T,sequential,A> : public smart_pointer<array_rep<T,sequential,A> > {
 public:
 
 // typedefs:
 
-    typedef array_seq_rep<T,A>  	  rep;
+    typedef array_rep<T,sequential,A>  	  rep;
     typedef smart_pointer<rep> 		  base;
 
     typedef sequential 			  memory_type;
@@ -458,10 +432,16 @@ public:
 // global modifiers (for compatibility with distributed interface):
 
     dis_reference dis_entry (size_type dis_i) { return base::data().dis_entry(dis_i); }
-    void dis_entry_assembly()                 {}
-    template<class SetOp> void dis_entry_assembly(SetOp my_set_op)        {}
-    template<class SetOp> void dis_entry_assembly_begin (SetOp my_set_op) {}
-    template<class SetOp> void dis_entry_assembly_end (SetOp my_set_op)   {}
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly (SetOp my_set_op = SetOp()) {}
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_begin (SetOp my_set_op = SetOp()) {}
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_end (SetOp my_set_op = SetOp()) {}
+
+    void dis_entry_assembly_begin() {}
+    void dis_entry_assembly_end()   {}
+    void dis_entry_assembly()       {}
 
     void reset_dis_indexes() const {}
     template<class Set> void set_dis_indexes    (const Set& ext_idx_set) const {}
@@ -539,12 +519,12 @@ array<T,sequential,A>::resize (
 #ifdef _RHEOLEF_HAVE_MPI
 //<verbatim:
 template <class T, class A>
-class array<T,distributed,A> : public smart_pointer<array_mpi_rep<T,A> > {
+class array<T,distributed,A> : public smart_pointer<array_rep<T,distributed,A> > {
 public:
 
 // typedefs:
 
-    typedef array_mpi_rep<T,A>  	  rep;
+    typedef array_rep<T,distributed,A>    rep;
     typedef smart_pointer<rep> 		  base;
 
     typedef distributed 		  memory_type;
@@ -603,16 +583,18 @@ public:
 
 // global modifiers (for compatibility with distributed interface):
 
-    dis_reference dis_entry (size_type dis_i)       { return base::data().dis_entry(dis_i); }
+    dis_reference dis_entry (size_type dis_i) { return base::data().dis_entry(dis_i); }
 
-    void dis_entry_assembly()                       { return base::data().dis_entry_assembly(); }
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_begin (SetOp my_set_op = SetOp()) { base::data().dis_entry_assembly_begin (my_set_op); }
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly_end   (SetOp my_set_op = SetOp()) { base::data().dis_entry_assembly_end   (my_set_op); }
+    template<class SetOp = typename default_set_op<T>::type>
+    void dis_entry_assembly       (SetOp my_set_op = SetOp()) { base::data().dis_entry_assembly       (my_set_op); }
 
-    template<class SetOp>
-    void dis_entry_assembly       (SetOp my_set_op) { return base::data().dis_entry_assembly       (my_set_op); }
-    template<class SetOp>
-    void dis_entry_assembly_begin (SetOp my_set_op) { return base::data().dis_entry_assembly_begin (my_set_op); }
-    template<class SetOp>
-    void dis_entry_assembly_end   (SetOp my_set_op) { return base::data().dis_entry_assembly_end   (my_set_op); }
+    void dis_entry_assembly_begin() { base::data().template dis_entry_assembly_begin<typename default_set_op<T>::type>(); }
+    void dis_entry_assembly_end()   { base::data().template dis_entry_assembly_end<typename default_set_op<T>::type>(); }
+    void dis_entry_assembly()       { dis_entry_assembly_begin(); dis_entry_assembly_end(); }
 
 // apply a partition:
  
