@@ -25,7 +25,7 @@ using namespace std;
 // instanciation in library
 // ----------------------------------------------------------------------------
 namespace rheolef {
-template class array_seq_rep<Float,std::allocator<Float> >;
-template class array_seq_rep<int,std::allocator<int> >;
-template class array_seq_rep<size_t,std::allocator<size_t> >;
+template class array_rep<Float,sequential,std::allocator<Float> >;
+template class array_rep<int,  sequential,std::allocator<int> >;
+template class array_rep<size_t,sequential,std::allocator<size_t> >;
 } // namespace rheolef
