@@ -74,6 +74,8 @@ public:
     bool is_symmetric() const { return _is_symmetric; }
     void set_symmetry (bool is_symm) const { _is_symmetric = is_symm; }
     void set_symmetry_by_check (const T& tol = std::numeric_limits<T>::epsilon()) const;
+    bool is_definite_positive() const { return _is_definite_positive; }
+    void set_definite_positive (bool is_defpos) const { _is_definite_positive = is_defpos; }
     size_type pattern_dimension() const { return _pattern_dimension; }
     void set_pattern_dimension(size_type dim) const { _pattern_dimension = dim; }
     size_type row_first_index () const { return 0; }
@@ -105,6 +107,7 @@ public:
     distributor          		                          _col_ownership;
     std::vector<std::pair<typename std::vector<T>::size_type,T> > _data;
     mutable bool						  _is_symmetric;
+    mutable bool						  _is_definite_positive;
     mutable size_type						  _pattern_dimension; // e.g. FEM 3d-pattern
 };
 template<class T>
@@ -114,7 +117,10 @@ csr_rep<T,sequential>::csr_rep(const asr<T,sequential,A>& a)
   : vector_of_iterator<pair_type>(a.nrow()+1),
    _row_ownership (a.row_ownership()),
    _col_ownership (a.col_ownership()),
-   _data(a.nnz())
+   _data(a.nnz()),
+   _is_symmetric(false),
+   _is_definite_positive(false),
+   _pattern_dimension(0)
 {
   build_from_asr (a);
 }
@@ -175,6 +181,8 @@ public:
     bool is_symmetric() const { return base::is_symmetric(); }
     void set_symmetry (bool is_symm) const { base::set_symmetry(is_symm); }
     void set_symmetry_by_check (const T& tol = std::numeric_limits<T>::epsilon()) const;
+    bool is_definite_positive() const { return base::is_definite_positive(); }
+    void set_definite_positive (bool is_defpos) const { base::set_definite_positive(is_defpos); }
     size_type pattern_dimension() const { return base::pattern_dimension(); }
     void set_pattern_dimension(size_type dim) const { base::set_pattern_dimension(dim); }
     size_type row_first_index () const { return row_ownership().first_index(); }
@@ -334,6 +342,8 @@ public:
     void set_symmetry (bool is_symm) const   { base::data().set_symmetry(is_symm); }
     void set_symmetry_by_check (const T& tol = std::numeric_limits<T>::epsilon()) const
                                              { base::data().set_symmetry_by_check(); }
+    bool is_definite_positive() const        { return base::data().is_definite_positive(); }
+    void set_definite_positive (bool is_defpos) const { base::data().set_definite_positive(is_defpos); }
     size_type pattern_dimension() const      { return base::data().pattern_dimension(); }
     void set_pattern_dimension(size_type dim) const { base::data().set_pattern_dimension(dim); }
     T max_abs () const                       { return base::data().max_abs(); }
@@ -471,6 +481,8 @@ public:
     void set_symmetry (bool is_symm) const   { base::data().set_symmetry(is_symm); }
     void set_symmetry_by_check (const T& tol = std::numeric_limits<T>::epsilon()) const
                                              { base::data().set_symmetry_by_check(); }
+    bool is_definite_positive() const        { return base::data().is_definite_positive(); }
+    void set_definite_positive (bool is_defpos) const { base::data().set_definite_positive(is_defpos); }
     size_type pattern_dimension() const      { return base::data().pattern_dimension(); }
     void set_pattern_dimension(size_type dim) const { base::data().set_pattern_dimension(dim); }
     T max_abs () const                       { return base::data().max_abs(); }
