@@ -59,6 +59,10 @@ struct csr_concat_value {
 // typedef:
  typedef enum { scalar, vector, vector_transpose, vector_vec, vector_vec_transpose, matrix} variant_type;
 // allocators:
+ csr_concat_value (int x)                           : s(x), v(),     vv(),     m(),  variant(scalar) {}
+#ifdef _RHEOLEF_HAVE_QD
+ csr_concat_value (double x)                        : s(x), v(),     vv(),     m(),  variant(scalar) {}
+#endif // _RHEOLEF_HAVE_QD
  csr_concat_value (const T& x)                      : s(x), v(),     vv(),     m(),  variant(scalar) {}
  csr_concat_value (const vec<T,M>& x)               : s(),  v(x),    vv(),     m(),  variant(vector) {}
  csr_concat_value (const vec_trans<T,M>& vt)        : s(),  v(vt.v), vv(),     m(),  variant(vector_transpose) {}
