@@ -49,6 +49,10 @@
 #ifdef _RHEOLEF_HAVE_MUMPS
 #include "solver_mumps.h"
 
+#ifdef DEBUG_FROGGY
+#undef _RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH // has failed on some linear systems: prefer seq-scotch when available
+#endif // DEBUG_FROGGY
+
 namespace rheolef {
 using namespace std;
 
@@ -74,10 +78,10 @@ nnz_dia_upper (const csr<T,M>& a)
   return dia_nnz;
 }
 // 2) extra-diagonal block case, for distributed csr
-template<class T>
+template<class T, class M>
 static
-typename csr<T,sequential>::size_type
-nnz_ext_upper (const csr<T,sequential>& a)
+typename csr<T,M>::size_type
+nnz_ext_upper (const csr<T,M>& a)
 {
   return 0;
 }
@@ -101,15 +105,15 @@ nnz_ext_upper (const csr<T,distributed>& a)
   return ext_nnz;
 }
 #endif // _RHEOLEF_HAVE_MPI
-template<class T>
+template<class T, class M>
 static
 void
-csr2mumps_struct (const csr<T,sequential>& a, vector<int>& row, vector<int>& col, bool use_symmetry)
+csr2mumps_struct_seq (const csr<T,M>& a, vector<int>& row, vector<int>& col, bool use_symmetry)
 {
-  typedef typename csr<T,sequential>::size_type size_type;
-  typename csr<T,sequential>::const_iterator dia_ia = a.begin();
+  typedef typename csr<T,M>::size_type size_type;
+  typename csr<T,M>::const_iterator dia_ia = a.begin();
   for (size_type i = 0, n = a.nrow(), q = 0; i < n; i++) {
-    for (typename csr<T,sequential>::const_data_iterator p = dia_ia[i]; p < dia_ia[i+1]; p++) {
+    for (typename csr<T,M>::const_data_iterator p = dia_ia[i]; p < dia_ia[i+1]; p++) {
       size_type j = (*p).first;	
       if (!use_symmetry || j >= i) {
         row[q] = i + 1;
@@ -119,15 +123,15 @@ csr2mumps_struct (const csr<T,sequential>& a, vector<int>& row, vector<int>& col
     }
   }
 }
-template<class T>
+template<class T, class M>
 static
 void
-csr2mumps_values (const csr<T,sequential>& a, vector<T>& val, bool use_symmetry)
+csr2mumps_values_seq (const csr<T,M>& a, vector<T>& val, bool use_symmetry)
 {
-  typedef typename csr<T,sequential>::size_type size_type;
-  typename csr<T,sequential>::const_iterator dia_ia = a.begin();
+  typedef typename csr<T,M>::size_type size_type;
+  typename csr<T,M>::const_iterator dia_ia = a.begin();
   for (size_type i = 0, n = a.nrow(), q = 0; i < n; i++) {
-    for (typename csr<T,sequential>::const_data_iterator p = dia_ia[i]; p < dia_ia[i+1]; p++) {
+    for (typename csr<T,M>::const_data_iterator p = dia_ia[i]; p < dia_ia[i+1]; p++) {
       size_type j = (*p).first;	
       if (!use_symmetry || j >= i) {
         val[q] = (*p).second;
@@ -140,7 +144,14 @@ csr2mumps_values (const csr<T,sequential>& a, vector<T>& val, bool use_symmetry)
 template<class T>
 static
 void
-csr2mumps_struct (const csr<T,distributed>& a, vector<int>& row, vector<int>& col, bool use_symmetry)
+csr2mumps_struct_dis (const csr<T,sequential>& a, vector<int>& row, vector<int>& col, bool use_symmetry, bool)
+{
+  // dummy case: for the compiler to be happy
+}
+template<class T>
+static
+void
+csr2mumps_struct_dis (const csr<T,distributed>& a, vector<int>& row, vector<int>& col, bool use_symmetry, bool drop_ext_nnz)
 {
   typedef typename csr<T,distributed>::size_type size_type;
   typename csr<T,distributed>::const_iterator dia_ia = a.begin();
@@ -156,6 +167,7 @@ csr2mumps_struct (const csr<T,distributed>& a, vector<int>& row, vector<int>& co
         q++;
       }
     }
+    if (drop_ext_nnz) continue;
     for (typename csr<T,distributed>::const_data_iterator p = ext_ia[i]; p < ext_ia[i+1]; p++) {
       size_type dis_i = first_i + i;
       size_type dis_j = a.jext2dis_j ((*p).first);
@@ -170,7 +182,14 @@ csr2mumps_struct (const csr<T,distributed>& a, vector<int>& row, vector<int>& co
 template<class T>
 static
 void
-csr2mumps_values (const csr<T,distributed>& a, vector<T>& val, bool use_symmetry)
+csr2mumps_values_dis (const csr<T,sequential>& a, vector<T>& val, bool use_symmetry, bool)
+{
+  // dummy case: for the compiler to be happy
+}
+template<class T>
+static
+void
+csr2mumps_values_dis (const csr<T,distributed>& a, vector<T>& val, bool use_symmetry, bool drop_ext_nnz)
 {
   typedef typename csr<T,distributed>::size_type size_type;
   size_type first_i = a.row_ownership().first_index();
@@ -185,6 +204,7 @@ csr2mumps_values (const csr<T,distributed>& a, vector<T>& val, bool use_symmetry
         q++;
       }
     }
+    if (drop_ext_nnz) continue;
     for (typename csr<T,distributed>::const_data_iterator p = ext_ia[i]; p < ext_ia[i+1]; p++) {
       size_type dis_i = first_i + i;
       size_type dis_j = a.jext2dis_j ((*p).first);
@@ -203,11 +223,20 @@ template<class T, class M>
 solver_mumps_rep<T,M>::solver_mumps_rep (const csr<T,M>& a, const solver_option_type& opt)
  : solver_abstract_rep<T,M>(opt),
    _has_mumps_instance(false),
+   _drop_ext_nnz(false),
    _mumps_par(),
    _row(),
    _col(),
-   _a00(0)
+   _a00(0),
+   _det()
 {
+  size_type nproc = communicator().size();
+  _drop_ext_nnz = is_distributed<M>::value && opt.force_seq;
+#ifndef TO_CLEAN
+  if (_drop_ext_nnz) {
+    warning_macro ("in test: _drop_ext_nnz=true and nproc=" << nproc);
+  }
+#endif // TO_CLEAN
   if (a.dis_nrow() <= 1) {
     // skip 0 or 1 dis_nrow, where mumps core dump
     if (a.nrow() == 1) {
@@ -220,15 +249,38 @@ solver_mumps_rep<T,M>::solver_mumps_rep (const csr<T,M>& a, const solver_option_
   // ----------------------------------
   //  step 0 : init a mumps instance
   // ----------------------------------
-  bool use_symmetry = a.is_symmetric();
+  bool use_symmetry          = a.is_symmetric();
+  bool use_definite_positive = a.is_definite_positive();
   _mumps_par.par          = 1; // use parallel
 #ifdef _RHEOLEF_HAVE_MPI
   _mumps_par.comm_fortran = MPI_Comm_c2f (a.row_ownership().comm());
 #endif // _RHEOLEF_HAVE_MPI
-  if (use_symmetry) {
-    _mumps_par.sym        = 2; // sym but can have either < 0 or > 0 eigenvalues
+  if (use_symmetry && !use_definite_positive) {
+    // sym invertible but could be undef: could have either < 0 or > 0 eigenvalues
+    // => leads to similar cpu with s.d.p. but without the asumption of positive or negative :
+    _mumps_par.sym        = 2;
+  } else if (use_symmetry && use_definite_positive) {
+#if 0
+    // sym def pos
+    // a) standard approach
+    // 	  Note: could fail in scalapack when sym def negative
+    _mumps_par.sym        = 1;
+    // b) second approach: from AmeButGue-2011-mumps, page 15:
+    // Another approach to suppress numerical pivoting which works with ScaLAPACK
+    // for both positive definite and negative definite matrices consists in setting SYM=2 and
+    // CNTL(1)=0.0D0 (recommended strategy).
+    _mumps_par.sym        = 2;
+    _mumps_par.cntl[1-1]  = 0.0;
+#endif // 0
+    // c) third approach: do not use the knowledge of positive or negative, since
+    //    this knowledge is contained in bilinear forms defined by the user
+    //    use only the general symetry:
+    //    tests show CPU is very similar than two previous approaches
+    //    and the useer do not need know about positive_definite or negative_definite
+    _mumps_par.sym        = 2;
   } else {
-    _mumps_par.sym        = 0; // is symmetric: 0=no, 1=sdp, 2=general sym
+    // unsym invertible
+    _mumps_par.sym        = 0;
   }
   _mumps_par.job = -1;
   dmumps_c(&_mumps_par);
@@ -248,13 +300,36 @@ solver_mumps_rep<T,M>::solver_mumps_rep (const csr<T,M>& a, const solver_option_
     _mumps_par.icntl[3-1] = -1;
     _mumps_par.icntl[4-1] =  0; 
   }
+  if (opt.compute_determinant) {
+    _mumps_par.icntl[33-1] = 1; 
+  }
   _mumps_par.icntl[5-1] = 0; // matrix is in assembled format
-  _mumps_par.icntl[7-1] = 7; // ordering: 7=auto; 3==scotch ; 4=pord, distributed with mumps
+#if defined(_RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH) || defined(_RHEOLEF_HAVE_MUMPS_WITH_SCOTCH)
+  _mumps_par.icntl[7-1] = 3; // sequential ordering: 3==scotch
+#elif defined(_RHEOLEF_HAVE_MUMPS_WITH_PARMETIS) || defined(_RHEOLEF_HAVE_MUMPS_WITH_METIS)
+  _mumps_par.icntl[7-1] = 5; // sequential ordering: 5==metis
+#else // _RHEOLEF_HAVE_MUMPS_WITH_PARMETIS
+  _mumps_par.icntl[7-1] = 7; // ordering: 7=auto
+#endif // _RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH
   _mumps_par.icntl[14-1] = 40; // default 20 seems to be insufficient in some cases
   _mumps_par.icntl[29-1] = 0; // 0: auto; 1: ptscotch; 2: parmetis
   _mumps_par.icntl[22-1] = 0; // 0: in-core ; 1: out-of-core TODO
   if (is_distributed<M>::value) {
     _mumps_par.icntl[18-1] = 3; // distributed
+
+#if defined(_RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH) || defined(_RHEOLEF_HAVE_MUMPS_WITH_PARMETIS)
+    _mumps_par.icntl[28-1] = 2; // parallel ordering
+#elif defined(_RHEOLEF_HAVE_MUMPS_WITH_SCOTCH) || defined(_RHEOLEF_HAVE_MUMPS_WITH_METIS)
+    _mumps_par.icntl[28-1] = 1; // sequential ordering
+#endif // _RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH || _RHEOLEF_HAVE_MUMPS_WITH_PARMETIS
+
+#ifdef _RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH
+    _mumps_par.icntl[29-1] = 1; // ptscotch=1
+#elif _RHEOLEF_HAVE_MUMPS_WITH_PARMETIS
+    _mumps_par.icntl[29-1] = 2; // parmetis=2
+#else
+    _mumps_par.icntl[29-1] = 0; // automatic choice
+#endif // _RHEOLEF_HAVE_MUMPS_WITH_PTSCOTCH
   } else {
     _mumps_par.icntl[18-1] = 0; // sequential
   }
@@ -262,23 +337,27 @@ solver_mumps_rep<T,M>::solver_mumps_rep (const csr<T,M>& a, const solver_option_
   _mumps_par.n = a.dis_nrow();
   size_type dia_nnz = ((use_symmetry) ? nnz_dia_upper(a) : a.nnz());
   size_type ext_nnz = ((use_symmetry) ? nnz_ext_upper(a) : a.ext_nnz());
-  size_type nnz = dia_nnz + ext_nnz;
-  _row.resize (nnz);
-  _col.resize (nnz);
-  csr2mumps_struct (a, _row, _col, use_symmetry);
+  size_type dis_nnz = dia_nnz + (_drop_ext_nnz ? 0 : ext_nnz);
   if (is_distributed<M>::value) {
-    _mumps_par.nz_loc  = nnz;
+#ifdef _RHEOLEF_HAVE_MPI
+    _row.resize (dis_nnz);
+    _col.resize (dis_nnz);
+    csr2mumps_struct_dis (a, _row, _col, use_symmetry, _drop_ext_nnz);
+    _mumps_par.nz_loc  = dis_nnz;
     _mumps_par.irn_loc = _row.begin().operator->();
     _mumps_par.jcn_loc = _col.begin().operator->();
+#endif // _RHEOLEF_HAVE_MPI
   } else { // sequential
-    _mumps_par.nz  = nnz;
-    _mumps_par.irn = _row.begin().operator->();
-    _mumps_par.jcn = _col.begin().operator->();
+    _row.resize (dia_nnz);
+    _col.resize (dia_nnz);
+    csr2mumps_struct_seq (a, _row, _col, use_symmetry);
+    _mumps_par.nz      = dia_nnz;
+    _mumps_par.irn     = _row.begin().operator->();
+    _mumps_par.jcn     = _col.begin().operator->();
   }
   _mumps_par.job = 1;
   dmumps_c(&_mumps_par);
-  check_macro (_mumps_par.infog[1-1] == 0, "mumps: an error has occured");
-
+  check_macro (_mumps_par.infog[1-1] == 0, "mumps: error infog(1)="<<_mumps_par.infog[1-1]<<" has occured (see mumps/infog(1) documentation; infog(2)="<<_mumps_par.infog[2-1]<<")");
   update_values (a);
 }
 template<class T, class M>
@@ -296,23 +375,35 @@ solver_mumps_rep<T,M>::update_values (const csr<T,M>& a)
   //  step 2 : numeric factorization
   // ----------------------------------
   bool use_symmetry = a.is_symmetric();
-  size_type nnz = a.nnz() + a.ext_nnz();
-  vector<T> val (nnz);
-  csr2mumps_values (a, val, use_symmetry);
+#ifdef TO_CLEAN
+  size_type dia_nnz = a.nnz();
+  size_type dis_nnz = dia_nnz + a.ext_nnz();
+#endif // TO_CLEAN
+  size_type dia_nnz = ((use_symmetry) ? nnz_dia_upper(a) : a.nnz());
+  size_type ext_nnz = ((use_symmetry) ? nnz_ext_upper(a) : a.ext_nnz());
+  size_type dis_nnz = dia_nnz + (_drop_ext_nnz ? 0 : ext_nnz);
+  vector<T> val;
   if (is_distributed<M>::value) {
+#ifdef _RHEOLEF_HAVE_MPI
+    val.resize (dis_nnz);
+    csr2mumps_values_dis (a, val, use_symmetry, _drop_ext_nnz);
     _mumps_par.a_loc   = val.begin().operator->();
+#endif // _RHEOLEF_HAVE_MPI
   } else { // sequential
+    val.resize (dia_nnz);
+    csr2mumps_values_seq (a, val, use_symmetry);
     _mumps_par.a       = val.begin().operator->();
   }
   _mumps_par.job = 2;
   bool finished = false;
   while (!finished && _mumps_par.icntl[14-1] <= 2000) {
     dmumps_c(&_mumps_par);
-    if (_mumps_par.infog[1-1] == -9 && _mumps_par.infog[2-1] >= 0) {
+    if ((_mumps_par.infog[1-1] == -8 || _mumps_par.infog[1-1] == -9) && 
+         _mumps_par.infog[2-1] >= 0) {
       // not enought working space:
       // default 20 seems to be insufficient in some cases (periodic,surfacic,p-laplacian)
       _mumps_par.icntl[14-1] += 10;
-      trace_macro ("numeric factorization: increase working space of "<<_mumps_par.icntl[14-1]<< "% and retry...");
+      dis_warning_macro ("numeric factorization: increase working space of "<<_mumps_par.icntl[14-1]<< "% and retry...");
     } else {
       finished = true;
     }
@@ -322,6 +413,13 @@ solver_mumps_rep<T,M>::update_values (const csr<T,M>& a)
 	                      << ", infog(2)=" <<_mumps_par.infog[2-1]
                               << ", icntl(14)="<<_mumps_par.icntl[14-1]
                               << ", icntl(23)="<<_mumps_par.icntl[23-1]);
+  if (_mumps_par.icntl[33-1] != 0) {
+    // get determinant from infos:
+    _det.mantissa = _mumps_par.rinfog[12-1];
+    //    imag part _mumps_par.rinfog[13-1] is zero as matrix is real here
+    _det.exponant = _mumps_par.infog[34-1];
+    _det.base     = 2;
+  }
 }
 template<class T, class M>
 vec<T,M>
