@@ -38,17 +38,18 @@ solver_wrapper_rep<T,M>::solver_wrapper_rep (const csr<T,M>& a, const solver_opt
   _sa(),
   _a()
 {
-#if !defined(_RHEOLEF_HAVE_TRILINOS) && !defined(_RHEOLEF_HAVE_PASTIX)
+#if defined(_RHEOLEF_HAVE_QD) || defined(_RHEOLEF_HAVE_FLOAT128)
+  // only ldlt available yet, for sym matrix
+  if (!_is_sym) { _is_direct = false; }
+#elif !defined(_RHEOLEF_HAVE_TRILINOS) && !defined(_RHEOLEF_HAVE_PASTIX)
   // no builtin ilu0 available yet:
   if (!_is_sym) { _is_direct = true; }
 #endif
-
   if (_is_direct) {
     if (_is_sym) { _sa = ldlt (a,opt); }
     else         { _sa = lu   (a,opt); }
   } else {
-    if (_is_sym) { _sa = ic0  (a,opt); }
-    else         { _sa = ilu0 (a,opt); }
+    _sa = eye_basic<T,M>();
     _a = a;
   }
 }
@@ -65,9 +66,9 @@ solver_wrapper_rep<T,M>::solve (const vec<T,M>& b) const
 {
   if (_is_direct) { return _sa.solve (b); }
 
-  T tol = std::numeric_limits<T>::epsilon();
-  T tol_reached = tol;
-  size_type max_iter = 1000; // TODO: use _opt to get tol & max_iter
+  size_type max_iter = option().max_iter;
+  T tol              = option().tol;
+  T tol_reached      = tol;
   vec<T,M> x (b.ownership(), 0);
   if (_is_sym) {
     int status = pcg (_a, x, b, _sa, max_iter, tol_reached, &derr);
@@ -76,14 +77,12 @@ solver_wrapper_rep<T,M>::solve (const vec<T,M>& b) const
     boost::numeric::ublas::matrix<T> h(m+1,m+1);
     //boost::numeric::ublas::vector<T> dummy(m);
     vec<T,sequential> dummy(m,0.0);
-    trace_macro ("pgmres...");
-    int status = pgmres (_a, x, b, _sa, h, dummy, m, max_iter, tol_reached);
-    trace_macro ("pgmres done");
+    int status = pgmres (_a, x, b, _sa, h, dummy, m, max_iter, tol_reached, &derr);
   }
   check_macro (tol_reached < sqrt(tol),
 	"solver: precision "<<tol<<" not reached: get "<<tol_reached
 	<< " after " << max_iter << " iterations");
-  if (tol_reached < tol) warning_macro (
+  if (tol_reached > tol) warning_macro (
 	"solver: precision "<<tol<<" not reached: get "<<tol_reached
 	<< " after " << max_iter << " iterations");
   return x;
@@ -98,6 +97,13 @@ solver_wrapper_rep<T,M>::trans_solve (const vec<T,M>& b) const
   // TODO: wrap a as a*x return a.trans_mult(x) and the same with _sa.solve(b)
   return vec<T,M>();
 }
+template <class T, class M>
+typename solver_wrapper_rep<T,M>::determinant_type
+solver_wrapper_rep<T,M>::det() const
+{
+  check_macro (_is_direct, "undefined determinant computation for iterative solver (HINT: use direct method)");
+  return _sa.det();
+}
 template<class T, class M>
 solver_wrapper_rep<T,M>::~solver_wrapper_rep()
 {
