@@ -23,18 +23,11 @@ using namespace rheolef;
 int main(int argc, char**argv)
 {
     environment rheolef(argc, argv);
-
     size_t n = 101;
-    array<Float> x;
-    if (x.comm().rank() == 0)
-        x.resize(n,n);
-    else
-        x.resize(n,0);
-
-    array<Float>::iterator iter = x.begin();
-    array<Float>::iterator last = x.end();
-    fill(iter, last, Float(0));
-
+    communicator comm;
+    distributor ownership (n, comm, comm.rank() == 0 ? n : 0);
+    array<Float> x (ownership);
+    fill(x.begin(), x.end(), Float(0));
     if (x.comm().rank() == x.comm().size()-1) {
         for (size_t i = 0; i < x.dis_size(); i++) {
 	    x.dis_entry (i) = 1000+i+0.5;
