@@ -22,15 +22,13 @@
 # include "rheolef/array.h"
 # include "rheolef/index_set.h"
 using namespace rheolef;
-using namespace std;
-
 int main(int argc, char**argv) {
-    environment distributed(argc, argv);
+    environment rheolef(argc, argv);
     communicator comm;
     size_t dis_n;
-    dcin >> dis_n;
+    din >> dis_n;
     array<index_set> x (dis_n);
-    x.get_values (dcin);
-    dcout << x.dis_size() << endl
-          << x            << endl;
+    x.get_values (din);
+    dout << x.dis_size() << std::endl
+         << x            << std::endl;
 }
