@@ -28,7 +28,7 @@ status=0
 if test "$MPIRUN" = ""; then
   echo "      (skipped)"
 else
-  run "awk -f array_of_idxset_gen.awk > tmp.valid" 
+  run "awk -f $SRCDIR/array_of_idxset_gen.awk > tmp.valid" 
   loop_mpirun "./array_of_idxset_io_tst < tmp.valid 2> /dev/null | diff -w -B tmp.valid - >/dev/null" 
   status=$?
   run "rm -f tmp.valid" 
