@@ -31,7 +31,7 @@ status=0
 if test "$MPIRUN" = "" || test "$NPROC_MAX" -lt 3; then
   echo "      skipped (requiers np >= 3)"
 else
-  run "awk -v n=6 -v m=20 -f array_of_idxset_gen.awk > tmp.input"
+  run "awk -v n=6 -v m=20 -f $SRCDIR/array_of_idxset_gen.awk > tmp.input"
   run "rm -f array_of_idxset_scatter_tst-[012]"
   run "$MPIRUN -np 3 ./array_of_idxset_scatter_tst < tmp.input 2> /dev/null && cat array_of_idxset_scatter_tst-[012] | diff -w -B $SRCDIR/array_of_idxset_scatter_tst.valid - >/dev/null" 
   status=$?
