@@ -30,7 +30,7 @@ NPROC_MAX=${NPROC_MAX-"8"}
 if test "$MPIRUN" = "" || test "$NPROC_MAX" -lt 3; then
   echo "      skipped (requiers np >= 3)"
 else
-  run "$MPIRUN -np 3 ./array_of_pairset_tst 2> /dev/null | diff -w -B array_of_pairset_tst.valid - >/dev/null" 
+  run "$MPIRUN -np 3 ./array_of_pairset_tst 2> /dev/null | diff -w -B $SRCDIR/array_of_pairset_tst.valid - >/dev/null" 
   status=$?
   exit $status
 fi
