@@ -24,8 +24,10 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"3"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
-if test "$MPIRUN" = ""; then
-  echo "      (skipped)"
+if test "$MPIRUN" = "" || test "$NPROC_MAX" -lt 3; then
+  echo "      skipped (requiers np >= 3)"
 else
   run "$MPIRUN -np 3 ./array_set_add_tst 2> /dev/null | diff -w -B $SRCDIR/array_set_add_tst.valid - >/dev/null" 
+  status=$?
+  exit $sattus
 fi
