@@ -24,7 +24,6 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"7"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
-#echo "      (skipped for foehn)" 	# TODO
-#exit 0
+if test "$QD_EXT" = ".float128"; then QD_EXT=""; fi
 
-loop_mpirun "./array_set_tst 2>/dev/null | diff ${SRCDIR}/vec_tst${DD_EXT}.out.valid - >/dev/null"
+loop_mpirun "./array_set_tst 2>/dev/null | diff ${SRCDIR}/array_set_tst${QD_EXT}.out.valid - >/dev/null"
