@@ -30,7 +30,7 @@ NPROC_MAX=${NPROC_MAX-"8"}
 if test "$MPIRUN" = "" || test "$NPROC_MAX" -lt 2; then
   echo "      skipped (requiers np >= 2)"
 else
-  run "$MPIRUN -np 2 ./asr_assembly_tst 2> /dev/null | diff -w -B asr_assembly_tst.mtx.valid - >/dev/null" 
+  run "$MPIRUN -np 2 ./asr_assembly_tst 2> /dev/null | diff -w -B $SRCDIR/asr_assembly_tst.mtx.valid - >/dev/null" 
   status=$?
   exit $status
 fi
