@@ -27,6 +27,7 @@ int main() { return 0; }
 #include "rheolef/dis_macros.h"
 #include <boost/mpi.hpp>
 #include "rheolef/index_set.h"
+#include "rheolef/rheostream.h" // itos
 using namespace std;
 using namespace rheolef;
 template<typename ForwardIterator>
@@ -171,8 +172,10 @@ int main(int argc, char** argv) {
   // wait on sends:
   mpi::wait_all(send_reqs.begin(), send_reqs.end());
   // print:
-  std::cout << "proc "<<world.rank()<<" : msg[0]=" << msg[0] << std::endl;
-  std::cout << "proc "<<world.rank()<<" : msg[1]=" << msg[1] << std::endl;
+  std::string filename = "mpi_boost_nonblocking_array_of_idxset2_tst-" + itos(world.rank());
+  std::ofstream out (filename.c_str());
+  out << "proc "<<world.rank()<<" : msg[0]=" << msg[0] << std::endl
+      << "proc "<<world.rank()<<" : msg[1]=" << msg[1] << std::endl;
   return 0;
 }
 #endif // _RHEOLEF_HAVE_MPI
