@@ -24,6 +24,8 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"7"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
-run "./pairset_tst 2> /dev/null | diff ${SRCDIR}/pairset_tst.out.valid - >/dev/null" 
+if test "$QD_EXT" = ".float128"; then QD_EXT=""; fi
+
+run "./pairset_tst 2> /dev/null | diff ${SRCDIR}/pairset_tst${QD_EXT}.out.valid - >/dev/null" 
 status=$?
 exit $status
