@@ -28,7 +28,7 @@ status=0
 
 for algo in direct iterative; do
 
-    loop_mpirun "./solver_tst -$algo -sym < $SRCDIR/solver_tst.mm 2>/dev/null >/dev/null"
+    loop_mpirun "./solver_tst -$algo -sym < $SRCDIR/solver_tst.mtx 2>/dev/null >/dev/null"
     if test $? -ne 0; then status=1; fi
   
     if false; then 
