@@ -34,7 +34,7 @@ int main (int argc, char **argv) {
     x[i] = first_i + i;
   }
 
-  vec<Float> y = {x, dis_n+3.};
+  vec<Float> y = {x, Float(dis_n+3.)};
 
   dout << "y=" << endl << y;
 }
