@@ -23,15 +23,14 @@
 # include "rheolef/dis_accumulate.h"
 # include "rheolef/dis_inner_product.h"
 using namespace rheolef;
-using namespace std;
 int main(int argc, char**argv)
 {
-    environment distributed(argc, argv);
+    environment rheolef(argc, argv);
     array<Float> x(100, Float(2.0)/Float(3.0));
     Float sum_x    = dis_accumulate    (x.begin(),x.size(),x.comm()); 
     Float norme2_x = dis_inner_product (x.begin(),x.begin(),x.size(),x.comm()); 
-    dcout << setprecision(10) 
-          << "dot(x,1) = " << sum_x    << endl
-          << "dot(x,x) = " << norme2_x << endl;
+    dout << std::setprecision(10) 
+          << "dot(x,1) = " << sum_x    << std::endl
+          << "dot(x,x) = " << norme2_x << std::endl;
     return 0;
 }
