@@ -24,9 +24,11 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"3"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
+if test "$QD_EXT" = ".float128"; then QD_EXT=""; fi
+
 status=0
 
-loop_mpirun "./vec_range_tst 2>/dev/null | diff -Bw $SRCDIR/vec_range_tst.valid - >/dev/null"
+loop_mpirun "./vec_range_tst 2>/dev/null | diff -Bw $SRCDIR/vec_range_tst${QD_EXT}.valid - >/dev/null"
 if test $? -ne 0; then status=1; fi
 
 exit $status
