@@ -29,9 +29,7 @@ DESCRIPTION:
  The class implement a reference counting wrapper for
  the STL @code{vector<T>} container class, with shallow copies.
  See also:
- @quotation
-  The Standard Template Library, by Alexander Stephanov and Meng Lee.
- @end quotation
+ @emph{The standard template library}, by Alexander Stephanov and Meng Lee.
 
  @noindent
  This class provides the full @code{vector<T>}
