@@ -28,7 +28,7 @@
 #endif
 
 #include <boost/iostreams/filter/gzip.hpp>
-
+#include <boost/iostreams/device/file.hpp>
 
 #ifdef _RHEOLEF_HAVE_UNISTD_H
 #include<unistd.h>  // readink()
@@ -47,7 +47,7 @@
 
 namespace rheolef {
 using namespace std;
-namespace io = boost::iostreams;
+namespace ios = boost::iostreams;
 
 std::string
 get_tmpdir() {
@@ -64,113 +64,102 @@ itos (string::size_type i)
 string
 ftos (const Float& x)
 {
-    char buffer [100];
-    sprintf (buffer, "%g", double(x));
-    return buffer;
+  std::ostringstream out;
+  out << x;
+  return out.str ();
 }
-#ifdef TO_CLEAN
-// read on a pipe with un-compression
-static
-string
-make_i_pipe_name (const string& name, const string& suffix)
-{
-    string full_name = get_full_name_from_rheo_path (name, suffix);
-    if (full_name.length() == 0) {
-        return string();
-    }
-    bool verbose = iorheo::getverbose(clog);
-    if (verbose) clog << "! load \"" << full_name << "\"\n";
-
-    if (!has_suffix (full_name, "gz")) {
-	
-	// classical ifstream
-	return "cat " + full_name;
-    }
-    return "gzip -dc < " + full_name;
-}
-// write on a pipe with compression
-static
-string
-make_o_pipe_name (const string& name, const string& suffix)
-{
-    string full_name;
-
-    // assume suffix.gz
-    if (suffix=="")
-        full_name = delete_suffix(name, "gz") + ".gz";
-    else
-        full_name = delete_suffix(delete_suffix(name, "gz"), suffix) 
-		       + "." + suffix + ".gz";
-    
-    bool verbose = iorheo::getverbose(clog);
-    if (verbose) clog << "! file \"" << full_name << "\" created.\n";
-
-    return "gzip -c9 > " + full_name;
-}
-#endif // TO_CLEAN
 // -----------------------------------------------------------------
 // output
 // -----------------------------------------------------------------
-void
-orheostream::open (const string& name, const string& suffix, bool use_gzip)
-{
-    using namespace io;
-    _use_gzip = use_gzip;
-
-    // append the '.gz' suffix:
-    if (suffix == "") {
-        _full_name = delete_suffix(name, "gz");
-    } else {
-        _full_name = delete_suffix(delete_suffix(name, "gz"), suffix) 
-		       + "." + suffix;
-    }
-    if (_use_gzip) {
-      _full_name = _full_name + ".gz";
-    }
-    // open the file.gz:
-    _ofs.open (_full_name.c_str(), ios_base::out | ios_base::binary);
-    bool verbose = iorheo::getverbose(clog);
-    if (verbose) clog << "! file \"" << _full_name << "\" created.\n";
-
-    // create the output pipe with the optional gzip filter:
-    if (_use_gzip) {
-      filtering_stream<output>::push (gzip_compressor());    
-    }
-    filtering_stream<output>::push (_ofs);    
-}
-orheostream::orheostream (const string& name, const string& suffix, bool use_gzip)
- : io::filtering_stream<io::output>(),
-   _use_gzip(false),
+orheostream::orheostream (const string& name, const string& suffix, io::mode_type mode)
+ : ios::filtering_stream<ios::output>(),
+   _mode(mode),
    _full_name()
 {
-    open (name, suffix, use_gzip);
-}
-void
-orheostream::close ()
-{
-    using namespace io;
-    while (! filtering_stream<io::output>::empty()) {
-             filtering_stream<io::output>::pop();
-    }
-    _ofs.close();
+    open (name, suffix, mode);
 }
 orheostream::~orheostream ()
 {
     orheostream::close();
 }
 void
-orheostream::flush ()
+orheostream::open (const string& name, const string& suffix, io::mode_type mode)
 {
-    using namespace io;
-    // boost::iostreams are not yet flushable when using gzip:
-    // thus close and re-open... it is a clostly procedure
-    close();
-    // then re-open: in append mode
-    _ofs.open (_full_name.c_str(), ios_base::out | ios_base::binary | ios_base::app);
-    if (_use_gzip) {
+    _mode = mode;
+    // append the '.gz' suffix:
+    if (suffix == "") {
+        _full_name = delete_suffix(name, "gz");
+    } else {
+        _full_name = delete_suffix(delete_suffix(name, "gz"), suffix) 
+		   + "." + suffix;
+    }
+    if (!(mode & io::nogz)) {
+      _full_name = _full_name + ".gz";
+    }
+    _open_internal (_mode);
+    bool verbose = iorheo::getverbose(clog);
+    if (verbose) {
+      std::string action = (mode & io::app) ? "appended" : "created";
+      clog << "! file \"" << _full_name << "\" " << action << endl;
+    }
+}
+void
+orheostream::_open_internal (io::mode_type mode)
+{
+    using namespace ios;
+    // create the output pipe with the optional gzip filter:
+    if (!(mode & io::nogz)) {
       filtering_stream<output>::push (gzip_compressor());    
     }
-    filtering_stream<output>::push (_ofs);    
+    // open the file.gz:
+    std::ios_base::openmode om = std::ios_base::out | std::ios_base::binary;
+    if (mode & io::app) { om |= std::ios_base::app; }
+    file_sink ofs (_full_name.c_str(), om);
+    filtering_stream<output>::push (ofs);    
+}
+void
+orheostream::_close_internal ()
+{
+    using namespace ios;
+    if (filtering_stream<ios::output>::empty()) {
+      return;
+    }
+#define _RHEOLEF_HAVE_BOOST_IOSTREAMS_GZIP_EMPTY_FILE_BUG
+#ifdef  _RHEOLEF_HAVE_BOOST_IOSTREAMS_GZIP_EMPTY_FILE_BUG
+    if (! (_mode & io::nogz)) {
+      *this << endl;
+    }
+#endif //  _RHEOLEF_HAVE_BOOST_IOSTREAMS_GZIP_EMPTY_FILE_BUG
+#ifdef  _RHEOLEF_HAVE_BOOST_IOSTREAMS_GZIP_EMPTY_FILE_BUG_OLD
+    if (! (_mode & io::nogz)) {
+      // bug with empty gziped boost::io: workarround by writting an empty thing !
+      // https://svn.boost.org/trac/boost/ticket/5237
+      // the previous fix is buggy: requires a non-empty thing: a newline that does not change rheolef semantic
+      trace_macro ("_close_internal: _full_name="<<_full_name<<": gziped => add a carriage return before closing...");
+      static char dummy = '\n'; // in rheolef, does not change the file format
+      static size_t length = 1; // instead of zero
+      filtering_stream<ios::output>::component<gzip_compressor>(0)->write(*filtering_stream<ios::output>::component<file_sink>(1), &dummy, length);
+    }
+#endif //  _RHEOLEF_HAVE_BOOST_IOSTREAMS_GZIP_EMPTY_FILE_BUG_OLD
+    while (! filtering_stream<ios::output>::empty()) {
+             filtering_stream<ios::output>::pop();
+    }
+}
+void
+orheostream::close ()
+{
+    _close_internal();
+    _full_name = "";
+    _mode = io::out;
+}
+void
+orheostream::flush ()
+{
+    // boost::iostreams are not yet flushable when using gzip:
+    // thus close and re-open... it is a clostly procedure
+    _close_internal();
+    // then reopen: in append mode
+    _open_internal (io::mode_type(_mode | io::app));
 }
 // -----------------------------------------------------------------
 // input
@@ -178,7 +167,7 @@ orheostream::flush ()
 void
 irheostream::open (const string& name, const string& suffix)
 {
-    using namespace io;
+    using namespace ios;
 
     // get full file path with the optional '.gz' suffix:
     string full_name = get_full_name_from_rheo_path (name, suffix);
@@ -208,14 +197,14 @@ irheostream::open (const string& name, const string& suffix)
 void
 irheostream::close ()
 {
-    using namespace io;
-    while (! filtering_stream<io::input>::empty()) {
-             filtering_stream<io::input>::pop();
+    using namespace ios;
+    while (! filtering_stream<ios::input>::empty()) {
+             filtering_stream<ios::input>::pop();
     }
     _ifs.close();
 }
 irheostream::irheostream (const string& name, const string& suffix)
- : io::filtering_stream<io::input>()
+ : ios::filtering_stream<ios::input>()
 {
     open (name, suffix);
 }
@@ -415,7 +404,7 @@ have_name_in_dir (const string& dir, const string& name, string& full_path)
     bool unzip_status = file_exists (unzip_full_name);
 
     if (unzip_status && zip_status) {
-	    warning_macro ("both compressed and uncompressed files exists:");
+            warning_macro ("both compressed and uncompressed files exists:");
 	    warning_macro ("    \"" <<   zip_full_name << "\"");
 	    warning_macro ("and \"" << unzip_full_name << "\"");
 	    error_macro   ("unrecoverable ambiguous situation (HINT: rename one of these files)");
