@@ -29,8 +29,12 @@ for file in in-animal-s2; do
   run "rm -f toto.hb toto.hb.gz"
   run "gzip -dc < ${TOP_SRCDIR}/skit/tst/$file.hb.gz | ./orheostream_tst toto hb 2>/dev/null"
   if test $? -ne 0; then status=1; fi
-  run "/bin/sh $TOP_SRCDIR/config/zdiff ${TOP_SRCDIR}/skit/tst/$file.hb.gz ./toto.hb.gz"
+  run "/bin/sh $TOP_SRCDIR/config/zdiff -B ${TOP_SRCDIR}/skit/tst/$file.hb.gz ./toto.hb.gz"
   run "rm -f toto.hb toto.hb.gz"
   if test $? -ne 0; then status=1; fi
+  run "rm -f toto.txt toto.txt.gz"
+  run "echo abc | ./orheostream_tst toto txt 2>/dev/null && echo def | ./orheostream_tst toto txt app 2>/dev/null && test \`zcat toto.txt.gz | grep -v '^$' | wc -l\` -eq 2"
+  if test $? -ne 0; then status=1; fi
+  run "rm -f toto.txt toto.txt.gz"
 done
 exit $status
